/*
PNG Texture
Created: 2004-06-22
Author : Sylvain Lefebvre
*/
//---------------------------------------------------------------------------
#include "CTexturePNG.h"
#include "CLibTextureException.h"
//---------------------------------------------------------------------------
#include <iostream>
#include <stdio.h>
#include <string.h>
//---------------------------------------------------------------------------
using namespace std;
//---------------------------------------------------------------------------
extern "C" {
#include "png.h"
}
//---------------------------------------------------------------------------
CTexturePNG::CTexturePNG(const char *n) : CTexture()
{
  strcpy(m_szName,n);
  CTexturePNG::load();
}
//---------------------------------------------------------------------------
void CTexturePNG::load()
{
  FILE *file=fopen(m_szName,"rb");

  if (file == NULL)
    throw CLibTextureException("Unable to load %s",m_szName);

  png_structp png_ptr =
    png_create_read_struct(PNG_LIBPNG_VER_STRING,
    0, // (png_voidp)user_error_ptr
    0, // user_error_fn
    0  // user_warning_fn
    );

  png_infop info_ptr = png_create_info_struct(png_ptr);
  png_infop end_info = png_create_info_struct(png_ptr);
  png_init_io(png_ptr, file);

  png_read_info(png_ptr, info_ptr);
  m_iWidth=png_get_image_width(png_ptr,info_ptr);
  m_iHeight=png_get_image_height(png_ptr,info_ptr);
  int ncomp=png_get_channels(png_ptr,info_ptr);
  int bit_depth=png_get_bit_depth(png_ptr,info_ptr);
  int color_type= png_get_color_type(png_ptr,info_ptr);
  if (m_iWidth <= 0 || m_iHeight <=0)
    throw CLibTextureException("CTexturePNG: corrupted PNG file (%s)",m_szName);
  if (ncomp < 3 || ncomp > 4)
    throw CLibTextureException("CTexturePNG: unsupported number of components (%d components in %s)",ncomp,m_szName);
  m_bAlpha=(ncomp == 4);
  if (bit_depth !=8 )
    throw CLibTextureException("CTexturePNG: unsupported bit depth (%d bit in %s)",ncomp,m_szName);
  if (color_type == PNG_COLOR_TYPE_PALETTE)
    throw CLibTextureException("CTexturePNG: palette not supported (%s)",m_szName);
  //    if (bit_depth == 16)
  //      png_set_strip_16(png_ptr);
  //    if (bit_depth < 8)
  //      png_set_packing(png_ptr);
  if (color_type == PNG_COLOR_TYPE_GRAY)
    png_set_gray_to_rgb(png_ptr); // grayscale => RGB

  std::cerr << "Loading PNG image " << m_szName << " " << m_iWidth << "x" << m_iHeight << std::endl;
  m_Data=new unsigned char[m_iWidth*m_iHeight*(isAlpha()?4:3)];
  png_bytep *rowptrs=new png_bytep[m_iHeight];
  for (int j=0;j<m_iHeight;j++)
  {
    rowptrs[j]=(png_bytep)&(m_Data[(j*m_iWidth)*(isAlpha()?4:3)]);
  }
  png_read_image(png_ptr, rowptrs);
  delete [](rowptrs);
  png_destroy_read_struct(&png_ptr, &info_ptr, &end_info);

  fclose(file);
}

//---------------------------------------------------------------------------

void CTexturePNG::save()
{
  FILE *file=fopen(m_szName,"wb");

  if (file == NULL)
    throw CLibTextureException("Unable to save %s",m_szName);

  png_structp png_ptr = png_create_write_struct(PNG_LIBPNG_VER_STRING,0,0,0);
  png_infop info_ptr  = png_create_info_struct(png_ptr);
  png_init_io(png_ptr, file);

  // turn off compression or set another filter
  //png_set_filter(png_ptr, 0, PNG_FILTER_NONE);
  //png_set_compression_level(png_ptr, 0);

  // ugly hack for windows preview compatibility
  unsigned char *data=m_Data;
#ifdef WIN32
  if (isAlpha())
  {
    cerr << endl << "WARNING: png alpha channel disabled for previewing pictures in Windows" << endl << endl;
    data=new unsigned char[m_iWidth*m_iHeight*3];
    for (int i=0;i<m_iWidth*m_iHeight;i++)
      for (int c=0;c<3;c++)
        data[i*3+c]=m_Data[i*4+c];
  }
#endif

  png_set_IHDR(png_ptr, info_ptr, m_iWidth, m_iHeight,
    8,
#ifdef WIN32
    PNG_COLOR_TYPE_RGB, 
#else
    (isAlpha() ? PNG_COLOR_TYPE_RGB_ALPHA : PNG_COLOR_TYPE_RGB),
#endif
    PNG_INTERLACE_NONE,
    PNG_COMPRESSION_TYPE_BASE,
    PNG_FILTER_TYPE_DEFAULT);

  png_write_info(png_ptr, info_ptr);

  cerr << "Saving PNG image " << m_szName << " " << m_iWidth << "x" << m_iHeight << " ... ";

  for (int j=0;j<m_iHeight;j++)
  {
#ifdef WIN32
    png_bytep rowptr=(png_bytep)&(data[(j*m_iWidth)*3]);
#else
    png_bytep rowptr=(png_bytep)&(data[(j*m_iWidth)*(isAlpha()?4:3)]);
#endif
    png_write_row(png_ptr, rowptr);
  }

  if (data != m_Data)
    delete [](data);

  png_write_end(png_ptr, NULL);
  png_destroy_write_struct(&png_ptr, &info_ptr);

  fclose(file);
  cerr << "done." << endl;
}
