
#ifndef object_water_h
#define object_water_h

#include "scene/scene.h"
#include "common90/d3d-util.h"

#define WATER_DEPTH 20.0f
#define WATER_CAUSTICS_SIZE 128

class cEnvironment
{
public:
    cEnvironment();
   ~cEnvironment();

    HRESULT Initialize(float fSize);

    HRESULT OnCreateDevice(IDirect3DDevice9* pDevice);
    HRESULT OnResetDevice();
    HRESULT OnLostDevice();
    HRESULT OnDestroyDevice();

    HRESULT SetSurfaces(
        IDirect3DTexture9* pXNeg, IDirect3DTexture9* pXPos, 
        IDirect3DTexture9* pYNeg, IDirect3DTexture9* pYPos,
        IDirect3DTexture9* pZNeg, IDirect3DTexture9* pZPos); 
        
    HRESULT Draw();

protected:
    float m_fSize;

    IDirect3DDevice9* m_pDevice;
    IDirect3DTexture9* m_pSurf[6];
};

struct WATER_REFRACT
{
    // Vrefract = (V + refract * N) * norm
    float fRefract;
    float fRefractNorm; 
    DWORD dwDiffuse;
};


struct WATER_SURFACE
{
    float fHeight;
    float fVelocity;
};


//-----------------------------------------------------------------------------
// Name: cWater
// Desc: The water that is rendered
//-----------------------------------------------------------------------------
class cWater
{
    FLOAT m_fSize;
    FLOAT m_fDepth;
    FLOAT m_fScaleTex;
    FLOAT m_fAvgHeight;

    FLOAT m_fSphereHeight;
    FLOAT m_fSphereRadius2;

    UINT m_uIndices;
    UINT m_uVertices;

    WATER_SURFACE *m_pSurface;
    WATER_REFRACT *m_pRefract;

    IDirect3DDevice9       *m_pDevice;
    IDirect3DIndexBuffer9  *m_pibIndices;
    IDirect3DVertexBuffer9 *m_pvbVertices;
    IDirect3DVertexBuffer9 *m_pvbCaustics;

public:
    cWater();
   ~cWater();

    HRESULT Initialize(float fSize, float fDepth);

    HRESULT OnCreateDevice(IDirect3DDevice9 *pDevice);
    HRESULT OnResetDevice();
    HRESULT OnLostDevice();
    HRESULT OnDestroyDevice();

    HRESULT Drop();
    HRESULT Update(D3DXVECTOR3 &vecPos, D3DXVECTOR3 &vecLight, BOOL bCalcCaustics);

    HRESULT DrawCaustics();
    HRESULT DrawSurface();
};


class object_water : public scene  
{
public:

    object_water();
	
    virtual ~object_water();

	scene* create(char* opts = 0);
    virtual HRESULT init(IDirect3DDevice9* device);
	virtual HRESULT restore(IDirect3DDevice9* device);
	virtual HRESULT compose(IDirect3DDevice9* device, DWORD t);
    

    virtual HRESULT render(  IDirect3DDevice9* device,
                        IDirect3DTexture9* texture );

	
	//int handle(char* type, char* ev);

	HRESULT GetNextTechnique(INT nDir, BOOL bBypassValidate);

	  LPDIRECT3DDEVICE9 m_pd3dDevice;        // The D3D rendering device
    D3DCAPS9          m_d3dCaps;           // Caps for the device

	   cWater                 m_Water;
    cEnvironment           m_Environment;

    LPD3DXEFFECT           m_pEffect;
    UINT                   m_iTechnique;
    LPD3DXRENDERTOSURFACE  m_pRenderToSurface;

    D3DXVECTOR4            m_vecLight;
    D3DXCOLOR              m_colorLight;

    LPDIRECT3DTEXTURE9     m_pFloorTex;
    LPDIRECT3DTEXTURE9     m_pCausticTex;
    LPDIRECT3DSURFACE9     m_pCausticSurf;
    LPDIRECT3DTEXTURE9     m_pSkyTex[6];
    LPDIRECT3DCUBETEXTURE9 m_pSkyCubeTex;

    CD3DFont*              m_pFont;
    CD3DFont*              m_pFontSmall;

    BYTE        m_bKey[256];

    BOOL        m_bPause;
    BOOL        m_bDrawWater;
    BOOL        m_bDrawCaustics;
    BOOL        m_bDrawEnvironment;
    BOOL        m_bShowHelp;

    FLOAT       m_fSpeed;
    FLOAT       m_fAngularSpeed;
    FLOAT       m_fTime;
    FLOAT       m_fSecsPerFrame;

    FLOAT       m_bAutoAddDrop;
    FLOAT       m_fAddDropTimer;
    FLOAT       m_fAddDropCountdown;

    D3DXVECTOR3 m_vecVelocity;
    D3DXVECTOR3 m_vecAngularVelocity;

    D3DXMATRIXA16  m_matIdentity;
    D3DXMATRIXA16  m_matView;
    D3DXMATRIXA16  m_matPosition;
    D3DXMATRIXA16  m_matProjection;

	//int onetime;
};
#endif 
