//------------------------------------------------------------------------------
// File: PlaneScene.cpp
//
// Desc: DirectShow sample code - implementation of the plane_scene class
//
// Copyright (c) Microsoft Corporation.  All rights reserved.
//------------------------------------------------------------------------------


#include "plane-scene.h"
#include "scene/util.h"




#define D3DFVF_CUSTOMVERTEX ( D3DFVF_XYZ | D3DFVF_DIFFUSE | D3DFVF_TEX1 )

extern HINSTANCE hInst;
extern IDirect3D9* g_d3d; 
extern IDirect3DDevice9* g_d3dd;
extern HWND  g_hWnd;

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

plane_scene::plane_scene()
{
    //g_myapp = new CMyD3DApplication();
	//g_myapp->Create(hInst);

    m_vertices[0].position = CUSTOMVERTEX::Position(-1.0f,  1.0f, 0.0f); // top left
    m_vertices[1].position = CUSTOMVERTEX::Position(-1.0f, -1.0f, 0.0f); // bottom left
    m_vertices[2].position = CUSTOMVERTEX::Position( 1.0f,  1.0f, 0.0f); // top right
    m_vertices[3].position = CUSTOMVERTEX::Position( 1.0f, -1.0f, 0.0f); // bottom right

    // set up diffusion:
	/*
   m_vertices[0].color = 0xfffffff; // 0xffffffff;
    m_vertices[1].color = 0xfffffff; //0xff0000ff;
    m_vertices[2].color = 0xfffffff; //0xffffffff;
    m_vertices[3].color = 0xfffffff; // 0xff0000ff;
*/
	m_vertices[0].color =  0xffffffff;
    m_vertices[1].color = 0xff0000ff;
    m_vertices[2].color = 0xffffffff;
    m_vertices[3].color = 0xff0000ff;

    // set up texture coordinates
    m_vertices[0].tu = 0.0f; m_vertices[0].tv = 0.0f; // low left
    m_vertices[1].tu = 0.0f; m_vertices[1].tv = 1.0f; // high left
    m_vertices[2].tu = 1.0f; m_vertices[2].tv = 0.0f; // low right
    m_vertices[3].tu = 1.0f; m_vertices[3].tv = 1.0f; // high right
}

plane_scene::~plane_scene()
{

}

HRESULT 
plane_scene::init(IDirect3DDevice9* device)
{
    HRESULT hr = S_OK;



	
    m_vertexBuffer = NULL;


    hr = device->CreateVertexBuffer(sizeof(m_vertices),
		0,D3DFVF_CUSTOMVERTEX,D3DPOOL_DEFAULT,& m_vertexBuffer.p, NULL );



    m_time = GetTickCount();

    return hr;
}

HRESULT 
plane_scene::restore(IDirect3DDevice9* device)
{
    
	HRESULT hr = S_OK;
	//ASSERT(3!=3);
	//return hr;
	// get the difference in time
    DWORD dwCurrentTime;
    dwCurrentTime = GetTickCount();
    double difference = m_time - dwCurrentTime ;
    
    // figure out the rotation of the plane
    float x = (float) ( -cos(difference / 2000.0 ) ) ;
    float y = (float) ( cos(difference / 2000.0 ) ) ;
    float z = (float) ( sin(difference / 2000.0 ) ) ;

    // update the two rotating vertices with the new position
    m_vertices[0].position = CUSTOMVERTEX::Position(x,  y, z);   // top left
    m_vertices[3].position = CUSTOMVERTEX::Position(-x, -y, -z); // bottom right

    // Adjust the color so the blue is always on the bottom.
    // As the corner approaches the bottom, get rid of all the other
    // colors besides blue
    DWORD mask0 = (DWORD) (255 * ( ( y + 1.0  )/ 2.0 ));
    DWORD mask3 = (DWORD) (255 * ( ( -y + 1.0  )/ 2.0 ));
    m_vertices[0].color = 0xff0000ff | ( mask0 << 16 ) | ( mask0 << 8 );
    m_vertices[3].color = 0xff0000ff | ( mask3 << 16 ) | ( mask3 << 8 );

    // write the new vertex information into the buffer
    
	void* pData;
    FAIL_RET( m_vertexBuffer->Lock(0,sizeof(pData), &pData,0) );
    memcpy(pData,m_vertices,sizeof(m_vertices));  
	//ASSERT(0);
    FAIL_RET( m_vertexBuffer->Unlock() );  
    
    D3DXMATRIX mat;
	D3DXMatrixIdentity( &mat );
    
    mat._43 = -2.0f;
	mat._42 = -2.0f;
    //device->SetTransform( D3DTS_WORLD, &mat );
	
	  return hr;
}


HRESULT 
plane_scene::render( IDirect3DDevice9* device, IDirect3DTexture9* texture ) 
{
    HRESULT hr;
    
    //ASSERT(4!=4);
   

	//FAIL_RET( device->SetTexture( 0, texture));

	render_begin(device);

	device->SetTextureStageState( 0, D3DTSS_COLORARG1, D3DTA_TEXTURE );
	device->SetTextureStageState( 0, D3DTSS_COLOROP,   D3DTOP_SELECTARG1 );

    FAIL_RET(hr = device->SetTextureStageState(0, D3DTSS_ALPHAOP, D3DTOP_MODULATE));
    FAIL_RET(hr = device->SetTextureStageState(0, D3DTSS_ALPHAARG1, D3DTA_TEXTURE));
    FAIL_RET(hr = device->SetTextureStageState(0, D3DTSS_ALPHAARG2, D3DTA_DIFFUSE));
    FAIL_RET(hr = device->SetTextureStageState(0, D3DTSS_COLORARG1, D3DTA_TEXTURE));

	FAIL_RET( device->SetTexture( 0, texture));
	//ASSERT(3!=3);
    FAIL_RET( device->SetStreamSource(0, m_vertexBuffer.p, 0, sizeof(plane_scene::CUSTOMVERTEX)  ) ); 
	//FAIL_RET( d3ddev->SetStreamSource(0, m_vertices, 0, sizeof(plane_scene::CUSTOMVERTEX)  ) );            /
	//set next source ( NEW )
    
	
	FAIL_RET( device->SetFVF( D3DFVF_CUSTOMVERTEX ) );
    FAIL_RET( device->DrawPrimitive(D3DPT_TRIANGLESTRIP,0,2) );  //draw quad 
    FAIL_RET( device->SetTexture( 0, NULL));
    //FAIL_RET( d3ddev->EndScene());

	render_end(device);

    return hr;
}

void
plane_scene::rectangle( float fTU, float fTV )
{
    m_vertices[0].tu = 0.0f; m_vertices[0].tv = 0.0f; // low left
    m_vertices[1].tu = 0.0f; m_vertices[1].tv = fTV;  // high left
    m_vertices[2].tu = fTU;  m_vertices[2].tv = 0.0f; // low right
    m_vertices[3].tu = fTU;  m_vertices[3].tv = fTV;  // high right
}
