
#ifndef character_h
#define character_h

#include <hush/export.h>
#include <scene/scene.h>

struct D3DXFRAME_DRVIP: public D3DXFRAME
{
    D3DXMATRIXA16        CombinedTransformationMatrix;
};

//-----------------------------------------------------------------------------
// Name: struct D3DXMESHCONTAINER_DRVIP
// Desc: Structure derived from D3DXMESHCONTAINER so we can add some app-specific
//       info that will be stored with each mesh
//-----------------------------------------------------------------------------
struct D3DXMESHCONTAINER_DRVIP: public D3DXMESHCONTAINER
{
    LPDIRECT3DTEXTURE9*  ppTextures;       // array of textures, entries are NULL if no texture specified    
                                
    // SkinMesh info             
    LPD3DXMESH           pOrigMesh;
    LPD3DXATTRIBUTERANGE pAttributeTable;
    DWORD                NumAttributeGroups; 
    DWORD                NumInfl;
    LPD3DXBUFFER         pBoneCombinationBuf;
    D3DXMATRIX**         ppBoneMatrixPtrs;
    D3DXMATRIX*          pBoneOffsetMatrices;
    DWORD                NumPaletteEntries;
    bool                 UseSoftwareVP;
    DWORD                iAttributeSW;     // used to denote the split between SW and HW if necessary for non-indexed skinning
};

//-----------------------------------------------------------------------------
// Name: class AllocateHierarchy_DRVIP
// Desc: Custom version of ID3DAllocateHierarchy with custom methods to create
//       frames and meshcontainers.
//-----------------------------------------------------------------------------
class AllocateHierarchy_DRVIP: public ID3DXAllocateHierarchy
{
public:
    AllocateHierarchy_DRVIP(IDirect3DDevice9* pDevice);
    virtual ~AllocateHierarchy_DRVIP();

    STDMETHOD(CreateFrame)(
        THIS_ LPCTSTR Name, 
        LPD3DXFRAME *ppNewFrame
        );

    STDMETHOD(CreateMeshContainer)(
        THIS_ LPCTSTR Name, 
        CONST D3DXMESHDATA *pMeshData,
        CONST D3DXMATERIAL *pMaterials, 
        CONST D3DXEFFECTINSTANCE *pEffectInstances, 
        DWORD NumMaterials, 
        CONST DWORD *pAdjacency, 
        LPD3DXSKININFO pSkinInfo, 
        LPD3DXMESHCONTAINER *ppNewMeshContainer
        );

    STDMETHOD(DestroyFrame)(
        THIS_ LPD3DXFRAME pFrameToFree
        );

    STDMETHOD(DestroyMeshContainer)(
        THIS_ LPD3DXMESHCONTAINER pMeshContainerBase
        );

private:
    HRESULT GenerateSkinnedMesh_(D3DXMESHCONTAINER_DRVIP *pMeshContainer);

    //data
private:
    IDirect3DDevice9 *m_pDevice;
};




class HUSH_API character : public scene  
{
public:

    character(char* opts = 0);

	scene* create(char* opts = 0);

	int operator()();
    virtual ~character();
    virtual int _load();
	virtual int _restore();
    virtual int _render();

	int ch(char* x=0);

	int handle(char* type, char* ev);

HRESULT SetupBoneMatrixPointers_(LPD3DXFRAME);
HRESULT SetupBoneMatrixPointersOnMesh_(LPD3DXMESHCONTAINER pMeshContainerBase);
HRESULT DrawFrame_(IDirect3DDevice9* pDevice, LPD3DXFRAME pFrame);
HRESULT DrawMeshContainer_(IDirect3DDevice9* pDevice,
                           LPD3DXMESHCONTAINER pMeshContainer, 
                           LPD3DXFRAME pFrame);
HRESULT UpdateFrameMatrices_(LPD3DXFRAME pFrameBase, D3DXMATRIX* pM);

DWORD GetAnimIndex( char sString[] );
void SetMoveKey();


// data
private:
    BOOL                        m_bInitialized;
	BOOL						m_bAlreadyFailed;
    LPD3DXFRAME                 m_pFrameRoot;
    IDirect3DDevice9*           m_pDevice;
    ID3DXAnimationController*   m_pAnimController;
    D3DXVECTOR3                 m_vObjectCenter;    // Center of bounding sphere of the character
    D3DXMATRIXA16*              m_pBoneMatrices;
    UINT                        m_NumBoneMatricesMax;
    FLOAT                       m_fObjectRadius;
	DWORD                       m_index;
	DWORD                       m_current;
	DWORD                       m_new;


};
#endif 
