/* 
 * tkGlx.c --
 *
 *	This module implements "GLxwin" widgets. This is the TCL
 *	interface to OpenGL.
 *
 * Copyright (c) 1991-1993 The Regents of the University of California.
 * All rights reserved.
 *
 * Permission is hereby granted, without written agreement and without
 * license or royalty fees, to use, copy, modify, and distribute this
 * software and its documentation for any purpose, provided that the
 * above copyright notice and the following two paragraphs appear in
 * all copies of this software.
 * 
 * IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY FOR
 * DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT
 * OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF THE UNIVERSITY OF
 * CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE.  THE SOFTWARE PROVIDED HEREUNDER IS
 * ON AN "AS IS" BASIS, AND THE UNIVERSITY OF CALIFORNIA HAS NO OBLIGATION TO
 * PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
 */

#include <tk.h>
#include <iostream.h>

#if (TK_MAJOR_VERSION >= 4)
#include "tkPort.h"
#else
//#include <tkConfig.h>
#endif

#include <tkInt.h>

#include <GL/glx.h>
#include <GL/gl.h>

/*
 * Currently there is no way to dynamically configure the requirements
 * of the GLX window. You have to change the following static variables
 * to get the set of OpenGL window attributes you want.
 */

#define SUNGL
#ifdef SUNGL
/* want to use COLOR INDEX mode */
static int attributes[] = {GLX_RGBA, GLX_DEPTH_SIZE, 16, GLX_DOUBLEBUFFER, None};
static GLboolean       doubleBuffer = GL_TRUE; //AE GL_TRUE;
#else
/* want to use RGB mode */
static int attributes[] = { GLX_RGBA,
			       GLX_RED_SIZE, 1,
			       GLX_GREEN_SIZE, 1,
			       GLX_ACCUM_BLUE_SIZE, 1,
			       //z GLX_BUFFER_SIZE, 2,
			       //z GLX_LEVEL, 1,
			       //z GLX_TRANSPARENT_TYPE_EXT, GLX_TRANSPARENT_INDEX_EXT,
			       //z GLX_DOUBLEBUFFER,
			       //z GLX_DEPTH_SIZE, 1,
			       GLX_USE_GL, 
			       0 };
static GLboolean       doubleBuffer = GL_FALSE; //AE GL_TRUE;
#endif

//static int dblBuf[] = {GLX_RGBA, GLX_DEPTH_SIZE, 16, None};
static int snglBuf[] = {GLX_RGBA, GLX_DEPTH_SIZE, 16, None};
static int dblBuf[] = {GLX_RGBA, GLX_DEPTH_SIZE, 16, GLX_DOUBLEBUFFER, None};
//static int dblBuf[] = {GLX_RGBA, None }; // GLX_DEPTH_SIZE, 16, None };//GLX_DOUBLEBUFFER, None};
static Display        *dpy;
static Window          win;
static GLfloat         xAngle = 42.0, yAngle = 82.0, zAngle = 112.0;

void
fatalError(char *message)
{
    fprintf(stderr, "glxsimple: %s\n", message);
    exit(1);
}



static GLXContext cx = 0;

/*
 * A data structure of the following type is kept for each glxwin
 * widget managed by this file:
 */

typedef struct {
    Tk_Window tkwin;		/* Window that embodies the glxwin.  NULL
				 * means window has been deleted but
				 * widget record hasn't been cleaned up yet. */
    Display *display;		/* X's token for the window's display. */
    Tcl_Interp *interp;		/* Interpreter associated with widget. */

    int width;			/* Width to request for window.  <= 0 means
				 * don't request any size. */
    int height;			/* Height to request for window.  <= 0 means
				 * don't request any size. */

    int flags;			/* Various flags;  see below for
				 * definitions. */

    int initialized;
    /*
     * Information used when displaying widget:
     */
} Glxwin;

/*
 * Information used for argv parsing.
 */

static Tk_ConfigSpec configSpecs[] = {
    {TK_CONFIG_PIXELS, "-height", "height", "Height",
       0, Tk_Offset(Glxwin, height), 0},
  
    {TK_CONFIG_PIXELS, "-width", "width", "Width",
       0, Tk_Offset(Glxwin, width), 0},

    {TK_CONFIG_END, (char *) NULL, (char *) NULL, (char *) NULL,
       (char *) NULL, 0, 0}
};

/*
 * Forward declarations for procedures defined later in this file:
 */

static int		GlxwinConfigure _ANSI_ARGS_((Tcl_Interp *interp,
			    Glxwin *glxwinPtr, int argc, char **argv,
			    int flags));
static void		GlxwinDestroy _ANSI_ARGS_((char* clientData));
static void		GlxwinEventProc _ANSI_ARGS_((ClientData clientData,
			    XEvent *eventPtr));
static int		GlxwinWidgetCmd _ANSI_ARGS_((ClientData clientData,
			    Tcl_Interp *, int argc, char **argv));

Tk_MakeGLXWindowExist(Tk_Window tkwin)
{
    TkWindow *winPtr = (TkWindow *) tkwin;
    TkWindow *winPtr2;
    XSetWindowAttributes swa;
    Window parent;
    Tcl_HashEntry *hPtr;
    int new_w;
    int dummy;

    /* These parameters are for the GLX window */
    Display 			*dpy	= winPtr->display;

    /* These values will be the same for all GLX windows. Just initialize
     * them once and use for all GLxwin's
     */
    static XVisualInfo		*vi;
    static Colormap 		cmap;

    if (winPtr->window != None) {
	return 0;
    }

    if (winPtr->flags & TK_TOP_LEVEL) {
	parent = XRootWindow(winPtr->display, winPtr->screenNum);
    } else {
	if (winPtr->parentPtr->window == None) {
	    Tk_MakeWindowExist((Tk_Window) winPtr->parentPtr);
	}
	parent = winPtr->parentPtr->window;
    }
    
    if(!glXQueryExtension(dpy, &dummy, &dummy)) fatalError("X server has no OpenGL GLX extension");
 

    if (cx == 0) {
	/* These values gets inittalized the first time a GLxwin widget
	 * is created.
	 */

	/* get an appropriate visual */
	vi = glXChooseVisual(winPtr->display, DefaultScreen(winPtr->display),
			     attributes); //AE attributeList);
    if (vi == NULL) {
       vi = glXChooseVisual(dpy, DefaultScreen(dpy), snglBuf);
       if (vi == NULL) fatalError("no RGB visual with depth buffer");
       doubleBuffer = GL_FALSE;
    }
    //if(vi->class != TrueColor) fatalError("TrueColor visual required for this program");


    /* create an OpenGL rendering context */
    cx = glXCreateContext(dpy, vi, /* no sharing of display lists */ None,
                          /* direct rendering if possible */ GL_TRUE);
    if (cx == NULL) fatalError("could not create rendering context");


   /*** (5) create an X window with the selected visual ***/

    /* create an X colormap since probably not using default visual */
    cmap = XCreateColormap(dpy, RootWindow(dpy, vi->screen), vi->visual, AllocNone);
    }
    // continue ---
    /* Set the attributes */
    winPtr->atts.border_pixel = 0;
    winPtr->atts.colormap     = cmap;
    //swa.event_mask = ExposureMask | ButtonPressMask | StructureNotifyMask;

    /* Now create the window from the X server */
    winPtr->window = XCreateWindow(winPtr->display, parent,
				   winPtr->changes.x, winPtr->changes.y,
				   winPtr->changes.width,
				   winPtr->changes.height,
				   winPtr->changes.border_width,
				   vi->depth, InputOutput, vi->visual,
				   CWBorderPixel|CWColormap|winPtr->dirtyAtts,
                                   &winPtr->atts);

    //glXMakeCurrent(winPtr->display, winPtr->window, cx);
    hPtr = Tcl_CreateHashEntry(&winPtr->dispPtr->winTable,
            (char *) winPtr->window, &new_w);
    Tcl_SetHashValue(hPtr, winPtr);

#ifdef TK_USE_INPUT_METHODS
    winPtr->inputContext = NULL;
#endif /* TK_USE_INPUT_METHODS */



#if (TK_MAJOR_VERSION >= 4)
    XSaveContext(winPtr->display, winPtr->window, XUniqueContext(),
    (caddr_t) winPtr);

#else
    XSaveContext(winPtr->display, winPtr->window, tkWindowContext,
	    (caddr_t) winPtr);
#endif
    winPtr->dirtyAtts = 0;
    winPtr->dirtyChanges = 0;

    /*
     * If any siblings higher up in the stacking order have already
     * been created then move this window to its rightful position
     * in the stacking order.
     *
     * NOTE: this code ignores any changes anyone might have made
     * to the sibling and stack_mode field of the window's attributes,
     * so it really isn't safe for these to be manipulated except
     * by calling Tk_RestackWindow.
     */

    if (!(winPtr->flags & TK_TOP_LEVEL)) {
	for (winPtr2 = winPtr->nextPtr; winPtr2 != NULL;
	     winPtr2 = winPtr2->nextPtr) {
	   if ((winPtr2->window != None) && !(winPtr2->flags & TK_TOP_LEVEL)) {
	       XWindowChanges changes;
	       changes.sibling = winPtr2->window;
	       changes.stack_mode = Below;
	       XConfigureWindow(winPtr->display, winPtr->window,
				CWSibling|CWStackMode, &changes);
	       break;
	   }
        }
    }


	// AE
        if ((winPtr->parentPtr != NULL) &&
                (winPtr->atts.colormap != winPtr->parentPtr->atts.colormap)) {
            TkWmAddToColormapWindows(winPtr);
	    }

    /*
    if ((winPtr->flags & TK_NEED_CONFIG_NOTIFY)
            && !(winPtr->flags & TK_ALREADY_DEAD)){
        winPtr->flags &= ~TK_NEED_CONFIG_NOTIFY;
        GlxwinConfigure(winPtr);
    }
    */

}


/*
 *--------------------------------------------------------------
 *
 * GlxwinCmd --
 *
 *	This procedure is invoked to process the "glxwin" Tcl
 *	command.  It creates a new "glxwin" widget.
 *
 * Results:
 *	A standard Tcl result.
 *
 * Side effects:
 *	A new widget is created and configured.
 *
 *--------------------------------------------------------------
 */

int
GlxwinCmd( //clientData, interp, argc, argv)
    ClientData clientData,	/* Main window associated with
				 * interpreter. */
    Tcl_Interp *interp,		/* Current interpreter. */
    int argc,			/* Number of arguments. */
    char **argv)		/* Argument strings. */
{
    Tk_Window main = (Tk_Window) clientData;
    Glxwin *glxwinPtr;
    Tk_Window tkwin;
    TkWindow* tkWin;

    if (argc < 2) {
	Tcl_AppendResult(interp, "wrong # args:  should be \"",
		argv[0], " pathName ?options?\"", (char *) NULL);
	return TCL_ERROR;
    }

    tkwin = Tk_CreateWindowFromPath(interp, main, argv[1], (char *) NULL);
    if (tkwin == NULL) {
	return TCL_ERROR;
    }
    Tk_SetClass(tkwin, "GLxwin");

    /*
     * Allocate and initialize the widget record.
     */

    glxwinPtr = (Glxwin *) ckalloc(sizeof(Glxwin));
    glxwinPtr->tkwin = tkwin;
    glxwinPtr->display = Tk_Display(tkwin);
    glxwinPtr->interp = interp;
    glxwinPtr->width  = 0;
    glxwinPtr->height = 0;

    Tk_MakeGLXWindowExist(glxwinPtr->tkwin);

    Tk_CreateEventHandler(glxwinPtr->tkwin, ExposureMask|StructureNotifyMask,
	    GlxwinEventProc, (ClientData) glxwinPtr);
    Tcl_CreateCommand(interp, Tk_PathName(glxwinPtr->tkwin), GlxwinWidgetCmd,
	    (ClientData) glxwinPtr, /*(void (*)())*/ NULL);
    if (GlxwinConfigure(interp, glxwinPtr, argc-2, argv+2, 0) != TCL_OK) {
	Tk_DestroyWindow(glxwinPtr->tkwin);
	return TCL_ERROR;
    }


    interp->result = Tk_PathName(glxwinPtr->tkwin);
    return TCL_OK;
}

/*
 *--------------------------------------------------------------
 *
 * GlxwinWidgetCmd --
 *
 *	This procedure is invoked to process the Tcl command
 *	that corresponds to a widget managed by this module.
 *	See the user documentation for details on what it does.
 *
 * Results:
 *	A standard Tcl result.
 *
 * Side effects:
 *	See the user documentation.
 *
 *--------------------------------------------------------------
 */

static int
GlxwinWidgetCmd(//clientData, interp, argc, argv)
    ClientData clientData,		/* Information about glxwin widget. */
    Tcl_Interp *interp,			/* Current interpreter. */
    int argc,				/* Number of arguments. */
    char **argv)			/* Argument strings. */
{
    Glxwin *glxwinPtr = (Glxwin *) clientData;
    int result = TCL_OK;
    int length;
    char c;


    if (argc < 2) {
	Tcl_AppendResult(interp, "wrong # args: should be \"",
		argv[0], " option ?arg arg ...?\"", (char *) NULL);
	return TCL_ERROR;
    }
    Tk_Preserve((ClientData) glxwinPtr);
    c = argv[1][0];
    length = strlen(argv[1]);
    if ((c == 'c') && (strncmp(argv[1], "configure", length) == 0)) {
	if (argc == 2) {
	    result = Tk_ConfigureInfo(interp, glxwinPtr->tkwin, configSpecs,
		    (char *) glxwinPtr, (char *) NULL, 0);
	} else if (argc == 3) {
	    result = Tk_ConfigureInfo(interp, glxwinPtr->tkwin, configSpecs,
		    (char *) glxwinPtr, argv[2], 0);
	} else {
	    result = GlxwinConfigure(interp, glxwinPtr, argc-2, argv+2,
		    TK_CONFIG_ARGV_ONLY);
	}
    } else if ((c == 'l') && (strncmp(argv[1], "link", length) == 0)) {

	/* connect the context to the window
	 * All future OpenGL rendering will be drawn into this window.
	 */
	//cerr << "make current" << endl;
	glXMakeCurrent(glxwinPtr->display, Tk_WindowId(glxwinPtr->tkwin), cx);

    } else if ((c == 's') && (strncmp(argv[1], "swap", length) == 0)) {

	// double
	//cerr << "... swap" << endl;
	if(doubleBuffer) 
		glXSwapBuffers(glxwinPtr->display, Tk_WindowId(glxwinPtr->tkwin));
	else glFlush();

    } else if ((c == 'f') && (strncmp(argv[1], "flush", length) == 0)) {
	//   double buffered
	//cerr << "... swap" << endl;

	if(doubleBuffer) 
		glXSwapBuffers(glxwinPtr->display, Tk_WindowId(glxwinPtr->tkwin));
	else glFlush();

    } else {
	Tcl_AppendResult(interp, "bad option \"", argv[1],
		"\":  must be configure, position, or size", (char *) NULL);
	goto error;
    }

    Tk_Release((ClientData) glxwinPtr);
    return result;

    error:
    Tk_Release((ClientData) glxwinPtr);
    return TCL_ERROR;
}

/*
 *----------------------------------------------------------------------
 *
 * GlxwinConfigure --
 *
 *	This procedure is called to process an argv/argc list in
 *	conjunction with the Tk option database to configure (or
 *	reconfigure) a glxwin widget.
 *
 * Results:
 *	The return value is a standard Tcl result.  If TCL_ERROR is
 *	returned, then interp->result contains an error message.
 *
 * Side effects:
 *	Configuration information, such as colors, border width,
 *	etc. get set for glxwinPtr;  old resources get freed,
 *	if there were any.
 *
 *----------------------------------------------------------------------
 */

static int
GlxwinConfigure(//interp, glxwinPtr, argc, argv, flags)
    Tcl_Interp *interp,			/* Used for error reporting. */
    Glxwin *glxwinPtr,			/* Information about widget. */
    int argc,				/* Number of valid entries in argv. */
    char **argv,			/* Arguments. */
    int flags)				/* Flags to pass to
					 * Tk_ConfigureWidget. */
{
    if (Tk_ConfigureWidget(interp, glxwinPtr->tkwin, configSpecs,
	    argc, argv, (char *) glxwinPtr, flags) != TCL_OK) {
	return TCL_ERROR;
    }

    /*
     * Register the desired geometry for the window.  Then arrange for
     * the window to be redisplayed.
     */

    if ((glxwinPtr->width > 0) || (glxwinPtr->height > 0)) {
	Tk_GeometryRequest(glxwinPtr->tkwin, glxwinPtr->width,
			   glxwinPtr->height);
    }
    return TCL_OK;
}

/*
 *--------------------------------------------------------------
 *
 * GlxwinEventProc --
 *
 *	This procedure is invoked by the Tk dispatcher for various
 *	events on glxwins.
 *
 * Results:
 *	None.
 *
 * Side effects:
 *	When the window gets deleted, internal structures get
 *	cleaned up.  When it gets exposed, it is redisplayed.
 *
 *--------------------------------------------------------------
 */

static void
GlxwinEventProc(//clientData, eventPtr)
    ClientData clientData,	/* Information about window. */
    XEvent *eventPtr)		/* Information about event. */
{
    Glxwin *glxwinPtr = (Glxwin *) clientData;

    if (eventPtr->type == DestroyNotify) {
	Tcl_DeleteCommand(glxwinPtr->interp, Tk_PathName(glxwinPtr->tkwin));
	glxwinPtr->tkwin = NULL;
	Tk_EventuallyFree((ClientData) glxwinPtr, GlxwinDestroy);
    }
    else {
   	int w = glxwinPtr->width;
   	int h = glxwinPtr->height; 
   	glMatrixMode(GL_PROJECTION);
   	glLoadIdentity();
   	glViewport (0, 0, (GLsizei) w, (GLsizei) h);
        glMatrixMode(GL_MODELVIEW);
        glLoadIdentity();
   }
}

/*
 *----------------------------------------------------------------------
 *
 * GlxwinDestroy --
 *
 *	This procedure is invoked by Tk_EventuallyFree or Tk_Release
 *	to clean up the internal structure of a glxwin at a safe time
 *	(when no-one is using it anymore).
 *
 * Results:
 *	None.
 *
 * Side effects:
 *	Everything associated with the glxwin is freed up.
 *
 *----------------------------------------------------------------------
 */

static void
GlxwinDestroy(char* clientData)
    //ClientData clientData)	/* Info about glxwin widget. */
{
    Glxwin *glxwinPtr = (Glxwin *) clientData;

    Tk_FreeOptions(configSpecs, (char *) glxwinPtr, glxwinPtr->display, 0);
    ckfree((char *) glxwinPtr);
}


/*----------------------------------------------------------------------
 * TkGLx_Init()
 *
 * TkGlx Initialization routine: Create the command that manipulates
 * OpenGL widgets.
 *
 *----------------------------------------------------------------------
 */

int TkGLx_Init(Tcl_Interp *interp, Tk_Window  topLevel)
{
    Tcl_CreateCommand(interp, "tkglxwin", GlxwinCmd, topLevel, NULL);

    return TCL_OK;
}
