/*
 * Define a JS object called "it".  Give it class operations that printf why
 * they're being called for tutorial purposes.
 */
enum hush_tinyid {
    HUSH_COLOR, HUSH_HEIGHT, HUSH_WIDTH, HUSH_FUNNY, HUSH_ARRAY, HUSH_RDONLY
};

static JSPropertySpec hush_props[] = {
    {"color",           HUSH_COLOR,	JSPROP_ENUMERATE},
    {"height",          HUSH_HEIGHT,	JSPROP_ENUMERATE},
    {"width",           HUSH_WIDTH,	JSPROP_ENUMERATE},
    {"funny",           HUSH_FUNNY,	JSPROP_ENUMERATE},
    {"array",           HUSH_ARRAY,	JSPROP_ENUMERATE},
    {"rdonly",		HUSH_RDONLY,	JSPROP_READONLY},
    {0}
};

static JSBool hush_noisy;    /* whether to be noisy when finalizing it */

static JSBool
hush_addProperty(JSContext *cx, JSObject *obj, jsval id, jsval *vp)
{
    if (hush_noisy) {
	printf("adding hush property %s,",
	       JS_GetStringBytes(JS_ValueToString(cx, id)));
	printf(" initial value %s\n",
	       JS_GetStringBytes(JS_ValueToString(cx, *vp)));
    }
    return JS_TRUE;
}

static JSBool
hush_delProperty(JSContext *cx, JSObject *obj, jsval id, jsval *vp)
{
    if (hush_noisy) {
	printf("deleting hush property %s,",
	       JS_GetStringBytes(JS_ValueToString(cx, id)));
	printf(" current value %s\n",
	       JS_GetStringBytes(JS_ValueToString(cx, *vp)));
    }
    return JS_TRUE;
}

static JSBool
hush_getProperty(JSContext *cx, JSObject *obj, jsval id, jsval *vp)
{
    if (hush_noisy) {
	printf("getting hush property %s,",
	       JS_GetStringBytes(JS_ValueToString(cx, id)));
	printf(" current value %s\n",
	       JS_GetStringBytes(JS_ValueToString(cx, *vp)));
    }
    return JS_TRUE;
}

static JSBool
hush_setProperty(JSContext *cx, JSObject *obj, jsval id, jsval *vp)
{
    if (hush_noisy) {
	printf("setting hush property %s,",
	       JS_GetStringBytes(JS_ValueToString(cx, id)));
	printf(" new value %s\n",
	       JS_GetStringBytes(JS_ValueToString(cx, *vp)));
    }
    if (JSVAL_IS_STRING(id) &&
	!strcmp(JS_GetStringBytes(JSVAL_TO_STRING(id)), "noisy")) {
	return JS_ValueToBoolean(cx, *vp, &hush_noisy);
    }
    return JS_TRUE;
}

static JSBool
hush_enumerate(JSContext *cx, JSObject *obj)
{
    if (hush_noisy)
	printf("enumerate hush properties\n");
    return JS_TRUE;
}

static JSBool
hush_resolve(JSContext *cx, JSObject *obj, jsval id)
{
    if (hush_noisy) {
	printf("resolving hush property %s\n",
	       JS_GetStringBytes(JS_ValueToString(cx, id)));
    }
    return JS_TRUE;
}

static JSBool
hush_convert(JSContext *cx, JSObject *obj, JSType type, jsval *vp)
{
    if (hush_noisy)
	printf("converting it to %s type\n", JS_GetTypeName(cx, type));
    return JS_TRUE;
}

static void
hush_finalize(JSContext *cx, JSObject *obj)
{
    if (hush_noisy)
	printf("finalizing it\n");
}

static JSClass hush_class = {
    "hush", 0,
    hush_addProperty,  hush_delProperty,  hush_getProperty,  hush_setProperty,
    hush_enumerate,    hush_resolve,      hush_convert,      hush_finalize
};

static void
hush_ErrorReporter(JSContext *cx, const char *message, JSErrorReport *report)
{
    int i, j, k, n;

    fputs("js: ", stderr);
    if (!report) {
	fprintf(stderr, "%s\n", message);
	return;
    }

    if (report->filename)
	fprintf(stderr, "%s, ", report->filename);
    if (report->lineno)
	fprintf(stderr, "line %u: ", report->lineno);
    fputs(message, stderr);
    if (!report->linebuf) {
	putc('\n', stderr);
	return;
    }

    fprintf(stderr, ":\n%s\n", report->linebuf);
    n = report->tokenptr - report->linebuf;
    for (i = j = 0; i < n; i++) {
	if (report->linebuf[i] == '\t') {
	    for (k = (j + 8) & ~7; j < k; j++)
		putc('.', stderr);
	    continue;
	}
	putc('.', stderr);
	j++;
    }
    fputs("^\n", stderr);
}
