#include <hush/hush.h>
#include <hush/dv/plugin/kit.h>
#include <hush/dv/plugin/event.h>
#include <hush/dv/plugin/file.h>

#include <tcl.h>

#include <iostream.h>

extern int plugin_callback(void* data, Tcl_Interp*, int, char**);
extern void plugin_timer_callback( void* data );

#define INTERP ((Tcl_Interp*) _interp)

plugin_kit::plugin_kit() : kit(this) {
	_interp = (void*) Tcl_CreateInterp();
	Tcl_Init(INTERP);
	}

plugin_kit::~plugin_kit() {
        Tcl_DeleteInterp(INTERP);
	}

plugin_kit::plugin_kit(void* x) {
	_interp = x;
	}

int plugin_kit::source(const char* cmd) {
        if (tracelevel()>0) cerr << "source " << (char*) cmd << endl;
        int err = Tcl_EvalFile(INTERP,(char*)cmd);
        if (err) 
          cerr << "WARNING: source " << cmd << " evaluates to FALSE" << endl;
	return err;
        }

long plugin_kit::eval(const char* cmd) {
        if (tracelevel()>0) cerr << (char*) cmd << endl;
        int err = Tcl_Eval(INTERP,(char*)cmd);
        if (err) 
          cerr << "WARNING: " << cmd << " evaluates to FALSE" << endl;
        return err;
        }

char* plugin_kit::result(long) {
	return INTERP->result;
	}

binding* plugin_kit::bind(const char* name, handler* h) {
	Tcl_CreateCommand(INTERP,(char*) name, plugin_callback,
        (ClientData) h,0);
        binding* tmp = new binding(this,name,0);
        _register(tmp);
        return tmp;
	}

binding* plugin_kit::bind(int fd, handler* h, int mask) {
	plugin_file_callback* data = new plugin_file_callback(h,fd);
	int tmask = mask;
	if (mask==kit::readable) tmask = TCL_READABLE;
	else if (mask == kit::writable) tmask = TCL_WRITABLE;
	else if (mask == kit::exception) tmask = TCL_EXCEPTION;
	else if (mask == 0)  tmask = TCL_READABLE | TCL_EXCEPTION;
	//Tcl_CreateFileHandler(fd, tmask, (Tcl_ChannelProc*) plugin_file_handler_callback, (ClientData)data);
        binding* tmp = new binding(this,"nop",0);
        _register(tmp);
        return tmp;
	}

void plugin_kit::after(int t, handler* h) {
	char buf[1024];
        if (!h) {
	        sprintf(buf,"after %d %s",t,"");
		}
        else {
		Tcl_CreateTimerHandler(t, (void (*)(ClientData)) plugin_timer_callback, (ClientData) h);
                }
        }

