////////////////////////////////////////////////////////////////////////
//                                                                    //
// default.h                                                          //
//                                                                    //
// contains functions for retrieving the default part from a SIGMA    //
// input-file. Also members for reading and displaying this part      //
// are available. Private members are just the default fields from    //
// the input-file.                                                    //
//                                                                    //
// (c) 1995 A.R. van der Valk, A. Eliens                              //
//                                                                    //
////////////////////////////////////////////////////////////////////////


#ifndef DEFAULT_H
#define DEFAULT_H

#include <hush/dv/sigma/list.h>

typedef enum {STOP_ON_EVENT, STOP_ON_TIME, INITIAL} endingConditionType;

class defaults 
{
private:
    char *name;  
    char *description;  
    char *declaration;  
    char* action;
    char *outputFile;  
    sigma_list *trace_vars; 
    int seed;  
    sigma_list *initial_values;    
    endingConditionType ending_condition; 
    char *stop_event;
    double ending_condition_nr;  
    sigma_list *trace_events; 

public:
    defaults(); 
    virtual ~defaults();
    int read_defaults(istream& s, int* x =0);
    void display_defaults();

    char* return_name() { return name; }
    char* return_description() { return description; }
    char* return_declaration() { return declaration; }
    char* return_action() { return action; }
    char* return_ofile() { return outputFile; }
    sigma_list_item* get_first_trace_var();
    sigma_list_item* get_next_trace_var(sigma_list_item *i);
    char* trace_var_name(sigma_list_item *pt);
    char* get_stripped(sigma_list_item *pt);
    int return_seed() { return seed; }
    sigma_list_item* get_first_initial_val();
    sigma_list_item* get_next_initial_val(sigma_list_item *i);
    double return_value(sigma_list_item *i);
    endingConditionType return_end_condition() { return ending_condition; }
    char* return_stop_event() { return stop_event; }
    double return_end_cond_nr() { return ending_condition_nr; }
    char* get_first_trace_event();
    char* get_next_trace_event(sigma_list_item *i);
};

#endif
