////////////////////////////////////////////////////////////////////////
//                                                                    //
// edge.c                                                             //
//                                                                    //
// the implementation of the edge class, it provides members for      //
// reading the edges from a SIGMA input file and retrieving each      //
// separate edge and its features.                                    //
//                                                                    //
// (c) 1995 A.R. van der Valk, A. Eliens                              //
//                                                                    //
////////////////////////////////////////////////////////////////////////

#include <hush/dv/sigma/edge.h>
#include <hush/dv/sigma/list.h>
#include <hush/dv/sigma/convert.h>

#include <stdlib.h>
#include <string.h>
#include <iostream.h>
#include <fstream.h>
#include <strstream.h>
#include <iomanip.h>

#define NULL 0

// the destructor, removes all elements from the list of edges

edge_list::~edge_list()
{
    while (is_empty() != true)
        remove();
}


// removes the first element from the list of edges

void
edge_list::remove()
{
    if (is_empty() == true)
    {
        cout << "remove() on empty list\n";
        exit(-1);
    }

    edge_l_item *pt = front;
    front = front->next;

    pt->val.name = NULL;
    pt->val.description = NULL;
    pt->val.action = NULL;
    pt->val.sort_edge = Scheduling;
    pt->val.origin = NULL;
    pt->val.destination = NULL;
    pt->val.condition = NULL;
    pt->val.delay = NULL;
    pt->val.priority = 0;
    pt->val.attributes->~sigma_list();

    //AE delete pt;
}


// add an element at the end of the edge list 

void
edge_list::add(edge_type item) {
 
    edge_l_item *pt = new edge_l_item(item);

    if (is_empty())
        front = back = pt;
    else
    {
        back->next = pt;
        back = pt;
    }
}


// display the edges on screen

void
edge_list::display_edges()
{
    cout << "EDGES" << endl << endl;

    for (edge_l_item *pt=front; pt; pt=pt->next)
    {
        cout << "Name: " << pt->val.name << endl;
        cout << "Description: " << pt->val.description << endl;
        cout << "Action: " << pt->val.action << endl;
        cout << "Type: " << pt->val.sort_edge << endl;
        cout << "Origin: " << pt->val.origin << endl;
        cout << "Destination: " << pt->val.destination << endl;
        cout << "Condition: " << pt->val.condition << endl;
        cout << "Delay: " << pt->val.delay << endl;
        cout << "Priority: " << pt->val.priority << endl;
        cout << "Attributes: ";
        pt->val.attributes->display();
        cout << endl;
    }
}
    

// read the edges from the SIGMA input-file into the edge list

int
edge_list::read_edges(istream& s, int* cont)
{
    const bufsize = 1024;
    char inBuf[bufsize];
    edge_type hulp_var;
    char *hulp_buffer;
    int go_on = 1;
    int i=0, j=0;

    hulp_buffer = new char[bufsize];

    if (*cont == 1) strcpy(inBuf,"Edge:");


    //  lezen totdat edges gedeelte begint

    while ( !*cont && (s >> setw(bufsize) >> inBuf)) {
	//cerr << "Edging " << inBuf << endl;
	if ((strcasecmp(inBuf,"Edge") == 0) 
	    || (strcasecmp(inBuf,"Arrow:") == 0)
	    || (strcasecmp(inBuf,"Edge:") == 0)
	    || (strcasecmp(inBuf,"Description:") == 0)
	    || (strcasecmp(inBuf,"Info:") == 0)
	    )
		{
		cerr << "Got " << inBuf << endl;
		*cont = 1;
		}
	}


    while (*cont || (s >> setw(bufsize) >> inBuf))
    {
	*cont = 0;
	cerr << "Processing edges: " << inBuf << endl;
        if ( (strcasecmp(inBuf, "Name:") == 0)
	     || (strcasecmp(inBuf, "Edge:") == 0) 
	     || (strcasecmp(inBuf, "Arrow:") == 0) )
        {
	    *cont = 0;
            s >> setw(bufsize) >> inBuf;
            hulp_var.name = new char[bufsize];
            strcpy(hulp_var.name, inBuf);
	    cerr << "Name: " << inBuf << endl;;
        }

        if ((strcasecmp(inBuf, "Description:") == 0)
	    || (strcasecmp(inBuf, "Desc:") == 0)
	    || (strcasecmp(inBuf, "Info:") == 0))
        {
            hulp_var.description = new char[bufsize];
            while (go_on)
            {
                s >> setw(bufsize) >> inBuf;
                if ((strcasecmp(inBuf,"Type:") == 0)
		    || (strcasecmp(inBuf,"Action:") == 0))
                    go_on = 0;
                else
                {
                    strcat(hulp_var.description, inBuf);
                    strcat(hulp_var.description, " ");
                }
            }
            go_on = 1;
        }
        if (strcasecmp(inBuf, "Action:") == 0)
        {
            hulp_var.action = new char[bufsize];
            hulp_var.action[0] = '\0';
            while (go_on)
            {
                s >> setw(bufsize) >> inBuf;
                if ((strcasecmp(inBuf,"Type:") == 0)
		    || (strcasecmp(inBuf,"Origin:") == 0))
                    go_on = 0;
                else
                {
                    strcat(hulp_var.action, inBuf);
                    strcat(hulp_var.action, " ");
                }
            }
            go_on = 1;
        }
        if (strcasecmp(inBuf,"Type:") == 0)
        {
            s >> setw(bufsize) >> inBuf;
            if (strcasecmp(inBuf,"Scheduling") == 0)
                hulp_var.sort_edge = Scheduling;
            else
                hulp_var.sort_edge = Cancelling;
        }
        if ((strcasecmp(inBuf,"Origin:") == 0)
	    || (strcasecmp(inBuf,"Source:") == 0)
	    || (strcasecmp(inBuf,"Domain:") == 0))
        {
            s >> setw(bufsize) >> inBuf;
            hulp_var.origin = new char[bufsize];
            strcpy(hulp_var.origin, inBuf);
        }
        if ((strcasecmp(inBuf,"Destination:") == 0)
	    || (strcasecmp(inBuf,"Sink:") == 0)
	    || (strcasecmp(inBuf,"Range:") == 0)
	    || (strcasecmp(inBuf,"Codomain:") == 0))
        {
            s >> setw(bufsize) >> inBuf;
            hulp_var.destination = new char[bufsize];
            strcpy(hulp_var.destination, inBuf);
        }
        if ((strcasecmp(inBuf, "Condition:") == 0)
	    || (strcasecmp(inBuf, "If:") == 0)
	    || (strcasecmp(inBuf, "When:") == 0)
	    || (strcasecmp(inBuf, "Provided:") == 0)
	    || (strcasecmp(inBuf, "Whenever:") == 0))
        {
            hulp_var.condition = new char[bufsize];
            while (go_on)
            {
                s >> setw(bufsize) >> inBuf;
                if (strcasecmp(inBuf,"Delay:") == 0)
                    go_on = 0;
                else
                {
                    strcat(hulp_var.condition, inBuf);
                    strcat(hulp_var.condition, " ");
                }
            }
            go_on = 1;
        }
        if ((strcasecmp(inBuf, "Delay:") == 0)
	    || (strcasecmp(inBuf, "After:") == 0)
	    || (strcasecmp(inBuf, "Wait:") == 0))
        {
            hulp_var.delay = new char[bufsize];
            while (go_on)
            {
                s >> setw(bufsize) >> inBuf;
                if (strcasecmp(inBuf,"Priority:") == 0)
                    go_on = 0;
                else
                {
                    strcat(hulp_var.delay, inBuf);
                    strcat(hulp_var.delay, " ");
                }
            }
            go_on = 1;
        }
        if ((strcasecmp(inBuf,"Priority:") == 0)
	    || (strcasecmp(inBuf,"Prio:") == 0)
	    || (strcasecmp(inBuf,"Rank:") == 0))
        {
            s >> setw(bufsize) >> inBuf;
            hulp_var.priority = atoi(inBuf);
        }
        if ((strcasecmp(inBuf,"Attributes:") == 0)
	    || (strcasecmp(inBuf,"Attrib:") == 0)
	    || (strcasecmp(inBuf,"With:") == 0))
        {
	    int next = 0;
            i=j=0;
            hulp_buffer = new char[bufsize];
            hulp_var.attributes = new sigma_list();
            if ( (s >> setw(bufsize) >> inBuf) && (
	       (strcasecmp(inBuf, "Graphics") != 0))
	      && (strcasecmp(inBuf,"Sub-Edge") != 0)) 
            {
		if ( (strcasecmp(inBuf,"Edge:") == 0 )
		     || (strcasecmp(inBuf,"Arrow:") == 0 )) {
					next = 1;
					*cont = 1;
					}
                if (!next) while (inBuf[i] != '\0')
                {
                    switch (inBuf[i])
                    {
                        case ',' : hulp_var.attributes->add(hulp_buffer);
                                   hulp_buffer = new char[bufsize];
                                   j = 0;
                                   break;
                        default  : *(hulp_buffer + j) = inBuf[i];
                                   if (inBuf[i+1] == '\0') 
                                       hulp_var.attributes->add(hulp_buffer);
                                   j++;
                                   break;
                    }
                    i++;
                }
            }
        add(hulp_var);
        }
        
     }
     return (1);
}


// constructor fir a edge list item

edge_l_item::edge_l_item(edge_type value) 
{
    val.name = new char[1024];
    sprintf(val.name,"anonymous");
    val.description = new char[1024];
    sprintf(val.description,"unknown");
    val.action = new char[1024];
    sprintf(val.action,"// no action");
    val.sort_edge = Scheduling; // moet initial zijn ofzo
    val.origin = new char[1024];
    val.destination = new char[1024];
    val.condition = new char[1024];
    val.delay = new char[1024];
    val.priority = 0;
    val.attributes = new sigma_list();

    if (value.name) strcpy(val.name,value.name);
    if (value.description) strcpy(val.description,value.description);
    if (value.action) strcpy(val.action,value.action);
    val.sort_edge = value.sort_edge;
    strcpy(val.origin,value.origin);
    strcpy(val.destination,value.destination);
    strcpy(val.condition,value.condition);
    strcpy(val.delay,value.delay);
    val.priority = value.priority;
    val.attributes = value.attributes;

    next = 0;
}


// retrieve first attribute from a single edge

sigma_list_item*
edge_list::get_first_attr(edge_l_item *pt)
{
    sigma_list_item *i = pt->val.attributes->get_front();
    if (i == NULL)
        return (NULL);
    else
        return (i);
}
    

// retrieve next attribute from a single edge

sigma_list_item*
edge_list::get_next_attr(edge_l_item *pt, sigma_list_item *j)
{
    sigma_list_item *i = pt->val.attributes->get_next(j);
    if (i == NULL)
        return (NULL);
    else
        return (i);
}


// retrieve the name from an attribute from a single edge

char*
edge_list::get_attrib_name(edge_l_item *pt, sigma_list_item *j)
{
    return (pt->val.attributes->get_val(j));
}


// retrieve the edge condition which is useful for C++. This means
// `&' is replaced by `&&' etc.

char*
edge_list::return_condition(edge_l_item *pt)
{   

    int j = 0;
    char *hulp = new char[1024];
    int lengte = strlen(pt->val.condition);
    char *buf = new char[lengte];
    strncpy(buf,pt->val.condition,lengte);

    //if (convert::mode > 0) return buf; // AE

    for (int i = 0; i < lengte; i++)
    {
        switch (buf[i])
        {
            case '{': 
              *(hulp + j) = '(';
              break;
            case '}':
              *(hulp + j) = ')';
              break;
            case ';':
              *(hulp + j) = ',';
              break;
            case '&':
              *(hulp + j) = '&';
              if (buf[i+1]!='&') j++;
              *(hulp + j) = '&';
              break;
            case '|':
              *(hulp + j) = '|';
              if (buf[i+1]!='|') j++; // AE bug?
            break;
            default:
              *(hulp + j) = buf[i];
              break;
        }
        j++;
    }
    return (hulp);
}


// retrieve the edge delay time

char*
edge_list::return_delay(edge_l_item *pt)
{
    int length_original;
    int length_hulp;
    int place;
    char *buffer;
    char *hulp;

    length_original = strlen(pt->val.delay);    
    buffer = new char[length_original];
    strcpy(buffer, pt->val.delay);

    if (strstr(buffer, "{") == NULL)
        return (buffer);
    else
    {
        while ((hulp = strstr(buffer, "{")) != NULL)
        {
            length_hulp = strlen(hulp);
            place = (length_original - length_hulp);
            *(buffer + place) = '(';
        }
    
      
        while ((hulp = strstr(buffer, "}")) != NULL)
        {
            length_hulp = strlen(hulp);
            place = (length_original - length_hulp);
            *(buffer + place) = ')';
        }
    }

    return (buffer);
}
