////////////////////////////////////////////////////////////////////////
//                                                                    //
// edge.h                                                             //
//                                                                    //
// contains members for reading the edges into a list, because there  //
// is no limit to the number of edges. The elements of the list       //
// consist of a struct containing the fields necessary for            //
// representing the edges.                                            //
//                                                                    //
// (c) 1995 A.R. van der Valk, A. Eliens                              //
//                                                                    //
////////////////////////////////////////////////////////////////////////


#ifndef EDGE_H
#define EDGE_H

#include <hush/dv/sigma/list.h>

typedef enum {Scheduling, Cancelling} sort;

struct edge_type {
    edge_type() { name = 0; description = 0; action = 0; }
    char *name;  
    char *description;  
    char *action;  
    sort sort_edge;     
    char *origin;
    char *destination;
    char *condition;
    char *delay;
    int priority;
    sigma_list *attributes; 
};

class edge_l_item {
    friend class edge_list;

private:
    edge_l_item(edge_type);
    edge_type val;
    edge_l_item *next;
};

class edge_list 
{
private:
    edge_l_item *front;
    edge_l_item *back;
    
public:
    edge_list() {front = back = 0; }
    ~edge_list();
    void remove();
    void add(edge_type);
    void display_edges();
    bool is_empty()
    { return front == 0 ? true : false; }

    int read_edges(istream& s, int* x = 0);

    edge_l_item* return_front() {return front;}
    edge_l_item* return_next(edge_l_item *pt) { return pt->next; }

    char* return_name(edge_l_item *pt) { return pt->val.name; }
    char* return_description(edge_l_item *pt) { return pt->val.description; }
    char* return_action(edge_l_item *pt) { return pt->val.action; }

    sort return_sort(edge_l_item *pt) { return pt->val.sort_edge; }
    char* return_origin(edge_l_item *pt) { return pt->val.origin; }
    char* return_destination(edge_l_item *pt) { return pt->val.destination; }
    char* return_condition(edge_l_item *pt); /* in C++ language*/
    char* return_delay(edge_l_item *pt); /* in C++ language */
    int return_priority(edge_l_item *pt) { return pt->val.priority; }
    sigma_list_item* get_first_attr(edge_l_item *pt);
    sigma_list_item* get_next_attr(edge_l_item *pt, sigma_list_item *j);
    char* get_attrib_name(edge_l_item *pt, sigma_list_item *j);
};

#endif
