////////////////////////////////////////////////////////////////////////
//                                                                    //
// model.c                                                            //
//                                                                    //
// the implementation of the model class, it provides members for     //
// creating and reading a model. Also, members are provided to        //
// retrieve each separate part from this model.                       //
//                                                                    //
// (c) 1995 A.R. van der Valk, A. Eliens                              //
//                                                                    //
////////////////////////////////////////////////////////////////////////

#include <hush/dv/sigma/model.h>

#include <iostream.h>
#include <fstream.h>
#include <strstream.h>
#include <iomanip.h>


// constructor for a model

model::model()
{
    d = new defaults();
    s = new state_var_l();
    v = new vertex_list();
    e = new edge_list();
}


// destructor for a model

model::~model()
{
    d->~defaults();
    s->~state_var_l();
    v->~vertex_list();
    e->~edge_list();
}


// read a model, this means read the four different parts

void
model::read_model(char *input_file)
{
    ifstream ifile(input_file, ios::in);
    
    int cont = 0; // AE to check for keywords
    d->read_defaults(ifile, &cont);
    s->read_state_vars(ifile,&cont);
    v->read_vertices(ifile,&cont);
    e->read_edges(ifile,&cont);
    ifile.close();
}


// display the model definition on screen

void
model::display_model()
{
    d->display_defaults();
    s->display_state_vars();
    v->display_vertices();
    e->display_edges();
}


// write the model to the output-file

int
model::write_model(char *output_file)
{
    ofstream ofile(output_file, ios::out);
    ofile.close();
    return(1);
}


// read the default section out of an input-file

int
model::read_defaults(char *input_file)
{
    ifstream ifile(input_file, ios::in);
    d->read_defaults(ifile);
    ifile.close();
    return(1);
}


// write the default section

void
model::write_defaults()
{
    d->display_defaults();
}


// read the state variables out of an input-file

int
model::read_state_vars(char *input_file)
{
    ifstream ifile(input_file, ios::in);
    s->read_state_vars(ifile);
    ifile.close();
    return(1);
}


void
model::write_state_vars()
{
    s->display_state_vars();
}


// read the vertices out of an input-file

int
model::read_vertices(char *input_file)
{
    ifstream ifile(input_file, ios::in);
    v->read_vertices(ifile);
    ifile.close();
    return(1);
}


void
model::write_vertices()
{
    v->display_vertices();
}


// read the edges out of an input-file

int
model::read_edges(char *input_file)
{
    ifstream ifile(input_file, ios::in);
    e->read_edges(ifile);
    ifile.close();
    return(1);
}


void
model::write_edges()
{
    e->display_edges();
}


/***********************************************************************
 functions to retrieve the default section of a model
********************************************************************* */

char*
model::def_name()
{
    return (d->return_name());
}


char*
model::def_description()
{
    return (d->return_description());
}

char*
model::def_declaration()
{
    return (d->return_declaration());
}

char*
model::def_action()
{
    return (d->return_action());
}


char*
model::def_outputfile()
{
    return (d->return_ofile());
}


sigma_list_item*
model::get_f_trace_var()
{
    return (d->get_first_trace_var());
}


sigma_list_item*
model::get_n_trace_var(sigma_list_item *pt)
{
    return (d->get_next_trace_var(pt));
}


char*
model::trace_var_name(sigma_list_item *pt)
{
    return (d->trace_var_name(pt));
}


char*
model::stripped_trace_var(sigma_list_item *pt)
{
    return (d->get_stripped(pt));
}



int
model::def_seed()
{
    return (d->return_seed());
}


sigma_list_item*
model::get_f_init_val()
{
    return (d->get_first_initial_val());
}


sigma_list_item*
model::get_n_init_val(sigma_list_item *pt)
{
    return (d->get_next_initial_val(pt));
}


double
model::get_value(sigma_list_item *pt)
{
    return (d->return_value(pt));
}


endingConditionType
model::def_ending_cond()
{
    return (d->return_end_condition());
}


char*
model::def_stop_event()
{
    return (d->return_stop_event());
}


double 
model::def_end_cond_nr()
{
    return (d->return_end_cond_nr());
}


char*
model::get_f_trace_event()
{
    return (d->get_first_trace_event());
}


char*
model::get_n_trace_event(sigma_list_item *pt)
{
    return (d->get_next_trace_event(pt));
}


/***********************************************************************
        functions to retrieve the state variables out of a model
***********************************************************************/

st_var_l_item*
model::first_st_var()
{
    return (s->return_front());
}


st_var_l_item*
model::next_st_var(st_var_l_item *pt)
{
    return (s->return_next(pt));
}


char*
model::state_var_name(st_var_l_item *pt)
{
    return (s->return_name(pt));
}


char*
model::state_var_description(st_var_l_item *pt)
{
    return (s->return_description(pt));
}


var_type
model::state_var_type(st_var_l_item *pt)
{
    return (s->return_type(pt));
}


char*
model::state_var_size(st_var_l_item *pt)
{
    return (s->return_size(pt));
}

st_var_l_item*
model::search_st_var(char *str)
{
    return (s->search(str));
}


int
model::is_array(st_var_l_item *pt)
{
    return (s->sv_array(pt));
}

/***********************************************************************
      functions to retrieve the vertices out of a model
***********************************************************************/

vertex_l_item*
model::first_vertex()
{
    return (v->return_front());
}


vertex_l_item*
model::next_vertex(vertex_l_item *pt)
{
    return(v->return_next(pt));
}


char*
model::vertex_name(vertex_l_item *pt)
{
    return (v->return_name(pt));
}


char*
model::vertex_description(vertex_l_item *pt)
{
    return (v->return_description(pt));
}


char*
model::vertex_action(vertex_l_item *pt)
{
    return (v->return_action(pt));
}

sigma_list_item*
model::get_f_st_change(vertex_l_item *pt)
{
    return(v->get_first_st_change(pt));
}


sigma_list_item*
model::get_n_st_change(vertex_l_item *pt, sigma_list_item *j)
{
    return(v->get_next_st_change(pt, j));
}


char*
model::st_change_name(vertex_l_item *pt, sigma_list_item *j)
{
    return(v->get_st_change_name(pt, j));
}


char*
model::stripped_st_ch(vertex_l_item *pt, sigma_list_item *j)
{
    return(v->return_stripped(pt, j));
}


sigma_list_item*
model::get_f_param(vertex_l_item *pt)
{
    return(v->get_first_param(pt));
}


sigma_list_item*
model::get_n_param(vertex_l_item *pt, sigma_list_item *j)
{
    return(v->get_next_param(pt, j));
}


char*
model::param_name(vertex_l_item *pt, sigma_list_item *j)
{
    return(v->get_param_name(pt, j));
}


/***********************************************************************
       functions to retrieve the edges out of a model
***********************************************************************/

edge_l_item*
model::first_edge()
{
    return (e->return_front());
}


edge_l_item*
model::next_edge(edge_l_item *pt)
{
    return (e->return_next(pt));
}

char*
model::edge_name(edge_l_item *pt)
{
    return (e->return_name(pt));
}

char*
model::edge_description(edge_l_item *pt)
{
    return (e->return_description(pt));
}

char*
model::edge_action(edge_l_item *pt)
{
    return (e->return_action(pt));
}


sort
model::edge_sort(edge_l_item *pt)
{
    return (e->return_sort(pt));
}


char*
model::edge_origin(edge_l_item *pt)
{
    return (e->return_origin(pt));
}


char*
model::edge_destination(edge_l_item *pt)
{
    return (e->return_destination(pt));
}


char*
model::edge_condition(edge_l_item *pt)
{
    return (e->return_condition(pt));
}


char*
model::edge_delay(edge_l_item *pt)
{
    return (e->return_delay(pt));
}


int
model::edge_priority(edge_l_item *pt)
{
    return (e->return_priority(pt));
}


sigma_list_item*
model::get_f_attr(edge_l_item *pt)
{
    return (e->get_first_attr(pt));
}


sigma_list_item*
model::get_n_attr(edge_l_item *pt, sigma_list_item *j)
{
    return (e->get_next_attr(pt, j));
}


char*
model::attrib_name(edge_l_item *pt, sigma_list_item *j)
{
    return(e->get_attrib_name(pt, j));
}
