////////////////////////////////////////////////////////////////////////
//                                                                    //
// model.h                                                            //
//                                                                    //
// the definition of a model as described in the SIGMA input-file.    //
// A model consists of four distinct parts, described in the default  //
// part of the class model. Functions are available for reading,      //
// displaying an retrieving the whole model or just one of the parts. //
//                                                                    //
// (c) 1995 A.R. van der Valk, A. Eliens                              //
//                                                                    //
////////////////////////////////////////////////////////////////////////


#ifndef MODEL_H
#define MODEL_H


#include <hush/dv/sigma/default.h>
#include <hush/dv/sigma/statevar.h>
#include <hush/dv/sigma/vertex.h>
#include <hush/dv/sigma/edge.h>

class model
{
private:
    defaults *d;
    state_var_l *s;
    vertex_list *v;
    edge_list *e;

public:
    model();
    ~model();
    void read_model(char *input_file);
    int write_model(char *output_file);
    void display_model();
    int read_defaults(char *input_file);
    void write_defaults();
    int read_state_vars(char *input_file);
    void write_state_vars();
    int read_vertices(char *input_file);
    void write_vertices();
    int read_edges(char *input_file);
    void write_edges();

/**********************************************************************
        functions for retrieving the defaults from a input-file
***********************************************************************/

    char* def_name(); 
    char* def_description();
    char* def_declaration();
    char* def_action();
    char* def_outputfile();
    sigma_list_item* get_f_trace_var();
    sigma_list_item* get_n_trace_var(sigma_list_item *pt);
    char* trace_var_name(sigma_list_item *pt);
    char* stripped_trace_var(sigma_list_item *pt);
    int def_seed();
    sigma_list_item* get_f_init_val();
    sigma_list_item* get_n_init_val(sigma_list_item *pt);
    double get_value(sigma_list_item *pt);
    endingConditionType def_ending_cond();
    char* def_stop_event();
    double def_end_cond_nr();
    char* get_f_trace_event();
    char* get_n_trace_event(sigma_list_item *pt);

/***********************************************************************
        functions for retrieving the state variables from a input-file
***********************************************************************/

    st_var_l_item* first_st_var();
    st_var_l_item* next_st_var(st_var_l_item *pt);
    char* state_var_name(st_var_l_item *pt);
    char* state_var_description(st_var_l_item *pt);
    var_type state_var_type(st_var_l_item *pt);
    char* state_var_size(st_var_l_item *pt);
    st_var_l_item* search_st_var(char *str);
    int is_array(st_var_l_item *pt);

/***********************************************************************
        functions for retrieving the vertices from a input-file
***********************************************************************/

    vertex_l_item* first_vertex();
    vertex_l_item* next_vertex(vertex_l_item *pt);
    char* vertex_name(vertex_l_item *pt);
    char* vertex_description(vertex_l_item *pt);
    char* vertex_action(vertex_l_item *pt);
    sigma_list_item* get_f_st_change(vertex_l_item *pt);
    sigma_list_item* get_n_st_change(vertex_l_item *pt, sigma_list_item *j);
    char* st_change_name(vertex_l_item *pt, sigma_list_item *j);
    char* stripped_st_ch(vertex_l_item *pt, sigma_list_item *j);

    sigma_list_item* get_f_param(vertex_l_item *pt);
    sigma_list_item* get_n_param(vertex_l_item *pt, sigma_list_item *j);
    char* param_name(vertex_l_item *pt, sigma_list_item *j);

/***********************************************************************
        functions for retrieving the edges from a input-file
***********************************************************************/

    edge_l_item* first_edge();
    edge_l_item* next_edge(edge_l_item *pt);
    char* edge_name(edge_l_item *pt);
    char* edge_description(edge_l_item *pt);
    char* edge_action(edge_l_item *pt);
    sort edge_sort(edge_l_item *pt);
    char* edge_origin(edge_l_item *pt);
    char* edge_destination(edge_l_item *pt);
    char* edge_condition(edge_l_item *pt);
    char* edge_delay(edge_l_item *pt);
    int edge_priority(edge_l_item *pt);
    sigma_list_item* get_f_attr(edge_l_item *pt);
    sigma_list_item* get_n_attr(edge_l_item *pt, sigma_list_item *j);
    char* attrib_name(edge_l_item *pt, sigma_list_item *j);

};

#endif
