////////////////////////////////////////////////////////////////////////
//                                                                    //
// default.c                                                          //
//                                                                    //
// the implementation of the default class, it provides members for   //
// reading the default section of a SIGMA input file and retrieving   //
// each separate part from this section.                              //
//                                                                    //
// (c) 1995 A.R. van der Valk, A. Eliens                              //
//                                                                    //
////////////////////////////////////////////////////////////////////////

#include <hush/dv/sigma/default.h>
#include <hush/dv/sigma/list.h>

#include <stdlib.h>
#include <string.h>
#include <iostream.h>
#include <fstream.h>
#include <strstream.h>
#include <iomanip.h>


// constructor for the default class, initialize all default features
	
defaults::defaults()
{
    name = NULL;
    description = NULL;
    declaration = NULL;
    action = 0;
    outputFile = NULL;
    trace_vars = new sigma_list();
    seed = 0;
    initial_values = new sigma_list();
    ending_condition = INITIAL;
    stop_event = NULL;
    ending_condition_nr = 0.0;
    trace_events = new sigma_list();
}


// read the default section from the input-file

int defaults::read_defaults(istream& s, int* cont)
{
    const bufsize = 1024;
    char inBuf[bufsize];
    char *hulp_buffer;
    int go_on = 1;
    int i = 0;
    int j = 0;

    hulp_buffer = new char[bufsize];

    while ( *cont || (
	    ((strcasecmp(inBuf,"STATE") != 0) 
	     && (strcasecmp(inBuf,"Name:") != 0)
	     && (strcasecmp(inBuf,"Var:") != 0)
	     && (strcasecmp(inBuf,"Variable:") != 0)
	     && (strcasecmp(inBuf,"State:") != 0))
	     && (s >> setw(bufsize) >> inBuf)))
    {
        if ((strcasecmp(inBuf,"Name:") == 0) 
	    || (strcasecmp(inBuf,"Model:") == 0))
        {
            s >> setw(bufsize) >> inBuf;
            name = new char[bufsize];
            strcpy(name, inBuf);
        }
        if (strcasecmp(inBuf,"Description:") == 0)
        {
            description = new char[bufsize];
            while (go_on)
            {
                s >> setw(bufsize) >> inBuf;
                if ((strcasecmp("Output", inBuf) == 0)
		    || (strcasecmp("File:", inBuf) == 0)
		    || (strcasecmp("Mode:", inBuf) == 0)
		    || (strcasecmp("Declaration:", inBuf) == 0)
		    || (strcasecmp("Action:", inBuf) == 0)
		    )
                    go_on = 0;
                else
                {
                    strcat(description, inBuf);
                    strcat(description, " ");
                }
            }
            go_on = 1;
        }
        if (strcasecmp(inBuf,"Declaration:") == 0)
        {
            declaration = new char[bufsize];
            while (go_on)
            {
                s >> setw(bufsize) >> inBuf;
                if ((strcasecmp("Output", inBuf) == 0)
		    || (strcasecmp("File:", inBuf) == 0)
		    || (strcasecmp("Vars:", inBuf) == 0)
		    || (strcasecmp("Mode:", inBuf) == 0)
		    || (strcasecmp("Action:", inBuf) == 0)
		    )
                    go_on = 0;
                else
                {
                    strcat(declaration, inBuf);
                    strcat(declaration, " ");
                }
            }
            go_on = 1;
        }
        if (strcasecmp(inBuf,"Action:") == 0)
        {
            action = new char[bufsize];
            while (go_on)
            {
                s >> setw(bufsize) >> inBuf;
                if ((strcasecmp("Output", inBuf) == 0)
		    || (strcasecmp("File:", inBuf) == 0)
		    || (strcasecmp("Vars:", inBuf) == 0)
		    || (strcasecmp("Mode:", inBuf) == 0)
		    )
                    go_on = 0;
                else
                {
                    strcat(action, inBuf);
                    strcat(action, " ");
                }
            }
            go_on = 1;
        }
        if (strcasecmp(inBuf,"File:") == 0)
        {
            s >> setw(bufsize) >> inBuf;
            outputFile = new char[bufsize];
            strcpy(outputFile,inBuf);
        }
        if (strcasecmp(inBuf,"Vars:") == 0)
        {
            s >> setw(bufsize) >> inBuf;
            while (inBuf[i] != '\0') 
            {
                switch (inBuf[i])
                {
		    case ';' :
                    case ',' : trace_vars->add(hulp_buffer);
                               hulp_buffer = new char[bufsize];
                               j = 0;
                               break;
                    default  : *(hulp_buffer + j) = inBuf[i];
                               if (inBuf[i+1] == '\0') // er is 1 var
                                   trace_vars->add(hulp_buffer);

                               j++;
                               break;
                }
                i++;
            }
        }
        if (strcasecmp(inBuf,"Seed:") == 0)
        {
            s >> setw(bufsize) >> inBuf;
            seed = atoi(inBuf);
        }
        if (strcasecmp(inBuf,"Values:") == 0)
        {
            i=j=0;
            hulp_buffer = new char[bufsize];
            s >> setw(bufsize) >> inBuf;
            if (strcasecmp(inBuf, "Ending") != 0)
            {
            while (inBuf[i] != '\0') 
            {
                cout << "in while bij initial values" << endl;
                switch (inBuf[i])
                {
                    case ',' : initial_values->add(hulp_buffer);
                               hulp_buffer = new char[bufsize];
                               j = 0;
                               break;
                    default  : *(hulp_buffer + j) = inBuf[i];
                               if (inBuf[i+1] == '\0') // er is 1 value
                                   initial_values->add(hulp_buffer);
                 
                               j++;
                               break;
                }
                i++;
            }
            }
        }
        if (strcasecmp(inBuf,"Condition:") == 0)
        {
            s >> setw(bufsize) >> inBuf;
            if (strcasecmp(inBuf, "STOP_ON_TIME") == 0)
                ending_condition = STOP_ON_TIME;
            else
                ending_condition = STOP_ON_EVENT;
        }
        // STOP_ON_TIME, keyword is "Time:"
        if (strcasecmp(inBuf,"Time:") == 0)
        {
	    ending_condition = STOP_ON_TIME;
            s >> setw(bufsize) >> inBuf;
            ending_condition_nr = atof(inBuf);
        }
        // STOP_ON_EVENT, keyword is "Event:"
        if (strcasecmp(inBuf,"Event:") == 0)
        {
	    ending_condition = STOP_ON_EVENT;
            s >> setw(bufsize) >> inBuf;
            stop_event = new char[bufsize]; 
            strcpy(stop_event,inBuf);

            while (strcasecmp("Run:",inBuf) != 0) 
                (s >> setw(bufsize) >> inBuf);

            s >> setw(bufsize) >> inBuf; //inBuf bevat nu aantal keren
                                         // dat event moet optreden
            ending_condition_nr = atof(inBuf);
        }
        if ( (strcasecmp(inBuf,"Events:") == 0)
	     || (strcasecmp(inBuf,"Trace:") == 0)
	     )
        {
            i=j=0;
            hulp_buffer = new char[bufsize];
            s >> setw(bufsize) >> inBuf;
            while (inBuf[i] != '\0') 
            {
                switch (inBuf[i])
                {
                    case ',' : trace_events->add(hulp_buffer);
                               j = 0;
                               hulp_buffer = new char[bufsize];
                               break;
                    default:   *(hulp_buffer + j) = inBuf[i];
                               if (inBuf[i+1] == '\0') // er is 1 value
                                   trace_events->add(hulp_buffer);
                               
                               j++;
                               break;
                }
                i++;
            }
        }
    }            
    if ((strcasecmp(inBuf,"State:") == 0)
         || (strcasecmp(inBuf,"Name:") == 0)
         || (strcasecmp(inBuf,"Variable:") == 0)
         || (strcasecmp(inBuf,"Var:") == 0))
		cerr << "Defaults: Got " << inBuf << endl;
                *cont = 1; // skip Header

    return(1);
}


// display the default section on screen

void
defaults::display_defaults()
{
    cout << "MODEL DEFAULTS" << endl << endl;
    cout << "name: " << name << endl;
    cout << "description: "<< description << endl;
    cout << "declaration: "<< declaration << endl;
    cout << "action: "<< action << endl;
    cout << "file: "<< outputFile << endl;
    trace_vars->display();
    cout << "seed: " << seed << endl;
    initial_values->display();
    cout << "condition: " << ending_condition << endl;
    if (ending_condition == STOP_ON_EVENT)
        cout << stop_event << " " << "en wel "; 
    cout << "ending Cond. nr. : " << ending_condition_nr << " keer" << endl;
    trace_events->display();
}
    

// the destructor

defaults::~defaults()
{

    name = NULL;
    description = NULL;
    declaration = NULL;
    action = NULL;
    outputFile = NULL;
    trace_vars->~sigma_list();
    seed = 0;
    initial_values->~sigma_list();
    ending_condition = INITIAL;
    stop_event = NULL;
    ending_condition_nr = 0.0;
    trace_events->~sigma_list();
}


// retrieve the first traced variable from the list

sigma_list_item*
defaults::get_first_trace_var()
{
    sigma_list_item *i = trace_vars->get_front();
    if (i == NULL)
        return (NULL);
    else
        return (i);
}


// retrieve the next traced variable from the list

sigma_list_item*
defaults::get_next_trace_var(sigma_list_item *i)
{
    sigma_list_item *j = trace_vars->get_next(i);
    if (j == NULL)
        return (NULL);
    else
        return (j);
}


// retrieve the name from the traced variable

char*
defaults::trace_var_name(sigma_list_item *pt)
{
    return (trace_vars->get_val(pt));
}


// retrieve the stripped version of a variable, for example
// W[IN] becomes WIN

char*
defaults::get_stripped(sigma_list_item *pt)
{

    int j = 0;
    int lengte = strlen(trace_vars->get_val(pt));
    char *buf = new char[lengte];
    strncpy(buf, trace_vars->get_val(pt), lengte);
    char *hulp = new char[1024];

    for (int i = 0; i < lengte; i++)
    {
        if ((buf[i] != '[') && (buf[i] != ']')) 
        {
            *(hulp + j) = buf[i];
            j++;
        }
    }
    return (hulp);
}


// retrieve the first initial value given in the input-file

sigma_list_item*
defaults::get_first_initial_val()
{
    sigma_list_item *i = initial_values->get_front();
    if (i == NULL)
    {
        //cout << "front is null bij init values " << endl;
        return (NULL);
    }
    else
        return (i);
}


// retrieve the next initial value given in the input-file

sigma_list_item*
defaults::get_next_initial_val(sigma_list_item *i)
{
    sigma_list_item *j = initial_values->get_next(i);
    if (j == NULL)
        return (NULL);
    else
        return (j);
}


// retrieve the real value with the use of a pointer to a initial value

double
defaults::return_value(sigma_list_item *i)
{
    return (atof(initial_values->get_val(i)));
}


// retrieve the first traced event

char*
defaults::get_first_trace_event()
{
    sigma_list_item *i = trace_events->get_front();
    if (i == NULL)
        return (NULL);
    else
        return (trace_events->get_val(i));
}


// retrieve the next traced event

char*
defaults::get_next_trace_event(sigma_list_item *i)
{
    sigma_list_item *j = trace_events->get_next(i);
    if (j == NULL)
        return (NULL);
    else
        return (trace_events->get_val(j));
}
