////////////////////////////////////////////////////////////////////////
//                                                                    //
// statevar.c                                                         //
//                                                                    //
// the implementation of the state variable class, it provides        //
// members for creating a list of state variables. Also, members      //
// are provided for retrieving each separate state variable and its   //
// features.                                                          //
//                                                                    //
// (c) 1995 A.R. van der Valk, A. Eliens                              //
//                                                                    //
////////////////////////////////////////////////////////////////////////

#include <hush/dv/sigma/statevar.h>

#include <stdlib.h>
#include <string.h>
#include <iostream.h>
#include <fstream.h>
#include <strstream.h>
#include <iomanip.h>
#include <stdio.h>

#define NULL 0


// destructor of the list of state variables

state_var_l::~state_var_l()
{
    while (is_empty() != true)
        remove();
}


// removes the first element of the state variable list

void
state_var_l::remove()
{
    if (is_empty() == true)
    {
        cout << "remove() on empty list \n";
        exit (-1);
    }
   
    st_var_l_item *pt = front;
    front = front->next;

    pt->val.name = NULL;
    pt->val.description = NULL;
    pt->val.code = NULL;
    pt->val.type = INTEGER;
    pt->val.var_size = NULL;

    delete pt;
}


// add an element at the end of the state variable list

void
state_var_l::add(state_var_type item) {
 
    st_var_l_item *pt = new st_var_l_item(item);

    if (is_empty())
        front = back = pt;
    else
    {
        back->next = pt;
        back = pt;
    }
}


// display the state variables on screen

void
state_var_l::display_state_vars()
{
    cout << "STATE VARIABLES" << endl << endl;;

    for (st_var_l_item *pt=front; pt; pt=pt->next)
    {
        cout << pt->val.name << endl;
        cout << pt->val.description << endl;
        cout << pt->val.code << endl;
        cout << pt->val.type << endl;
        cout << pt->val.var_size << endl;
    }
}
    

// read the state variables from the SIGMA input-file into the
// state variables list

int
state_var_l::read_state_vars(istream& s, int* cont)
{
    const bufsize = 1024;
    char inBuf[bufsize];
    state_var_type hulp_var;
    int go_on = 1;


    // lezen totdat state variablen gedeelte begint
    //while ((strcasecmp(inBuf,"VARIABLES") != 0) && (s >> setw(bufsize) >> inBuf));

    if ( *cont == 1 ) strcpy(inBuf,"Name:");

    while ( !*cont && (s >> setw(bufsize) >> inBuf)) {
	if ((strcasecmp(inBuf,"Variables") == 0) 
	    || (strcasecmp(inBuf,"Variable:") == 0)
	    || (strcasecmp(inBuf,"Name:") == 0)
	    )
		*cont = 1;
	}

    while (*cont || (
     ((strcasecmp(inBuf,"VERTICES") != 0)
      && (strcasecmp(inBuf,"Vertex:") != 0)
      && (strcasecmp(inBuf,"Node:") != 0))
     && (s >> setw(bufsize) >> inBuf)))
    {
	    *cont = 0;
	    //cerr << "Processing state:" << inBuf << endl;
            if ((strcasecmp(inBuf, "Name:") == 0)
		|| (strcasecmp(inBuf, "Var:") == 0)
		|| (strcasecmp(inBuf, "Variable:") == 0))
            {
                s >> setw(bufsize) >> inBuf;
                hulp_var.name = new char[bufsize];
                strcpy(hulp_var.name, inBuf);
            }
            if ((strcasecmp(inBuf, "Description:") == 0)
		|| (strcasecmp(inBuf, "Desc:") == 0)
		|| (strcasecmp(inBuf, "Info:") == 0))
            {
                hulp_var.description = new char[bufsize];
                while (go_on)
                {
                    s >> setw(bufsize) >> inBuf;
                    if ((strcasecmp(inBuf,"Type:") == 0)
			|| (strcasecmp(inBuf,"Declaration:") == 0))
                        go_on = 0;
                    else
                    {
                        strcat(hulp_var.description, inBuf);
                        strcat(hulp_var.description, " ");
                    }
                }
                go_on = 1;
            }
            if ((strcasecmp(inBuf, "Declaration:") == 0)
		|| (strcasecmp(inBuf, "Code:") == 0))
            {
		hulp_var.type = USER;
                hulp_var.code = new char[bufsize];
                while (go_on)
                {
                    s >> setw(bufsize) >> inBuf;
                    if ((strcasecmp(inBuf,"Type:") == 0)
			|| (strcasecmp(inBuf,"Size:") == 0))
                        go_on = 0;
                    else
                    {
                        strcat(hulp_var.code, inBuf);
                        strcat(hulp_var.code, " ");
                    }
                }
                go_on = 1;
            }
            if (strcasecmp(inBuf, "Type:") == 0)
            {
                s >> setw(bufsize) >> inBuf;
                if ((strcasecmp(inBuf, "INTEGER") == 0)
		    || (strcasecmp(inBuf, "Int:") == 0))
                    hulp_var.type = INTEGER;
                else if (strcasecmp(inBuf, "REAL") == 0)
                    hulp_var.type = REAL;
                else
                    hulp_var.type = USER;
            }
            if (strcasecmp(inBuf, "Size:") == 0)
            {
                s >> setw(bufsize) >> inBuf;
                hulp_var.var_size = new char[bufsize];
                strcpy(hulp_var.var_size, inBuf);
                add(hulp_var);
            }
    }
    if ((strcasecmp(inBuf,"Vertex:") == 0)
	 || (strcasecmp(inBuf,"Node:") == 0))
		*cont = 1; // skip Header

    return (1);
}


// constructor of a state variable list element

st_var_l_item::st_var_l_item(state_var_type value) 
{
    val.name = new char[1024];
    sprintf(val.name,"anonymous");
    val.description = new char[1024];
    sprintf(val.description,"unknown");
    val.code = new char[1024];
    sprintf(val.code,"// declaration");
    val.type = INTEGER;
    val.var_size = new char[1024];
    sprintf(val.var_size,"1");

    if (value.name) strcpy(val.name,value.name);
    if (value.description) strcpy(val.description,value.description);
    //if (value.code) strcpy(val.code,value.code);
    val.type = value.type;
    if (value.var_size) strcpy(val.var_size,value.var_size); 

    next = 0;
}


// search in the state variable list for a certain element

st_var_l_item*
state_var_l::search(char *s)
{
    st_var_l_item *pt = front;
    while (pt != NULL)
    {
        if (strcasecmp(pt->val.name, s) == 0)
            return (pt);

        pt = pt->next;
    }
    return (NULL);
};


// return zero if the state variable is no array, otherwise the size
// of the array

int
state_var_l::sv_array(st_var_l_item *pt)
{
    if (strstr(pt->val.name, ",") == NULL)
        if (atoi(pt->val.var_size) > 1)
            return (atoi(pt->val.var_size));

    return (0);
}
