////////////////////////////////////////////////////////////////////////
//                                                                    //
// statevar.h                                                         //
//                                                                    //
// provides a list to contain the state variables. Elements consist   //
// of fields specifying the name, description, type and size of the   //
// state variable. Members are available for reading and displaying   //
// the list and for retrieving the different state variable parts.    //
//                                                                    //
// (c) 1995 A.R. van der Valk, A. Eliens                              //
//                                                                    //
////////////////////////////////////////////////////////////////////////


#ifndef STATEVAR_H
#define STATEVAR_H

#include <string.h>
#include <hush/dv/sigma/list.h>

typedef enum {INTEGER,REAL,USER} var_type;

struct state_var_type
{
    state_var() { name=0; description=0;code=0; type=INTEGER; }
    char *name;                        
    char *description;                
    char* code;
    var_type type;                   
    char *var_size;                 
};


class st_var_l_item 
{
friend class state_var_l;

private:
    st_var_l_item(state_var_type); 
    state_var_type val;
    st_var_l_item *next;
};


class state_var_l 
{
private:
    st_var_l_item *front;
    st_var_l_item *back;
    
public:
    state_var_l() {front = back = 0; }  // constructor
    ~state_var_l();
    void remove();
    void add(state_var_type);
    void display_state_vars();
    bool is_empty() { return front == 0 ? true : false; }
    int read_state_vars(istream& s, int* x = 0); // inlezen v/d toestandvars.
    st_var_l_item* return_front() { return front; }
    st_var_l_item* return_next(st_var_l_item *pt) { return pt->next; }
    char* return_name(st_var_l_item *pt) { return pt->val.name; }
    char* return_description(st_var_l_item *pt) { return pt->val.description; }
    char* return_declaration(st_var_l_item *pt) { return pt->val.code; }
    var_type return_type(st_var_l_item *pt) { return pt->val.type; }
    char* return_size(st_var_l_item *pt) {return pt->val.var_size; }
    
    st_var_l_item* search(char *s);
    int sv_array(st_var_l_item *pt);
};

#endif
