////////////////////////////////////////////////////////////////////////
//                                                                    //
// vertex.c                                                           //
//                                                                    //
// the implementation of the vertex class, it provides members for    //
// creating a list of vertices. Also, members are provided for        //
// retrieving each separate vertex and its features.                  //
//                                                                    //
// (c) 1995 A.R. van der Valk, A. Eliens                              //
//                                                                    //
////////////////////////////////////////////////////////////////////////

#include <hush/dv/sigma/vertex.h>
#include <hush/dv/sigma/list.h>

#include <stdlib.h>
#include <string.h>
#include <iostream.h>
#include <fstream.h>
#include <strstream.h>
#include <iomanip.h>


// destructor of the vertex list

vertex_list::~vertex_list()
{
    while (is_empty() != true)
        remove();
}


// removes the first element from the vertex list

void
vertex_list::remove()
{
    if (is_empty() == true)
    {
        cout << "remove() on empty list\n";
        exit(-1);
    }
    
    vertex_l_item *pt = front;
    front = front->next;

    pt->val.name = NULL;
    pt->val.description = NULL;
    pt->val.action = NULL;
    pt->val.state_changes->~sigma_list();
    pt->val.parameters->~sigma_list();
    pt->val.coordinate.x = 0;
    pt->val.coordinate.y = 0;
    pt->val.coordinate.z = 0;
        
    delete pt;
}


// add an element at the end of the vertex list

void
vertex_list::add(vertex_type item) {
 
    vertex_l_item *pt = new vertex_l_item(item);

    if (is_empty())
        front = back = pt;
    else
    {
        back->next = pt;
        back = pt;
    }
}


// display the vertices on screen

void
vertex_list::display_vertices()
{
    cout << "VERTICES" << endl << endl;

    for (vertex_l_item *pt=front; pt; pt=pt->next)
    {
        cout << "Name: " ;
        cout << pt->val.name << " " << endl;
        cout << "Description: " ;
        cout << pt->val.description << " " << endl;
        cout << "Action: " ;
        cout << pt->val.action << " " << endl;
        cout << "State Changes: " ;
        pt->val.state_changes->display();
        cout << endl;
        cout << "Parameters: ";
        pt->val.parameters->display();
        cout << endl;
        cout << "Coordinates: ";
        cout << pt->val.coordinate.x << " ";
        cout << pt->val.coordinate.y << " ";
        cout << pt->val.coordinate.z << " ";
        cout << endl;
    }
}
    

// read the vertices from the SIGMA input-file into the vertex list

int
vertex_list::read_vertices(istream& s, int* cont)
{
    const bufsize = 1024;
    char inBuf[bufsize];
    vertex_type hulp_var;
    char *hulp_buffer;
    int go_on = 1;
    int i=0, j=0;

    hulp_buffer = new char[bufsize];

    if (*cont == 1 ) strcpy(inBuf,"Vertex:");
    //  lezen totdat verices gedeelte begint
    //while ((strcmp(inBuf,"Vertex") != 0) && (s >> setw(bufsize) >> inBuf));
    while ( !*cont && (s >> setw(bufsize) >> inBuf)) {
	//cerr << "reading " << inBuf << endl;
	if ((strcasecmp(inBuf,"Vertex") == 0) 
	    || (strcasecmp(inBuf,"Vertex:") == 0)
	    || (strcasecmp(inBuf,"Node:") == 0)
	    || (strcasecmp(inBuf,"Name:") == 0)
	    ) {
		*cont = 1;
		cerr << "Got " << inBuf << endl;
		}
	}


    while ( *cont || (
	    ((strcasecmp(inBuf,"EDGES") != 0)
	    && (strcasecmp(inBuf,"Arrows") != 0)
	    && (strcasecmp(inBuf,"Arrow:") != 0)
	    && (strcasecmp(inBuf,"Edge:") != 0))
	    && (s >> setw(bufsize) >> inBuf)))
    {
	cerr << "Processing vertices" << endl;
	*cont = 0;
        if ( (strcasecmp(inBuf, "Name:") == 0)
	   || (strcasecmp(inBuf, "Vertex:") == 0)
	   || (strcasecmp(inBuf, "Node:") == 0))
        {
            s >> setw(bufsize) >> inBuf;
            hulp_var.name = new char[bufsize];
            strcpy(hulp_var.name, inBuf);
	    cerr << "Name: " << inBuf << endl;
        }
        if ((strcasecmp(inBuf, "Description:") == 0)
	    || (strcasecmp(inBuf, "Desc:") == 0)
	    || (strcasecmp(inBuf, "Info:") == 0))
        {
            hulp_var.description = new char[bufsize];
            while (go_on)
            {
                s >> setw(bufsize) >> inBuf;
                if ((strcasecmp(inBuf,"State") == 0)
		    || (strcasecmp(inBuf,"State:") == 0)
		    || (strcasecmp(inBuf,"Change:") == 0)
		    || (strcasecmp(inBuf,"Changes:") == 0)
		    || (strcasecmp(inBuf,"Action:") == 0))
                    go_on = 0;
                else
                {
                    strcat(hulp_var.description, inBuf);
                    strcat(hulp_var.description, " ");
                }
            }
            go_on = 1;
        }
        if (strcasecmp(inBuf, "Action:") == 0)
        {
            hulp_var.action = new char[bufsize];
            hulp_var.action[0] = '\0';
            while (go_on)
            {
                s >> setw(bufsize) >> inBuf;
                if (
		    (strcasecmp(inBuf,"State") == 0) 
		    || (strcasecmp(inBuf,"Changes:") == 0)
		    || (strcasecmp(inBuf,"Change:") == 0)
		    || (strcasecmp(inBuf,"State:") == 0))
                    go_on = 0;
                else
                {
                    strcat(hulp_var.action, inBuf);
                    strcat(hulp_var.action, " ");
                }
            }
            go_on = 1;
        }
        if ((strcasecmp(inBuf,"Changes:") == 0)
	    || (strcasecmp(inBuf,"Change:") == 0)
	    || (strcasecmp(inBuf,"State:") == 0))
        {
            hulp_var.state_changes = new sigma_list();
            s >> setw(bufsize) >> inBuf;
            if ((strcasecmp(inBuf, "Parameter(s):") != 0)
		|| (strcasecmp(inBuf, "Parameters:") != 0)
		|| (strcasecmp(inBuf, "Parameter:") != 0)
		|| (strcasecmp(inBuf, "Param(s):") != 0))
            {
            while (inBuf[i] != '\0')
            {
                switch (inBuf[i])
                {
		    case ';' :
                    case ',' : hulp_var.state_changes->add(hulp_buffer);
                               hulp_buffer = new char[bufsize];
                               j = 0;
                               break;
                    default  : *(hulp_buffer + j) = inBuf[i];
                               if (inBuf[i+1] == '\0') // er is 1 var
                                   hulp_var.state_changes->add(hulp_buffer);
                               j++;
                               break;
   
                }
                i++;
             }
             }
        }
           
        i=j=0;
        hulp_buffer = new char[bufsize];
        if ((strcasecmp(inBuf,"Parameter(s):") == 0) 
	    || (strcasecmp(inBuf,"Param(s):") == 0)
	    || (strcasecmp(inBuf,"Parameter:") == 0)
	    || (strcasecmp(inBuf,"Parameters:") == 0))
        {
            hulp_var.parameters = new sigma_list();
            s >> setw(bufsize) >> inBuf;
            if (strcasecmp(inBuf,"Location:") != 0)
            {

                 while (inBuf[i] != '\0')
                 {
                     switch (inBuf[i])
                     {
                         case ',' : hulp_var.parameters->add(hulp_buffer);
                                    hulp_buffer = new char[bufsize];
                                    j = 0;
                                    break;
                         default  : *(hulp_buffer + j) = inBuf[i];
                                    if (inBuf[i+1] == '\0') // er is 1 var
                                     hulp_var.parameters->add(hulp_buffer);
                                     j++;
                                     break;
       
                     }
                     i++;
                 }
             }
             else
             {
                 while ((s >> setw(bufsize) >> inBuf) && (strcasecmp(inBuf,"X:") != 0)); 
                 s >> setw(bufsize) >> inBuf;
                 hulp_var.coordinate.x = atof(inBuf);
                 while ((s >> setw(bufsize) >> inBuf) && (strcasecmp(inBuf,"Y:") != 0)); 
                 s >> setw(bufsize) >> inBuf;
                 hulp_var.coordinate.y = atof(inBuf);
                 while ((s >> setw(bufsize) >> inBuf) && (strcasecmp(inBuf,"Z:") != 0)); 
                 s >> setw(bufsize) >> inBuf;
                 hulp_var.coordinate.z = atof(inBuf);
                 add(hulp_var);

             }
         }
         if (strcasecmp(inBuf, "Location:") == 0)
         {
             while ((s >> setw(bufsize) >> inBuf) && (strcasecmp(inBuf,"X:") != 0)); 
             s >> setw(bufsize) >> inBuf;
             hulp_var.coordinate.x = atof(inBuf);
             while ((s >> setw(bufsize) >> inBuf) && (strcasecmp(inBuf,"Y:")!= 0)); 
             s >> setw(bufsize) >> inBuf;
             hulp_var.coordinate.y = atof(inBuf);
             while ((s >> setw(bufsize) >> inBuf) && (strcasecmp(inBuf,"Z:") != 0)); 
             s >> setw(bufsize) >> inBuf;
             hulp_var.coordinate.z = atof(inBuf);
             add(hulp_var);
         }
     }
	 if ((strcasecmp(inBuf,"Arrow:") == 0)
		  || (strcasecmp(inBuf,"Edge:") == 0))
				  *cont = 1; // skip Header

     return (1);
}


//constructor of a vertex list element

vertex_l_item::vertex_l_item(vertex_type value) 
{
    val.name = new char[1024];
    sprintf(val.name,"anonymous");
    val.description = new char[1024];
    sprintf(val.description,"unknown");
    val.action = new char[1024];
    sprintf(val.action,"// action");
    val.state_changes = new sigma_list();
    val.parameters = new sigma_list();
    val.coordinate.x = 0.0;
    val.coordinate.y = 0.0;
    val.coordinate.z = 0.0;

    if (value.name) strcpy(val.name,value.name);
    if (value.description) strcpy(val.description,value.description);
    if (value.action) strcpy(val.action,value.action);
    val.state_changes = value.state_changes;
    val.parameters = value.parameters;
    val.coordinate.x = value.coordinate.x;
    val.coordinate.y = value.coordinate.y;
    val.coordinate.z = value.coordinate.z;

    next = 0;
}


// retrieve the first state change of a vertex

sigma_list_item*
vertex_list::get_first_st_change(vertex_l_item *pt)
{
    sigma_list_item *i = pt->val.state_changes->get_front();
    if (i == NULL)
        return (NULL);
    else
        return (i);
}
    

// retrieve the next state change of a vertex

sigma_list_item*
vertex_list::get_next_st_change(vertex_l_item *pt, sigma_list_item *j)
{
    sigma_list_item *i = pt->val.state_changes->get_next(j);
    if (i == NULL)
        return (NULL);
    else
        return (i);
}


// retrieve the name of the vertex state change

char*
vertex_list::get_st_change_name(vertex_l_item *pt, sigma_list_item *j)
{
    return (pt->val.state_changes->get_val(j));
}


// retrieve the first parameter of a vertex

sigma_list_item*
vertex_list::get_first_param(vertex_l_item *pt)
{
    sigma_list_item *i = pt->val.parameters->get_front();
    if (i == NULL)
        return (NULL);
    else
        return (i);
}
    

// retrieve the next parameter of a vertex

sigma_list_item*
vertex_list::get_next_param(vertex_l_item *pt, sigma_list_item *j)
{
    sigma_list_item *i = pt->val.parameters->get_next(j);
    if (i == NULL)
        return (NULL);
    else
        return (i);
}


// retrieve the parameter name 

char*
vertex_list::get_param_name(vertex_l_item *pt, sigma_list_item *j)
{
    return (pt->val.parameters->get_val(j));
}


// return the useful C++ version of a state change. This means a `&'
// is replaced by a `&&' etc.

char*
vertex_list::return_stripped(vertex_l_item *pt, sigma_list_item *i)
{

    int j = 0;
    char *hulp = new char[1024];
    int lengte = strlen(pt->val.state_changes->get_val(i));
    char *buf = new char[lengte];
    strncpy(buf,pt->val.state_changes->get_val(i),lengte);

    for (int k = 0; k < lengte; k++)
    {
        switch (buf[k])
        {
            case '{': 
              *(hulp + j) = '(';
              break;
            case '}':
              *(hulp + j) = ')';
              break;
            case ';':
              *(hulp + j) = ',';
              break;
            case '&':
              *(hulp + j) = '&';
              if (buf[k+1]!='&') j++;
              *(hulp + j) = '&';
              break;
            case '|':
              *(hulp + j) = '|';
              if (buf[k+1]!='|') j++;
            break;
            default:
              *(hulp + j) = buf[k];
              break;
        }
        j++;
    }
    return (hulp);
}
