////////////////////////////////////////////////////////////////////////
//                                                                    //
// edge.h                                                             //
//                                                                    //
// contains a list to hold the edges. Elements consist of a struct    //
// with field to represent the edges descriped in the SIGMA input-    //
// file. Members are available for reading, displaying and retrieving //
// the whole edge or just a field.                                    //
//                                                                    //
// (c) 1995 A.R. van der Valk, A. Eliens                              //
//                                                                    //
////////////////////////////////////////////////////////////////////////


#ifndef VERTEX_H
#define VERTEX_H

#include <hush/dv/sigma/list.h>

struct coordinate_type
{
    double x, y, z;
};


struct vertex_type 
{
    vertex_type() { name = 0; description = 0; action = 0; }
    char *name;         
    char *description;  
    char *action;
    sigma_list *state_changes; 
    sigma_list *parameters;
    coordinate_type coordinate;
};

    
class vertex_l_item 
{
friend class vertex_list;

private:
    vertex_l_item(vertex_type);
    vertex_type val;
    vertex_l_item *next;
};


class vertex_list 
{
private:
    vertex_l_item *front;
    vertex_l_item *back;
    
public:
    vertex_list() {front = back = 0; }
    ~vertex_list();
    void remove();
    void add(vertex_type);
    void display_vertices();
    bool is_empty()
    { return front == 0 ? true : false; }

    int read_vertices(istream& s, int* x = 0);

    vertex_l_item* return_front() {return front;}
    vertex_l_item* return_next(vertex_l_item *pt) {return pt->next;}

    char* return_name(vertex_l_item *pt) {return pt->val.name;}
    char* return_description(vertex_l_item *pt) {return pt->val.description;}
    char* return_action(vertex_l_item *pt) {return pt->val.action;}
    sigma_list_item* get_first_st_change(vertex_l_item *pt);
    sigma_list_item* get_next_st_change(vertex_l_item *pt, sigma_list_item *j);
    char* get_st_change_name(vertex_l_item *pt, sigma_list_item *j);

    /* returns the state_change in C++ language. */
    char* return_stripped(vertex_l_item *pt, sigma_list_item *i);

    sigma_list_item* get_first_param(vertex_l_item *pt);
    sigma_list_item* get_next_param(vertex_l_item *pt, sigma_list_item *j);
    char* get_param_name(vertex_l_item *pt, sigma_list_item *j);
};

#endif
