// kit.c  -- (no session.c because of namemangling conflicts)
// Rename into tclkit.c for (some) Cfront compilers -- thanks to Richard A. Rausch

#include <hush/dv/tk/tcl/kit.h>
#include <hush/dv/tk/tcl/tcl.h>

#include <hush/sys/bind.h>
#include <hush/widget.h>
#include <hush/event.h>

#include <hush/assert.h>

#include <hush/flags.h>
#include <hush/sys/port.h>

#include <hush/dv/tk/tcl/event.h>
#include <hush/dv/tk/tcl/file.h>

#include <stdlib.h>
#include <string.h>
//#include <iostream.h>
#include <hush/io.h>
//#define endl "\n"

/// Change this for a different tk file

extern "C" {
#include <tcl.h>
}

//struct TkWindow;

void init_tk(kit*); /// defined below

#undef  INTERP
#define INTERP (Tcl_Interp*) _interp

//extern int tk_tcl_appinit(kit*);
//static char _buf[BUFSIZ];
//static char hush_library[BUFSIZ];
//static char tk_tcl_library[BUFSIZ];
//static char tk_library[BUFSIZ];

int tk_tcl_kit::_no = 0;

//// tk_tcl_callbacks

static tk_tcl_callback* active_callback;

static int call_back(ClientData c, Tcl_Interp*, int argc, char** argv) {
tk_tcl_callback* x = ((tk_tcl_callback*) c);
active_callback = x; 
int result =  (*((hushcommand*)x->proc))(x->data, x->tk,argc,argv);
return result;
}

//// The tcl kit
	
tk_tcl_kit::tk_tcl_kit(void* x) : kit(this) {
        _interp = (Tcl_Interp*) x;
	}

tk_tcl_kit::tk_tcl_kit() : kit(this) {
	_trace = 0;
	kit* x = kit::lookup("tcl");
	if (!x) x = kit::lookup("tk");
	if (_no==0) {
		if (x)
			_interp = x->interp();
		else
			_interp = dvtcl::create();
		_no++;
		}
	else {
		require( x );
		if (tracelevel() > 6) err() << "interp" << ++_no << endl;
		_interp = x->interp();
		}

	buf = new char[4*1024];
	_root = 0;
	promise ( _interp );
}

tk_tcl_kit::~tk_tcl_kit() {
	if (buf) delete buf;
	//dvtcl::_delete(INTERP);
	}

void tk_tcl_kit::_destroy() {
	dvtcl::_delete(INTERP);
	}


long tk_tcl_kit::eval(const char* cmd) {
	if (tracelevel()>0) err() << (char*) cmd << endl;
	int result = dvtcl::eval(_interp,cmd); //Tcl_Eval(INTERP,(char*)cmd);
	if (result == 1) 
	  err() << "WARNING: " << cmd << " evaluates to FALSE" << endl;
	return (long) result;
	}

char* tk_tcl_kit::evaluate(const char* cmd) {
	eval(cmd);
	return result();
	}

char* tk_tcl_kit::result(long) { return dvtcl::result(_interp); }

void tk_tcl_kit::result( const char* s) { dvtcl::result(_interp,s); }

int tk_tcl_kit::source(const char* f) { return dvtcl::source(_interp,f); }

int tk_tcl_kit::error() { return dvtcl::error(_interp); }

void tk_tcl_kit::error(const char* msg) { dvtcl::error(_interp,msg); }

void tk_tcl_kit::after(int t, const char* cmd) {
	sprintf(buf,"after %d %s",t,cmd);
	eval(buf);
	}

void tk_tcl_kit::after(int t, void* f, void* d) {
	_last = dvtcl::timer(_interp,t,f,d);
	}

void tk_tcl_kit::after(int t, handler* h) {
        if (!h) after(t,"");
        else {
   		struct tk_tcl_callback* x = new tk_tcl_callback();
   		x->tk = this;
   		x->proc = (void*) tk_tcl_event::callback;
   		x->data = h;
   		after(t, (void*) tk_tcl_event::timer, (void*) x);
           	}
	}
	

char* tk_tcl_kit::send(const char* it, const char* cmd) {
	sprintf(buf,"send %s %s",it,cmd);
        return evaluate(buf);
	}

char* tk_tcl_kit::selection(const char* options) const {
	sprintf(buf,"selection get %s", options);
	return ((tk_tcl_kit*)this)->evaluate( buf );
	}

void tk_tcl_kit::update(const char* options) { 
	sprintf(buf,"update %s", options);
	eval( buf );
	}


int tk_tcl_kit::safe() const {
	return dvtcl::safe(INTERP);
	}

kit* tk_tcl_kit::slave(char* options) {
	kit* res = 0;
	char cmd[BUFSIZ];
	sprintf(cmd,"interp create %s ",options);
	int error = this->eval( cmd );
	if (error) { 
		err() << "ERROR: CREATING INTERPRETER " << endl;
		res = this;
	} else {
	   void* x = dvtcl::slave(_interp, this->result()); 
	   if (x) res =  new tk_tcl_kit(x);
	   _register(res); 
	}
	return res;
}

binding* tk_tcl_kit::bind(handler* h) {
	return bind(gensym("tcl-anonymous"), h);
	}

binding* tk_tcl_kit::bind(void* proc, void* data) {
	return bind(gensym("anonymous"),proc,data);
	}

binding* tk_tcl_kit::bind(const char* name) {
	binding* tmp = new binding(this,name);
	_register(tmp);
	return tmp;
	}

binding* tk_tcl_kit::bind(const char* name, handler* h) {
	return bind(name, (void*) tk_tcl_event::callback, h );
	}

binding* tk_tcl_kit::bind(const char* name, void* proc, void* data) {
	struct tk_tcl_callback* x = new struct tk_tcl_callback(); //AE
	x->tk = this; x->proc = proc; x->data = data;
	command(name,(void*)call_back,(void*)x);
	binding* tmp = new binding(this,name);
	_register(tmp);
	return tmp;
	}

binding* tk_tcl_kit::bind(int fd, handler* h, int mask) {
    tk_tcl_file_callback *s = new tk_tcl_file_callback(h,fd);
    command(fd, (void*) tk_tcl_file_event::callback,(void*) s, mask);
    binding* tmp = new binding(this,"nop");
    _register(tmp);
    return tmp;
}

void tk_tcl_kit::command(const char* name, void* proc, void* data) {
	dvtcl::command(_interp,name,proc,data);
	}

void tk_tcl_kit::command(int fd, void* proc, void* data, int hushmask) {
        // jrvosse: hushmask is now more independent of Tcl:
	/*AE in tcl.c
        int tclmask = 0;
        if (hushmask & readable)  tclmask |= TCL_READABLE;
        if (hushmask & writable)  tclmask |= TCL_WRITABLE;
        if (hushmask & exception) tclmask |= TCL_EXCEPTION;
	*/
 
        dvtcl::command(_interp,fd,proc,data,hushmask);
}


void tk_tcl_kit::unbind(int fd) {
	dvtcl::unbind(_interp,fd);
	}

void tk_tcl_kit::unbind(char* n, char* s ) { // AE
	dvtcl::unbind(_interp,n,s);
	}

void tk_tcl_kit::quit() {
	eval("quit");
	err() << "Quitting .. " << "\n";
	session::_quit = 1;
	}

event* tk_tcl_kit::get() const {
	if (tracelevel() > 3 ) err() << "asking for event" << endl;
	event* x = (event*) _last;
#ifdef GNU
	return event( (event*) x );
#else
	return new event( (event*) x );
#endif
	}

void* tk_tcl_kit::token() const { return _last; }

void* tk_tcl_kit::interp() const { promise(_interp); return _interp; }


void tk_tcl_kit::put(event* p, int fl) {
	require( p );
	_last = p;
	_type = fl;
}

