#include <hush/dv/tk/tcl/tcl.h>
#include <tcl.h>

#ifndef NOLINKVARS
extern "C" {
// AE WARNING -- to avoid link problems
int Tcl_LinkVar (Tcl_Interp *, char*, char *, int ) { return 0; } // AE
void Tcl_UnlinkVar (Tcl_Interp *, char* ) { }
}
#endif

#include <hush/hush.h>
#include <hush/assert.h>
#include <hush/io.h>
//#define endl "\n"

#include <hush/flags.h>
//#include <iostream.h>
#include <string.h>
#include <hush/sys/unistd.h>
#include <stdlib.h>    // jrvosse for getenv()

#ifndef HUSH_LIBRARY
#define HUSH_LIBRARY "Please define HUSH_LIBRARY"
#endif

#ifndef TCL_LIBRARY
#define TCL_LIBRARY "Please define TCL_LIBRARY"
#endif

#ifndef TK_LIBRARY
#define TK_LIBRARY "Please define TK_LIBRARY"
#endif

extern "C" { /// must eliminate tk dependencies

#if (TCL_MAJOR_VERSION == 8 && TCL_MINOR_VERSION >= 0)
#undef MULTINTERP
#define MULTINTERP
#define Tk_DoWhenIdle Tcl_DoWhenIdle
#define Tk_DeleteFileHandler(X) Tcl_DeleteFileHandler(X)
#define Tk_CreateFileHandler(I,F,X,M) Tcl_CreateFileHandler(I,F,X,M)
#endif
#if (TCL_MAJOR_VERSION == 7 && TCL_MINOR_VERSION >= 5)
#define Tk_DeleteFileHandler(X) Tcl_DeleteFileHandler(X)
#define Tk_CreateFileHandler(I,F,X,M) Tcl_CreateFileHandler(I,F,X,M)
#undef MULTINTERP
#define MULTINTERP
//#include <tk.h>
#endif

#if (TCL_MAJOR_VERSION == 7 && TCL_MINOR_VERSION <= 5)
#include <tk.h>
#define TCL_READABLE      (1<<1)
#undef MULTINTERP
#ifdef MULTINTERP
//#include <tcl/interpCmd.h>
#endif // MULTINTERP
#endif // VERSION
}

static char* hush_name = "hush";
static char *hushrc_filename = "~/.hushrc";

static char hush_library[BUFSIZ];
static char tcl_library[BUFSIZ];

static Tcl_DString buffer;
static int tty;
   
static int code;
    
static char *filename = NULL;
static char *appname = NULL;



#undef INTERP
#define INTERP(X) ((Tcl_Interp*) X)

typedef void timerproc( ClientData );


void* dvtcl::create() { return (void*)  Tcl_CreateInterp(); }
void dvtcl::_delete(void* x) { Tcl_DeleteInterp(INTERP(x)); }

void dvtcl::init(void* x) {
	if (Tcl_Init(INTERP(x)) == TCL_ERROR) {
		*handler::_err << "ERROR: Tcl init" << endl;
		}
	}

int dvtcl::eval(void* x, const char* s) {
	char *buf  = new char[strlen(s) +1];
	strcpy(buf, s);
	int result = Tcl_Eval(INTERP(x),buf); 
	delete[] buf;
	return result;
	}

char* dvtcl::result(void * x) {
	return INTERP(x)->result;
	}


void dvtcl::result(void * x, const char* s) {
	Tcl_SetResult(INTERP(x), (char*)s, TCL_VOLATILE);
	}

int dvtcl::source(void * x, const char* s) {
	return Tcl_EvalFile(INTERP(x), (char*) s);
	}

int dvtcl::error(void * x) {
	return INTERP(x)->errorLine;
	}

void dvtcl::error(void * x, const char* s) {
	Tcl_AddErrorInfo(INTERP(x), (char*) s);
	}


void* dvtcl::timer(void*, int t, void* f, void* d) { // AE void* x is superflu
#if (TCL_MAJOR_VERSION == 7 && TCL_MINOR_VERSION < 5)
	Tk_CreateTimerHandler(t, (timerproc*) f, (ClientData) d);
#else
  return (void*) Tcl_CreateTimerHandler(t, (timerproc*) f, (ClientData) d);
#endif
	}

void dvtcl::idle(void* , void* f, void* d) {
#if (TCL_MAJOR_VERSION == 7 && TCL_MINOR_VERSION < 5)
	Tk_DoWhenIdle((timerproc*) f, (ClientData) d);
#else
	Tcl_DoWhenIdle((timerproc*) f, (ClientData) d);
#endif
	}

void dvtcl::cancel(void* , void* f, void* d) {
#if (TCL_MAJOR_VERSION >= 7 && TCL_MINOR_VERSION < 5)
	Tk_DoWhenIdle((timerproc*) f, (ClientData) d);
#else
	Tcl_CancelIdleCall((timerproc*) f, (ClientData) d);
#endif
	}

int dvtcl::safe(void* x) {
#ifdef MULTINTERP
        int res = Tcl_IsSafe(INTERP(x));
#else
        int res = x?0:0; // to avoid warnings 
#endif
        return res;
        }

void* dvtcl::slave(void* x, const char* s) {
#ifdef MULTINTERP
	return Tcl_GetSlave(INTERP(x), (char*) s );
#else
	return 0;
#endif
}

void dvtcl::command(void* x, const char* name, void* proc, void* data) {
        Tcl_CreateCommand(INTERP(x),(char*) name,
        (Tcl_CmdProc*) proc, (ClientData) data,
        (Tcl_CmdDeleteProc*) 0);
        }

void dvtcl::command(void*,int fd, void* proc, void* data,int mask) { // AE x
#if (TCL_MAJOR_VERSION == 7 && TCL_MINOR_VERSION < 5)
Tk_CreateFileHandler(fd, mask, (Tk_FileProc*)proc, (ClientData)data);
#else
//AE?
//fprintf(stderr,"MASK  %d %d %d \n", TCL_READABLE ,TCL_WRITABLE,TCL_EXCEPTION);
int tmask = mask;
if (mask==kit::readable) tmask = TCL_READABLE;
else if (mask == kit::writable) tmask = TCL_WRITABLE;
else if (mask == kit::exception) tmask = TCL_EXCEPTION;
else if (mask == 0)  tmask = TCL_READABLE | TCL_EXCEPTION;
/* Tcl_CreateFileHandler(Tcl_GetFile((ClientData)fd, TCL_UNIX_FD), // AE 7.6 8.0 ? 
 * 		tmask, (Tcl_ChannelProc*)proc, (ClientData)data);
 */

Tcl_CreateFileHandler(fd, tmask, (Tcl_ChannelProc*)proc, (ClientData)data); // JvO 8.0b1

/*
Tcl_CreateChannelHandler(Tcl_GetStdChannel(fd), 
		mask, (Tcl_ChannelProc*)proc, (ClientData)data);	// JvO Tcl v7.?
		*/
#endif
}

void dvtcl::unbind(void*, int fd) { // AE x
#if (TCL_MAJOR_VERSION == 7 && TCL_MINOR_VERSION < 5)
	Tk_DeleteFileHandler(fd);
#else
	//Tcl_DeleteChannelHandler(Tcl_GetStdChannel(fd),0,0);
	// Tcl_DeleteFileHandler(Tcl_GetFile((ClientData)fd, TCL_UNIX_FD)); // AE 8.0 ?
	Tcl_DeleteFileHandler(fd); // JvO 8.0b1 ?
#endif
	}

void dvtcl::unbind(void* x, char* n, char*) { // AE s for rename ?
	Tcl_DeleteCommand(INTERP(x),n);
	}


void dvtcl::main(kit*) { }


int dvtcl::check(void* x, int argc, char** argv, char* _name) { 

    Tcl_Interp* interp = (Tcl_Interp*) x;

    Tcl_SetVar(interp, "dummy", "DUMMY", TCL_GLOBAL_ONLY);

//     if (Tk_ParseArgv(interp, (Tk_Window) NULL, &argc, argv, argTable, 0)
// 	    != TCL_OK) {
// 	exit(1);
//     }

 // Recognize _single_ filename argument (5/31/95, jrvosse@cs.vu.nl):
    if (argc == 2 && argv[1][0] != '-') {
        filename = argv[1];
    }


    while ( argc > 0 ) {
	if (argv[0][0]=='-') {
	switch (argv[0][1]) {
		case 'f': 
			filename = argv[1];
			argv += 2; argc -= 2;
			break;
		case '-':
		case 'p':
		case 'x':
			argc--; argv++; break;
			//argv += 2; argc -= 2;
		default:
			*handler::_err << "No such option " << argv[0] << endl;
			argc--; argv++;
			break;
		}
	} else {
	argc--; argv++;
	}
      }
//resume:

    if (appname == NULL) appname = _name;
    if (appname == 0) appname = hush_name;




     // arragne etc
    
    tty = isatty(0);


    char* args = Tcl_Merge(argc-1, argv+1);
    Tcl_SetVar(interp, "argv", args, TCL_GLOBAL_ONLY);
    ckfree(args);
    char buf[BUFSIZ];
    sprintf(buf, "%d", argc-1);
    Tcl_SetVar(interp, "argc", buf, TCL_GLOBAL_ONLY);
    sprintf(buf,"%s",(filename != NULL) ? filename : hush_name);
    Tcl_SetVar(interp, "argv0", buf, TCL_GLOBAL_ONLY);

    Tcl_SetVar(interp, "tcl_interactive",
		    ((filename == NULL) && tty) ? "1" : "0", TCL_GLOBAL_ONLY);


    return 1;
}

int dvtcl::application(void* k) {
	kit* tk = (kit*) k;
	char buf[BUFSIZ];
	
	require( tk );
	
	// tk->trace(7);
	
	char* hushv = session::version();
	if (hushv) {
	        sprintf(buf, "set hushversion \"%s\"", hushv);
	        tk->eval(buf);
	}
	
	char* s = getenv("HUSH_LIBRARY");
	
	if (s == NULL)
		sprintf(::hush_library, "%s", HUSH_LIBRARY);
	else
		sprintf(::hush_library, "%s", s);
	
	char* tcl_d = getenv("TCL_LIBRARY");
	if (tcl_d == NULL)
		sprintf(::tcl_library, "%s",TCL_LIBRARY);
	else
		sprintf(::tcl_library, "%s", tcl_d);
		
	    
	
	// AE session* x = session::lookup("tcl");
	
	
	sprintf(buf,"set hush_library %s",::hush_library);
	tk->eval(buf );
	
	sprintf(buf,"set tcl_library %s",::tcl_library);
	tk->eval(buf );
	
	
	if (1) {
	//sprintf(buf,"source $tcl_library/%s","init.tcl");
	//tk->eval(buf );
	dvtcl::init(tk->interp());
	
	}
	
	if (0) {
	sprintf(buf,"source $hush_library/%s","init.tcl");
	tk->eval(buf );
	}
	return 0;
}





int dvtcl::process(void* x) { 
    //cerr << "Jacco wonders whether dvtcl::process(void*) is ever used ..." << endl;
    Tcl_Interp* interp = (Tcl_Interp*) x;

    (void) Tcl_Eval(interp, "update");

    if (filename != NULL) {
	session* x = session::lookup("tcl");
	if (x) x->_option("noprog",1); // may be overruled by go-back
		else *handler::_err << "ERROR: no tk session" << endl;
	require( x );
        code = Tcl_VarEval(interp, "source ", filename, (char *) NULL);
        if (code != TCL_OK) {
            goto error;
        }
        tty = 0;
    } else {

#ifndef __MSWIN__
 
        if (hushrc_filename != NULL) {
            Tcl_DString buffer;
            char *fullName;
 
            fullName = Tcl_TildeSubst(interp, hushrc_filename, &buffer);
            if (fullName == NULL) {
                *handler::_err << interp->result << endl;         // jpa
            } else {
                if (access(fullName, 4) == 0) { // R_OK
                    code = Tcl_EvalFile(interp, fullName);
                    if (code != TCL_OK) {
                        *handler::_err << interp->result << endl;     // jpa
                    }
                }
            }
            Tcl_DStringFree(&buffer);
        }
        // Tk_CreateFileHandler(0, TCL_READABLE, dvtcl::Stdin, (ClientData) 0);
        Tcl_CreateFileHandler(0, TCL_READABLE, 0, (ClientData) 0); // JvO 8.0b1
        if (tty) {
            dvtcl::Prompt(interp, 0);
        }
#endif // __MSWIN__ 
    }
    handler::_out->flush(); // jpa
    Tcl_DStringInit(&buffer);
    (void) Tcl_Eval(interp, "update");
    return 1; 


error:
    char* msg = Tcl_GetVar(interp, "errorInfo", TCL_GLOBAL_ONLY);
    if (msg == NULL) {
        msg = interp->result;
    }
    *handler::_err << msg << endl;
    //Tcl_Eval(interp, "destroy .");
    //exit(1);
    return 0;                   // Needed only to prevent compiler warnings. 
}




int dvtcl::doevent(int fl) {
#if (TCL_MAJOR_VERSION >= 7 && TK_MINOR_VERSION >= 5 )
	return Tcl_DoOneEvent( fl );
#else
	return fl?0:0; // to trick ..
#endif
}




void dvtcl::Prompt( void * x, int partial ) {
    Tcl_Interp *interp = (Tcl_Interp*) x;
    char *promptCmd;
    int code;
 
    promptCmd = Tcl_GetVar(interp,
        partial ? "tcl_prompt2" : "tcl_prompt1", TCL_GLOBAL_ONLY);
    if (promptCmd == NULL) {
        defaultPrompt:
        if (!partial) {
             *handler::_out  << "% " ;                 // jpa
        }
    } else {
        code = Tcl_Eval(interp, promptCmd);
        if (code != TCL_OK) {
            Tcl_AddErrorInfo(interp,
                    "\n    (script that generates prompt)");
            *handler::_err << interp->result << endl;         // jpa
            goto defaultPrompt;
        }
    }
    handler::_out->flush();                                // jpa
}



void dvtcl::Stdin( void* , int /* mask */ ) {
#ifndef __MSWIN__            // jpa
#define BUFFER_SIZE 4000
    char input[BUFFER_SIZE+1];
    static int gotPartial = 0;
    char *cmd;
    int code, count;
    session* x = session::lookup("tcl");
    if (!x) {
	      *handler::_err << "ERROR dvtcl::Stdin" << endl;
	      require( x );
	      }
    Tcl_Interp* interp = (Tcl_Interp*) x->_resource("interp");
    require( interp );


    count = read(fileno(stdin), input, BUFFER_SIZE);
    if (count <= 0) {
	if (!gotPartial) {
	    if (tty) {
		Tcl_Eval(interp, "exit");
		exit(0);
	    } else {
		Tk_DeleteFileHandler(0);
	    }
	    return;
	} else {
	    count = 0;
	}
    }
    cmd = Tcl_DStringAppend(&buffer, input, count);
    if (count != 0) {
	if ((input[count-1] != '\n') && (input[count-1] != ';')) {
	    gotPartial = 1;
	    goto prompt;
	}
	if (!Tcl_CommandComplete(cmd)) {
	    gotPartial = 1;
	    goto prompt;
	}
    }
    gotPartial = 0;

     // Disable the stdin file handler while evaluating the command;
     // otherwise if the command re-enters the event loop we might
     // process commands from stdin before the current command is
     // finished.  Among other things, this will trash the text of the
     // command being evaluated.

    Tk_CreateFileHandler(0, 0, dvtcl::Stdin, (ClientData) 0);
    code = Tcl_RecordAndEval(interp, cmd, 0);
    Tk_CreateFileHandler(0, TCL_READABLE, dvtcl::Stdin, (ClientData) 0);
    Tcl_DStringFree(&buffer);
    if (*interp->result != 0) {
	if ((code != TCL_OK) || (tty)) {
	    *handler::_out << interp->result << endl;
	}
    }

     // Output a prompt.

    prompt:
    if (tty) {
	dvtcl::Prompt(interp, gotPartial);
    }
#endif // __MSWIN__ 
}

