#include <hush/dv/tk/tk/event.h>


#include <hush/assert.h>
#include <hush/io.h>
//#define endl "\n"

#ifndef HCLONLY
extern "C" {
#include <X11/Xlib.h>
}
#endif

#define LAST ((XEvent*) last)

static char* tk_eventnames[] = {
"UserEvent",
"TclEvent",
"KeyPress",
"KeyRelease",
"ButtonPress",
"ButtonRelease",
"MotionNotify",
"EnterNotify", 
"LeaveNotify",
"FocusIn",   
"FocusOut", 
"KeymapNotify", 
"Expose",      
"GraphicsExpose", 
"NoExpose",      
"VisibilityNotify",
"CreateNotify",   
"DestroyNotify", 
"UnmapNotify",  
"MapNotify",   
"MapRequest", 
"ReparentNotify",
"ConfigureNotify",
"ConfigureRequest",  
"GravityNotify",    
"ResizeRequest",    
"CirculateNotify", //       26"
"CirculateRequest", //        27"
"PropertyNotify", //          28"
"SelectionClear", //          29"
"SelectionRequest", //        30"
"SelectionNotify", //          31"
"ColormapNotify", //          32"
"ClientMessage", //           33"
"MappingNotify", //           34"
"LASTEvent" //               35      /* must be bigger than any event # */"
};





static char* tk_eventname(int i) { return tk_eventnames[i]; }

class widget;
//class callback;
//class client;


tk_event::tk_event(void* p) : tk_tcl_event(0,0,0) { last = p; promise(last); }

tk_event::tk_event(class kit* _tk, int _argc, char* _argv[]) 
			: tk_tcl_event(_tk,_argc,_argv) {
        last = event::_last;
        }

tk_event::~tk_event() { }

char* tk_event::info() const { return "tk_event"; }
int tk_event::eventclass() const { return (int) event::tk; }
int tk_event::type() const { return last?LAST->type:0; }

char* tk_event::name() const { return tk_eventname(type()); } 
void tk_event::trace() const {
	err() << name() << "(" << x() << "," << y() << ")" << endl;
	}

void* tk_event::rawevent() { return last; }

event* tk_event::get() const { return (event*) this; }

int tk_event::x() const {
if (last) 
   switch (LAST->type) {
	case ButtonPress:
	case ButtonRelease:
		return LAST->xbutton.x;
	case MotionNotify:
		return LAST->xmotion.x;
	case KeyPress:
	case KeyRelease:
		return LAST->xkey.x;
	default: return -1;
	}
else return 0;
}

int tk_event::y() const {
if (last) 
    switch (LAST->type) {
	case ButtonPress:
	case ButtonRelease:
		return LAST->xbutton.y;
	case MotionNotify:
		return LAST->xmotion.y;
	case KeyPress:
	case KeyRelease:
		return LAST->xkey.y;
	default: return -1;
	}
else return 0;
}

int tk_event::x_root() const {
if (last)
switch (LAST->type) {
	case ButtonPress:
	case ButtonRelease:
		return LAST->xbutton.x_root;
	case MotionNotify:
		return LAST->xmotion.x_root;
	case KeyPress:
	case KeyRelease:
		return LAST->xkey.x_root;
	default: return -1;
	}
else return 0;
}

int tk_event::y_root() const {
if (last)
switch (LAST->type) {
	case ButtonPress:
	case ButtonRelease:
		return LAST->xbutton.y_root;
	case MotionNotify:
		return LAST->xmotion.y_root;
	case KeyPress:
	case KeyRelease:
		return LAST->xkey.y_root;
	default: return -1;
	}
else return 0;
}

int tk_event::button(int i) const {
const int n = buttno();
if ( buttonevent(i) && ( i==0?1:n==i ) )  return n;
else return 0;
}

int tk_event::buttonup(int i) const {
const int n = buttno();
if ( ( type() == ButtonPress ) && ( i==0?1:n==i ) )  return n;
else return 0;
}

int tk_event::keycode() const {
if (last && keyevent())
	return LAST->xkey.keycode; else return -1; 
}

int tk_event::buttonevent(int i) const {
return buttevent() && ( i==0?1:buttno()==i ); 
}

int tk_event::keyevent() const {
const int tp = type(); return (tp==KeyPress) || (tp==KeyRelease); 
}

int tk_event::motion() const { return (type() == MotionNotify); }

// private:

int tk_event::buttevent() const {
return ((type()==ButtonPress) || (type()==ButtonRelease));
}

int tk_event::buttno() const {
if (last && buttevent()) return LAST->xbutton.button; else return -1;
}


int tk_event::callback(void* data , kit* tk, int argc , char** argv) {
	 event* ev = new tk_event(tk,argc,argv); // AE
	 handler* h = (handler*) data;
	 h->dispatch(ev);
	 delete ev;
	 return 0;
}
