#include <hush/source.h>
#include <hush/dv/tk/tk/tk.h>
#include <hush/dv/tk/tcl/tcl.h>

#include <hush/hush.h>
#include <hush/assert.h>

#include <iostream.h>
#include <hush/io.h>
//#define endl "\n"
#include <hush/sys/unistd.h>
#include <stdlib.h> // jrvosse: for exit()

extern "C" {
#include <tk.h>
//#include <tkInt.h> we don t want this, hence ...
#if (TK_MAJOR_VERSION == 4 && TK_MINOR_VERSION < 1 )
 // we don t need this
#else
Tk_Window TkCreateMainWindow(Tcl_Interp*, char*, char*); // AE declaration
#endif
}

// jrvosse: these are not needed if environment is used:
#ifndef HUSH_LIBRARY
#define HUSH_LIBRARY "Please define HUSH_LIBRARY variable"
#endif

#ifndef TCL_LIBRARY
#define TCL_LIBRARY "Please define TCL_LIBRARY variable"
#endif

#ifndef TK_LIBRARY
#define TK_LIBRARY "Please define TK_LIBRARY variable"
#endif

//#ifdef linux
//#include <unistd.h>
//#endif

#define INTERP(X) ((Tcl_Interp*) X)


static char hush_library[BUFSIZ];
static char tcl_library[BUFSIZ];
static char tk_library[BUFSIZ];


static char* hush_name = "hush";
static char *hushrc_filename = "~/.hushrc";

static session* _session = 0;
static Tcl_Interp* _interp;
 
static Tk_Window MainWindow;

static Tcl_DString buffer;
static int tty;
 
static int code;

//AE static int synchronize = 0;

static char *filename = NULL;
static char *appname = NULL;
static char *display = NULL;
static char *geometry = NULL;

#ifdef __MSWIN__
/*
 * The following declarations refer to internal Tk routines.  The
 * interfaces are available for use, but are not supported.
 */
EXTERN void             TkConsoleCreate(void);
EXTERN int              TkConsoleInit(Tcl_Interp *interp);

// for redirection of cout/cerr to TkConsole:
#include "../mswin/redirect.h"
#endif

void dvtk::init(void* x) {
	dvtcl::init(x);
	if (Tk_Init(INTERP(x)) == TCL_ERROR) {
		*handler::_err << "ERROR: init Tk" << endl;
	}
#if (TK_MAJOR_VERSION == 4 && TK_MINOR_VERSION < 1 )
        // do nothing
#else
	Tcl_StaticPackage(INTERP(x), "Tk", Tk_Init, 0 );
#endif
#ifdef __MSWIN__
	TkConsoleCreate();
	if (TkConsoleInit(INTERP(x)) == TCL_ERROR) exit(1);
	// Memory leak ... jrvosse
    cout = new infostreambuf(x);
    cerr = new infostreambuf(x);
#endif	
}

extern void* thelastevent();

int traceX(ClientData , XEvent* last) {
event::_last = last;
//((kit*)k)->put((event*) last, event::x_event );
return 0;
}


void* dvtk::mainwindow(void* x,char* display,char* appname,char*,int synch ) {
Tk_Window tkwin = 0;
Tcl_Interp* interp = x?(Tcl_Interp*)x:(::_interp);
#if (TK_MAJOR_VERSION == 4 && TK_MINOR_VERSION < 1)
	tkwin = Tk_CreateMainWindow(interp, display, appname,"Tk");
#else
	tkwin = TkCreateMainWindow(interp, display, appname);
#endif
	if (!tkwin) {
		*handler::_err << interp->result << endl;
		exit(1);
		}
	Tk_SetClass(tkwin,"Tk");
#ifndef __MSWIN__
	if (synch) XSynchronize(Tk_Display(MainWindow), True);
#endif
	Tk_GeometryRequest(tkwin,50,50);
	::MainWindow = (Tk_Window) tkwin;
	return tkwin;
}

void dvtk::xtracer(void* x) {
	Tk_CreateGenericHandler(traceX,(ClientData)x);
	}

void dvtk::setclass(void* x, char* w, char* cl) {
	Tk_SetClass(Tk_NameToWindow(INTERP(x),w,Tk_MainWindow(INTERP(x))), cl);
	}

void dvtk::map(void* w) { Tk_MapWindow( (Tk_Window) w ); }
void dvtk::unmap(void* w) { Tk_UnmapWindow( (Tk_Window) w ); }

int dvtk::check(session* s, void* x,int argc,char** argv,char* _name) { 

    ::_session = s;
    ::_interp = (Tcl_Interp*) x;

    Tcl_Interp* interp = (Tcl_Interp*) x;

    //Tcl_SetVar(interp, "dummy", "DUMMY", TCL_GLOBAL_ONLY);

    /*
    if (Tk_ParseArgv(interp, (Tk_Window) NULL, &argc, argv, argTable, 0)
	    != TCL_OK) {
	*handler::_err << interp->result << endl;      // jpa
	exit(1);
    }
    */

 // Recognize _single_ filename argument (5/31/95, jrvosse@cs.vu.nl):
    if (argc == 2 && argv[1][0] != '-') {
        filename = argv[1];
    }


    while ( argc > 0 ) {
	if (argv[0][0]=='-') {
	switch (argv[0][1]) {
		case 'f': 
			filename = argv[1];
			argv += 2; argc -= 2;
			break;
		case '-':
		case 'p':
		case 'x':
			argc--; argv++; break;
			//argv += 2; argc -= 2;
		default:
			*handler::_err << "No such option " << argv[0] << endl;
			argc--; argv++;
			break;
		}
	} else {
	argc--; argv++;
	}
      }
//resume:

    if (appname == NULL) appname = _name;
    if (appname == 0) appname = hush_name;


   if (display != NULL) {
	Tcl_SetVar2(interp, "env", "DISPLAY", display, TCL_GLOBAL_ONLY);
	}


    /*
     * Initialize the Tk application and arrange to map the main window
     * after the startup script has been executed, if any.  This way
     * the script can withdraw the window so it isn't ever mapped
     * at all.
     */
    
    tty = isatty(0);

    /*
     * Make command-line arguments available in the Tcl variables "argc"
     * and "argv".  Also set the "geometry" variable from the geometry
     * specified on the command line.
     */

    char* args = Tcl_Merge(argc-1, argv+1);
    Tcl_SetVar(interp, "argv", args, TCL_GLOBAL_ONLY);
    ckfree(args);
    char buf[BUFSIZ];
    sprintf(buf, "%d", argc-1);
    Tcl_SetVar(interp, "argc", buf, TCL_GLOBAL_ONLY);
    sprintf(buf,"%s",(filename != NULL) ? filename : appname);
    Tcl_SetVar(interp, "argv0", buf, TCL_GLOBAL_ONLY);
    if ((geometry != NULL)) {
	Tcl_SetVar(interp, "geometry", geometry, TCL_GLOBAL_ONLY);
    }

    Tcl_SetVar(interp, "tcl_interactive",
		    ((filename == NULL) && tty) ? "1" : "0", TCL_GLOBAL_ONLY);


    return 1;
}

int dvtk::application(void* k) {
	kit* tk = (kit*) k;
	char buf[BUFSIZ];
	
	require( tk );
	
	// tk->trace(7);
	
	char* hushv = session::version();
	if (hushv) {
	        sprintf(buf, "set hushversion \"%s\"", hushv);
	        tk->eval(buf);
	}
	
	char* s = getenv("HUSH_LIBRARY");
	
	if (s == NULL)
		sprintf(::hush_library, "%s", HUSH_LIBRARY);
	else
		sprintf(::hush_library, "%s", s);
	
	char* tcl_d = getenv("TCL_LIBRARY");
	if (tcl_d == NULL)
		sprintf(::tcl_library, "%s",TCL_LIBRARY);
	else
		sprintf(::tcl_library, "%s", tcl_d);
		
	    
	char* tk_d = getenv("TK_LIBRARY");
	
	session* x = ::_session;
	int nowin = session::_nowin; // x?x->_option("nowin"):0;
	if (!nowin) {
	if (tk_d == NULL)
		sprintf(::tk_library, "%s",TK_LIBRARY);
	else
		sprintf(::tk_library, "%s", tk_d);
	}
	
	
	sprintf(buf,"set hush_library %s",::hush_library);
	tk->eval(buf );
	
	sprintf(buf,"set tcl_library %s",::tcl_library);
	tk->eval(buf );
	
	if (!nowin) {
		sprintf(buf,"set tk_library %s",::tk_library);
		tk->eval(buf );
		}
	
	
	if (0) {
	sprintf(buf,"source $tcl_library/%s","init.tcl");
	//tk->eval(buf );
	//AE dvtcl::init(tk->interp());
	
	sprintf(buf,"source $tk_library/%s","tk.tcl");
	//tk->eval(buf );
	if (!nowin) dvtk::init(tk->interp());
	}

        if (!nowin) dvtk::init(tk->interp());
	
	if (0) {
	sprintf(buf,"source $hush_library/%s","init.tcl");
	tk->eval(buf );
	}
	return 0;
}





int dvtk::process(void* x) { 
      
    Tcl_Interp* interp = (Tcl_Interp*) x;

    (void) Tcl_Eval(interp, "update");

    if ((geometry != NULL)) {
       code = Tcl_VarEval(interp, "wm geometry . ", geometry, (char *) NULL);
       if (code != TCL_OK) {
           *handler::_err << interp->result << endl;          // jpa
       }
    }
    if (filename != NULL) {
	session* x = _session;
	if (x) x->_option("noprog",1); // may be overruled by go-back
		else *handler::_err << "ERROR: no tk session" << endl;
	require( x );
        code = Tcl_VarEval(interp, "source ", filename, (char *) NULL);
        if (code != TCL_OK) {
            goto error;
        }
        tty = 0;
    } else {
#ifndef __MSWIN__
        /*
         * Commands will come from standard input, so set up an event
         * handler for standard input.  If the input device is aEvaluate the
         * .rc file, if one has been specified, set up an event handler
         * for standard input, and print a prompt if the input
         * device is a terminal.
         */
 
        if (hushrc_filename != NULL) {

            Tcl_DString buffer;
            char *fullName;
 
            fullName = Tcl_TildeSubst(interp, hushrc_filename, &buffer);
            if (fullName == NULL) {
                *handler::_err << interp->result << endl;         // jpa
            } else {
                if (access(fullName, 4) == 0) { // R_OK
                    code = Tcl_EvalFile(interp, fullName);
                    if (code != TCL_OK) {
                        *handler::_err << interp->result << endl;     // jpa
                    }
                }
            }
            Tcl_DStringFree(&buffer);
        }
        Tk_CreateFileHandler(0, TK_READABLE, dvtk::Stdin, (ClientData) 0);
        if (tty) {
            dvtk::Prompt(interp, 0);
        }
#endif /* __MSWIN__ */
    }
    // jrvosse ? cout.flush(); // jpa
    Tcl_DStringInit(&buffer);
    (void) Tcl_Eval(interp, "update");
    return 1; 


error:
    char* msg = Tcl_GetVar(interp, "errorInfo", TCL_GLOBAL_ONLY);
    if (msg == NULL) {
        msg = interp->result;
    }
    *handler::_err << msg << endl;
    //Tcl_Eval(interp, "destroy .");
    //exit(1);
    return 0;                   /* Needed only to prevent compiler warnings. */
}




#if (TK_MAJOR_VERSION == 4 && TK_MINOR_VERSION < 1)
extern int tk_NumMainWindows;
#endif

int dvtk::windows() {
#if (TK_MAJOR_VERSION == 4 && TK_MINOR_VERSION < 1 )
	return tk_NumMainWindows;
#else
	return Tk_GetNumMainWindows();
#endif
}

int dvtk::doevent(int fl) {
#if (TK_MAJOR_VERSION == 4 && TK_MINOR_VERSION < 1 )
	return Tk_DoOneEvent( fl );
#else
	return Tcl_DoOneEvent( fl );
#endif
}



void dvtk::alias(const char* , const widget*) {

    *handler::_err << "ERROR: dvtk::alias is not supported " << endl;
    /*
    TkWindow* tkwin = (Tk_Window*) w->tkwin(); 
    Tk_Window* win = (Tk_Window*) w->tkwin();
    Tcl_HashEntry *hPtr;
    int c_new;
 
    hPtr = Tcl_CreateHashEntry(& tkwin->mainPtr->nameTable,(char*) path, &c_new);
    if (!c_new)
     	*handler::_err << "WINDOW " << path << " " << "exists" << endl;
    Tcl_SetHashValue(hPtr, win);
    win->pathName = Tcl_GetHashKey( & tkwin->mainPtr->nameTable, hPtr);
    */
}

//// ???

void* thewindow(kit* tk, const char* path) {
Tk_Window mwin = (Tk_Window) ::MainWindow;
Tk_Window win = Tk_NameToWindow((Tcl_Interp*)tk->interp(), (char*) path, mwin);
return (void*)  win;
}


//void dvtk::main(kit* tk) { }








void dvtk::Prompt( void * x, int partial ) {
    Tcl_Interp *interp = (Tcl_Interp*) x;
    char *promptCmd;
    int code;
 
    promptCmd = Tcl_GetVar(interp,
        partial ? "tcl_prompt2" : "tcl_prompt1", TCL_GLOBAL_ONLY);
    if (promptCmd == NULL) {
        defaultPrompt:
        if (!partial) {
             cout  << "% " ;                 // jpa
	     cout.flush();
        }
    } else {
        code = Tcl_Eval(interp, promptCmd);
        if (code != TCL_OK) {
            Tcl_AddErrorInfo(interp,
                    "\n    (script that generates prompt)");
            *handler::_err << interp->result << endl;         // jpa
            goto defaultPrompt;
        }
    }
    cout.flush();                                // jpa
}



void dvtk::Stdin( void* , int /* mask */ ) {
#ifndef __MSWIN__            // jpa
#define BUFFER_SIZE 4000
    char input[BUFFER_SIZE+1];
    static int gotPartial = 0;
    char *cmd;
    int code, count;
    session* x = ::_session;
    if (!x) {
	      *handler::_err << "ERROR dvtk::Stdin" << endl;
	      require( x );
	      }
    Tcl_Interp* interp = ::_interp; // (Tcl_Interp*) x->_resource("interp");
    require( interp );


    count = read(fileno(stdin), input, BUFFER_SIZE);
    if (count <= 0) {
	if (!gotPartial) {
	    if (tty) {
		Tcl_Eval(interp, "exit");
		exit(0);
	    } else {
		Tk_DeleteFileHandler(0);
	    }
	    return;
	} else {
	    count = 0;
	}
    }
    cmd = Tcl_DStringAppend(&buffer, input, count);
    if (count != 0) {
	if ((input[count-1] != '\n') && (input[count-1] != ';')) {
	    gotPartial = 1;
	    goto prompt;
	}
	if (!Tcl_CommandComplete(cmd)) {
	    gotPartial = 1;
	    goto prompt;
	}
    }
    gotPartial = 0;

    /*
     * Disable the stdin file handler while evaluating the command;
     * otherwise if the command re-enters the event loop we might
     * process commands from stdin before the current command is
     * finished.  Among other things, this will trash the text of the
     * command being evaluated.
     */

    Tk_CreateFileHandler(0, 0, dvtk::Stdin, (ClientData) 0);
    code = Tcl_RecordAndEval(interp, cmd, 0);
    Tk_CreateFileHandler(0, TK_READABLE, dvtk::Stdin, (ClientData) 0);
    Tcl_DStringFree(&buffer);
    if (*interp->result != 0) {
	if ((code != TCL_OK) || (tty)) {
	    cout << interp->result << endl;
	}
    }

    /*
     * Output a prompt.
     */

    prompt:
    if (tty) {
	dvtk::Prompt(interp, gotPartial);
    }
#endif /* __MSWIN__ */
}

