//
//  Vrml 97 library
//  Copyright (C) 1998 Chris Morley
//
#ifndef _VRMLNAMESPACE_
#define _VRMLNAMESPACE_

//
// The VrmlNamespace class keeps track of defined nodes and prototypes.
//

#include "config.h"

#include _LIST_H_

NAMESPACE;

#ifdef _WIN32
#include "VrmlNodeType.h"
#else
class VrmlNodeType;
#endif

class VrmlNamespace {
public:

  VrmlNamespace();

  ~VrmlNamespace();

  // Namespace management functions.  PROTO definitions add node types
  // to the namespace.  PROTO implementations are a separate node
  // namespace, and require that any nested PROTOs NOT be available
  // outside the PROTO implementation.
  // addToNameSpace will print an error to stderr if the given type
  // is already defined.
  void addToNameSpace(VrmlNodeType *);
  void pushNameSpace();
  void popNameSpace();

  // Find a node type, given its name.  Returns NULL if type is not defined.
  const VrmlNodeType *find(const char *nm);

  // Return the first node type in scope (default EXTERNPROTO implementation)
  const VrmlNodeType *first();

private:

  void addToNameSpaceBuiltIn(VrmlNodeType *);
  void defineBuiltIns();

  // Builtin node types are stored (once) in this data structure:
  static list< VrmlNodeType* > builtInList;

  // Defined node types (PROTOs) are stored (per scene) in this data structure:
  list< VrmlNodeType* > typeList;

};

#endif // _VRMLNAMESPACE_
