//
//  Vrml 97 library
//  Copyright (C) 1998 Chris Morley
//  %W% %G%
#ifndef  _VRMLNODEPROTO_
#define  _VRMLNODEPROTO_

//
// A VrmlNodeProto object is created to represent an instance of a PROTOd node.
// The definition of the PROTO is stored in a VrmlNodeType object; the values
// of the fields are stored in the VrmlNodeProto object.
//

#include "config.h"
#include "VrmlNode.h"
#include "VrmlNodeType.h"
#include "Viewer.h"

#include _LIST_H_


class VrmlNodeProto : public VrmlNode {

public:

  virtual VrmlNodeType *nodeType() const;

  VrmlNodeProto(VrmlNodeType *nodeDef, VrmlScene *scene);
  virtual ~VrmlNodeProto();

  virtual void addToScene( VrmlScene * );
  virtual ostream& printFields(ostream& os, int indent);

  virtual void setField(const char *fieldName, const VrmlField &fieldValue);

  VrmlNode *firstNode() const;

  virtual bool isProtoNode() const;

  // These are passed along to the first implementation node of the proto.
  virtual bool isAnchorNode() const;
  virtual bool isAppearanceNode() const;
  virtual bool isChildNode() const;
  virtual bool isColorNode() const;
  virtual bool isCoordinateNode() const;
  virtual bool isFontStyleNode() const;
  virtual bool isGeometryNode() const;
  virtual bool isGroupNode() const;
  virtual bool isLightNode() const;
  virtual bool isScopedLightNode() const;
  virtual bool isMaterialNode() const;
  virtual bool isNormalNode() const;
  virtual bool isScriptNode() const;
  virtual bool isTextureCoordinateNode() const;
  virtual bool isTextureNode() const;
  virtual bool isTextureTransformNode() const;
  virtual bool isTouchSensorNode() const;

  virtual void render(Viewer *);

  typedef struct {
    char *name;
    VrmlField *value;
  } NameValueRec;

private:

  // Find a field by name
  NameValueRec *findField(const char *fieldName);

  VrmlNodeType *d_nodeType;	// Definition

  list<NameValueRec*> d_fields;	// Field values

  Viewer::Object d_viewerObject; // move to VrmlNode.h ? ...

};

#endif _VRMLNODEPROTO_
