%{
//@auto=0
//@class inc
%}
%class lex
%{
char* ext = ".rf";

#include <hush/grammar/input.h>
#include <hush/grammar/string.h>
#include <hush/grammar/converter.h>
#include <hush/grammar/attribute.h>
#include <iostream.h>
//extern "C" system(char*);
#include <hush/set.h>

#define entry kato_entry

struct entry {
string K;
string R;
string A;
string T;
string B;
string O;
string X;

char* key()   { return (char*) K; }
char* ref()   { return (char*) R; }
char* title() { return is_book()?(char*)B:(char*)T; }
char* author() { return (char*) A; }
char* publisher() { return (char*) O; }
char* comment()  { return (char*) X; }

int is_book() {
	if (B.length() != 0) return 1;
	else return 0;
	}

virtual void print(ostream& os) {
	os << "%K " << key();
	os << ref() << endl;
	os << "%A " << author() << endl;
	os << (is_book()?"%B ":"%T ") << title() << endl;
	os << "%O " << publisher() << endl;
	os << "%X " << comment() << endl;
	}

};

int operator==(const entry& e1, const entry& e2) {
        return (e1.A).compare(e2.A); }

int operator<(const entry& e1, const entry& e2) { 
	int result = (e1.A < e2.A);
	//cerr << "Comparing " << e1.A  << " and " << e2.A << " " << result << endl; 
	return result;
	}

ostream& operator<<(ostream& os, const entry& e) {
	((entry&)e).print(os);
	return os;
	}

static entry* current=0;
static oset<entry> entries;

static int in_biblio = 0;
static int do_biblio = 0;

%}
name [\~\/A-Za-z0-9\.][A-Za-z0-9\.\/\-\+]*
names [ \t]*{name}({name}|[ \t])*
atts [^\>]*
C (\%|\@|\.)
%%@ rules


"<bibliography>" {
	in_biblio++;
        if (in_biblio==1) { do_biblio=1;  cout << yytext << endl; }
        }
"</bibliography>" {
        if (in_biblio==1) {
		if (current) entries << current;
		current = 0;
		}
	in_biblio--;
        }

^\%"K ".* {
              if (current) {
			//current->print(cout);
			entries << current;
			}
              current = new entry;
              current->K = yytext+3;
        }


^\%"R ".* { current->R = yytext+3; }
^\%"A ".* { current->A = yytext+3; }
^\%"T ".* { current->T = yytext+3; }
^\%"B ".* { current->B = yytext+3; }
^\%"O ".* { current->O = yytext+3; }
^\%"X ".* { current->X = yytext+3; }


"<so "{names}">" { yytext[yyleng-1]='\0'; files(yytext+4,ext); }
"<cd "{names}">" { yytext[yyleng-1]='\0'; directories(yytext+4,ext); }
"\\so "{names}/("-->"|"}"|\n) { files(yytext+4,ext); }
"\\cd "{names}/("-->"|"}"|\n) { directories(yytext+4,ext); }
^{C}"so "{names}/("-->"|"}"|\n) { files(yytext+4,ext); }
^{C}"cd "{names}/("-->"|"}"|\n) { directories(yytext+4,ext); }


"<include "{atts}">" {
   int choice = 0;
   yytext[yyleng-1] = '\0';
   string dir = "";
   string ext = "";
   string file = "@temp";
   string cmd = "";
   string_converter x(yytext+8);
   for (int i = 1; i <= x.argc(); i++ ) {
       //cerr << "ATTR:" << x[i] << endl;
       attribute p(x[i]);
           if ( p.matches("file") ) { 
			if ((choice !=3) && (choice!=2)) choice = 1;
			file = p.value();
			}
           if ( p.matches("files") ) { choice = 1; file = p.value(); }
           if ( p.matches("dir") ) { choice = 2; dir = p.value(); }
           if ( p.matches("dirs") ) { choice = 2; dir = p.value(); }
           if ( p.matches("ext") ) { ext = p.value(); }
           if ( p.matches("cmd") ) { choice = 3; cmd = p.value(); }
           }
   if (choice == 0) file = yytext+9;
   if (choice == 2 && !strcmp(ext,"")) ext = ::ext;
   //cerr << "FILE:" << file << " EXT:" << (char*) ext << endl;
   if (choice==2) directories(dir,ext,file); 
   else if(choice==3) {
		char buf[BUFSIZ];
		sprintf(buf,"%s > %s",(char*)cmd,(char*)file);
		system(buf); files(file,"");
		}
   else files(file,ext);
   }

[ \t\n] { ;}
. { ; }
%%
//@ main

static char _argv[24][256];

int main(int argc, char* argv[]) {
lex l;
int _argc = 0;
while ((argc > 0) && (argv[0][0] == '-')) {
	if ( ( argv[0][1] == 'I' ) || ( argv[0][1] == 'R' ) ) { 
		strcpy(_argv[_argc],argv[0]);
		//cerr << "Copied " << _argv[_argc] << endl;
		_argc++;
		}
	argc--; argv++;
	}

l.paths(_argc, _argv);
l();
/*
if ( argc == 0 )  l(); 
else
	while (--argc > 0) 
		l.soinc( *++argv );
*/
if (current) entries.insert(current);
//cout << "**********" << endl;
cout << entries <<  endl;
//iter<entry> it = (entries);
/*
entry* p = 0;
while ( p = entries.remove() ) {
	if (p) cout << *p << endl;
	}
*/
if (do_biblio==1 && in_biblio==0)
	cout << "</bibliography>" << endl;
return 0;
}
//</>
