//@class=agent

#include <hush/source.h>

#include <hush/hush.h>
#include <hush/agent/agent.h>

#include <hush/agent/locator.h>
#include <hush/agent/finder.h>
#include <hush/agent/resolver.h>
#include <hush/agent/loader.h>

#include <hush/io.h>
#include <stdio.h>

agent* agent::_default = 0;

#define TYPE agent
#include <hush/sys/table.i>

///@method agent


void agent::set_default(agent* x) { _default = x; }
agent* agent::get_default() { return _default; }

agent::agent(const char* kind) {
	_agent = 0;
	if (kind) _agent = agent::lookup(kind);
	if (!_agent) {
	_locator  = new locator();
	_register(_locator);
	_resolver = new resolver();
	_register(_resolver);
	_loader = new loader();
	_register(_loader);
	_finder = new finder();
	_register(_finder);
	}
	//cerr << "created agent for resolving" << endl;
}


///@method lookin

void agent::lookin(const char* dir) {
	if (_agent) _agent->lookin(dir);
	else _finder->add(dir);
	}

///@method source

char* agent::source(const char* url) {
if (_agent) return _agent->source(url);
else {
	char* x = alias(url);
	if (x) url = x;
	if (marked(url)) {
		//cerr << "Warning: " << url << " is marked (source)" << endl;
		}
	char* p = _finder->find(url);
	if (!p) p = _resolver->file(url);
	if (p)
		; //cerr << "Resolved: " << p << endl; 
	else
		cerr << "Failed to resolve: " << url << endl; 
	if (p) {
		mark(url,p);
		//tk->source(p);
		}
	return p;
	}
}


///@method load

int agent::load(const char* url) {
	if (_agent) return _agent->load(url);
	char* x = alias(url);
	if (x) url = x;
	if (marked(url)) {
		cerr << "Agent: " << url << " is marked" << endl;
		}
	char buf[1024];
	sprintf(buf,"%s.so",url);
	char* p = _finder->find(buf);
	if (!p) p = _resolver->file(buf);
	cerr << "Resolved: " << p << endl; 
	int status = 0;
	if (p) {
		status = _loader->load(p,url,1);
		}
	if (!status) {
		cerr << "Error: no loading of " << url << endl;
		}
	else
		mark(url,p);
	return status;
	}


///@method alias

void agent::alias(const char* k, const char* v) {
if (_agent) _agent->alias(k,v); 
else {
	if (!_locator) {
		_locator = new locator();
		_register(_locator);
		}
	_locator->alias(k,v);
	}
}

char* agent::alias(const char* k) {
	if (_agent) return _agent->alias(k);
	if (!_locator) return 0;
	else return _locator->alias(k);
	}


///@method mark

void agent::mark(const char* k, const char* v) {
if (_agent) _agent->mark(k,v);
else {
	if (!_locator) {
		_locator = new locator();
		_register(_locator);
		}
	_locator->mark(k,v);
	}
}

char* agent::marked(const char* k) {
	if (_agent) return _agent->marked(k);
	if (!_locator) return 0;
	else return _locator->marked(k);
	}

///</>
