#include <hush/source.h>

#include <hush/agent/query.h>
#include <hush/agent/agent.h>

#include <stdio.h>
#include <stdlib.h>

#include <hush/grammar/string.h>
#include <hush/grammar/converter.h>
#include <hush/grammar/attribute.h>

#include <hush/io.h>

int query::no = 0;
int query::nodebug = 0;

query* query::_default = 0;

#define TYPE query
#include <hush/sys/table.i>

static char _idbuf[64];

char* query::id() {
	if (!_id) return 0;
	strcpy(_idbuf,_id);
	return _idbuf;
	}


query::query(int argc, char* argv[]) {
_query = 0; _handler = 0; _prepared = 0; _session = 0;
if (!session::_default) {
	_session = new session();
	session::_default = _session;
	session::initialize();
	kit::_default = new kit();
	session::_default->_register(kit::_default);
	}
if (!agent::_default) {
		agent::_default = new agent();
		_register(agent::_default);
		}
	strcpy(_command,"");
	_kit = 0;
	_result = 0;
	_once = 0;
	_exec = 0;
	//option("kit","pl");
	option("halt","");
	option("cmd","");
	option("sh","xterm");
	//string_converter x((char*)opts);
	for (int i = 1; i < argc; i++ ) {
	    //cerr << "query::argv " << argv[i] << endl;
	    if ( argv[i][0]=='-' ) {
		if ( argv[i][1] == 'c' ) option("cmd",argv[i+1]);
		i++;
	    } else {
		attribute p(argv[i]);
		option((char*)p.key(),(char*)p.value());
		}
	}
}

query::query(const char* opts) {
_query = 0; _handler = 0; _prepared = 0; _session = 0;
if (!session::_default) {
	_session = new session();
	session::_default = _session;
	session::initialize();
	kit::_default = new kit();
	session::_default->_register(kit::_default);
	}
if (!agent::_default) {
		agent::_default = new agent();
		_register(agent::_default);
		}
// if (opts) cerr << "query::create with " << opts << endl;
	strcpy(_command,"");
	_kit = 0;
	_result = 0;
	_once = 0;
	_exec = 0;
	option("halt","halt");
	option("kind","pl");
	option("mode","normal");
if (opts) {
	string_converter x((char*)opts);
	for (int i = 1; i <= x.argc(); i++ ) {
	attribute p(x[i]);
	option((char*)p.key(),(char*)p.value());
	}
}
}

query::~query() {
        if (_session) { 
		delete _session;
		session::statistics();
		}
	}


int query::intvalue(const char* arg) {
	char* ws = unquote( evaluate(arg) );
	int w; sscanf(ws,"%d",&w);
	return w;
}

float query::floatvalue(const char* arg) {
	char* ws = unquote( evaluate(arg) );
	float w; sscanf(ws,"%f",&w);
	return w;
}


char* query::strvalue(const char* arg) {
    return unquote( evaluate(arg) );
}

void query::init(int argc, char** argv) {
	if (_query) _query->init(argc,argv);
	}

void query::_init() {
	//cerr << "query::init" << endl;
	char buf[256];
	char* opt = option("file");
	if (!opt && option("out")) {
		sprintf(buf,"@query-out-%d", query::no++);
		if (!strcmp(option("out"),"true"))
					option("file",buf);
		}
	//cerr << "query getting " << option("kit") << " kit" << endl;
	if ( !strcmp(option("mode"),"exec") ) 
	  	_exec = 1;
	if (!_exec) {
	  kit::initialize();
	  if (!kit::_default) {
		kit::_default = new kit();
		_register(kit::_default);
		}
	  _kit = kit::_default->_kit(option("kit"));
	  opt = option("mode");
    if (opt && !strcmp(option("kit"),"pl") && strcmp(opt,"exec") )
			  _kit->mode(1);
	//cerr << "query got " << _kit->category() << " kit " << (void*) _kit << endl;
	}
	// cerr << "query::init succeeded " << endl;
}

char* query::category() const {
	if (_query) return _query->category();
	else return "query"; 
	}

void query::_send(const char* mode , const char* val ) {
	if (_query) _query->_send(mode,val);
	}
void query::_reset(const char* mode) { 
	if (_query) _query->_reset(mode);
	}


int query::prepare() {
if (_query) return _query->prepare();
if (_prepared) return _prepared;
_prepared = 1;
//cerr << "query::prepare" << endl;
_init();
if (option("id")) kit::declare(option("id"),this); 
if (!_exec) {
	//cerr << "calling pl kit" << endl;
	char* src = option("src");
	if (src) this->source(src);
	char* program = option("program");
	if (program) this->source(program);
	if (option("bind")) bind(option("bind"),_handler);
	return 1;
} else {
	//cerr << "query::exec" << endl;
	char buf[10*1024]; char cmd[24]; 
	char* x = option("kind"); if (!x) x = option("kit");
	if (!x)  strcpy(cmd,"sh");
	else strcpy(cmd,x);
	if (strcmp(x,"pl")) // x != pl
		x = "sh";
	sprintf(buf,"%s/exec-%s -sort %s %s ",BIN,x,cmd,_command);
	//cerr << "query::exec " << buf << endl;
	system( buf );
	return 0;
	}
}

int query::operator()() {
	if (_query) return _query->operator()();
	prepare();
	if (!_exec) {
		if (option("cmd")) eval(option("cmd"));
		}
	return OK;
	}

int query::source(const char* arg) {
	if (_query) _query->source(arg);
	if (!_prepared) prepare();
	if (_exec) return 0;
	char* f = agent::_default->source(arg);
        //if (_kit) cerr << "query src " << f << endl;
	return _kit?_kit->source(f):0;
	}

char* query::ask(const char* cmd) {
	return evaluate(cmd);
	}

void query::halt() {
	cerr << "query::halt is a noop" << endl;
	}

char* query::evaluate(const char* cmd) {
	long res = eval(cmd);
	return res?result(res):0;
	}

long query::eval(const char* arg) {
	if (_query) return _query->eval(arg);
	if (!_prepared) prepare();
	_result = 0;
	if (_exec) return 0;
	//cerr << "query::eval " << arg << endl;
	long res = _result = _kit?_kit->eval(arg):0;
	return res;
	}

void query::_assert(const char* x, const char* opt) {
	char buf[512];
	sprintf(buf,":assert%s(%s)",opt,x);
	eval(buf);
	}

void query::_retract(const char* x, const char* opt) {
	char buf[512];
	sprintf(buf,":retract%s(%s)",opt,x);
	eval(buf);
	}


void query::peek() {
	char* k = option("kind");
	if (!k) k = option("kit");
	if (k && ( !strcmp(k,"pl") ||
		   !strcmp(k,"dlp") ||
		   !strcmp(k,"prolog") )) { 
		//if (_kit) _result = (term*) _kit->result(); 
	        //_peek = _result?_result->next():0;
		}
	}

void query::result(const char*) {   }

char* query::result(long id) {
if (!id) id = _result;
if (_query) return _query->result(id);
if (!_prepared) prepare();
//cerr << "getting results " << endl;
if (_exec) {
return 0; //
} else {
	if (_kit) // && !strcmp(_kit->category(),"query"))
		return _kit->result(id);
	if (!_result) return 0;
	if (_once++ > 0) return 0;
	return 0;
	}
}

dvbinding* query::bind(const char* _name, handler* h) {
	if (_query) return _query->bind(_name,h);
	cerr << "query binding for " << _name << endl;
	//if (!_prepared) prepare();
	if (_kit) return _kit->bind(_name,h);
	else {
		option("bind",(char*)_name);
		_handler = h;
		}
    cerr << "query binding for " << _name << " completed" << endl;
	return 0;
	}

dvbinding* query::bind(handler* h) {
	if (_query) return _query->bind(h); 
	return _kit?_kit->kit::bind(h):0;
	}

dvbinding* query::bind(int fd, handler* h, int mask) {
	return _kit?_kit->kit::bind(fd,h,mask):0;
	}




void query::option(const char* k, const char* v) {
	//cerr << "query: setting option " << k << " = " << v << endl;
if (_query) {
	_query->option(k,v);
} else {
	if (!strcmp(k,"type")) {
		load("query");
		_query =  query::lookup(v);
		}
	else if (!strcmp(k,"sh")) {
		option("kind",v);
		option("mode","exec");
		_exec = 1;
		}
	else if (!strcmp(k,"kit")) {
		option("kind",v);
		option("mode","normal");
		_exec = 0;
		}
	sprintf(_command,"%s -%s \"%s\"",_command,k,v);
	handler::option(k,v);
	}
}

char* query::option(const char* k) {
	if (_query) return _query->option(k);
	else return handler::option(k);
	}
