#include <hush/source.h>

#include <hush/session.h>

#include <hush/kit.h>
#include <hush/event.h>
#include <hush/sys/stats.h>

#include <hush/agent/loader.h>
#include <hush/string.h>
#include <hush/iter.h>

#include <hush/assert.h>

//#include "iostream.h" //AE
#include <string.h>
#include <stdio.h>

#include <hush/api/io.h>

static char _version[124];


char* session::_depends = 0;

// static char* dv_depends = "hush:dv:widgets";
static char* dv_depends = "hush";
static char depbuf[512];

int session::do_quit() { return session::_quit; }
void session::set_quit(int i) { session::_quit = i; }
int session::has_nowin() { return session::_nowin; }


void session::set_default(session* x) { _default = x; }
session* session::get_default() { return _default; }

int session::initialize() {
	handler::initialize();
	if (!session::initialized) {
		session* s = session::lookup("global");
		if (s) session::_default = s;
		session::_depends = dv_depends;
		hush_ostream::initialize();
		}
	return session::initialized = 1;
	}

// AE -- temporary
static
session* a_new_session( const char* mode ="shell" ) {
	session* s = 0; // session::_default;
        if (!s) s = session::lookup("tk");
	if (!s) s = session::lookup("tcl");
	if (!s) s = session::lookup("hush");
        if (!s) s = session::lookup(mode);
	//if (!s) s = session::_default;
        //promise ( s );
        return s;
        }


#ifndef NOTIME
#include <sys/time.h>
#include <time.h>

static _inittime = 0;
static timeval inittime;
#endif

static session* _thesession;
session* thesession() { return ::_thesession; }
void thesession(session* s) { ::_thesession = s; }

int session::_java  = 0;
int session::_corba  = 0;

int session::_trace  = 0;
int session::_quit  = 0;
int session::_nowin  = 0;

int session::initialized = 0;

session* session::_default = 0;
void* session::_create = 0;

typedef session* (*createf)(const char*);
#define CREATE (session*) ((createf)session::_create)

#define TYPE session
#include "hush/sys/table.i"
#include "hush/sys/state.h"

struct session_state : public dvstate {
    session_state( int argc = 0, char** argv = 0, char* nm = 0,
              int fl = 0,
	      double c = 0, double s = 0, session* ss = 0) {
	_argc = argc; _argv = argv; _name = nm; _flags = fl;
	_clockval = c; _start = s; 
	_body = ss;
	session::created++;
	}
    ~session_state() {
	session::deleted++;
	}

    int _argc;
    char** _argv;
    char* _name;
    int _flags;
    double _clockval;
    double _start;

    //session* _body; // is defined in state
    };

#define STATE ((session_state*)_state)
#define BODY  (STATE->_body)  
#define HUSHDELETE if (_state && (--STATE->refs == 0)) delete 


#undef statistics

void session::statistics(char*) { dvstatistics::report(); }

session::session(session* x) {
	session::initialize();
	_state = new session_state();
	BODY = x;
}

session::session(const char* kind, int flags)  { // AE Incomplete
    if (!session::_default)
    	session::_default = this;
    _state = 0;
    //session::declare("global",this);
    if (! session::initialize()) {
	//AE err() << "ERROR session::initialize()" << endl;
	require( 0 );
	}
    _state = new session_state(0,0,(char*)kind,flags);
    //AE promise ( tk && BODY );
}


session::session(int _argc, char* _argv[], const char* nm, int flag) {
    //cerr << "Session " << nm << endl;
    if (!session::_default)
    	session::_default = this;
    session::declare("global",this);
    if (!nm) session::_nowin = 1;
    if (! session::initialize()) {
	//AE err() << "ERROR session::initialize()" << endl;
	require( 0 );
	}
    //cerr << "Session initialized" << endl;
    _state = new session_state(_argc,_argv,(char*)nm,flag);
    if (0 && nm) {
    	rename(nm);
    	session::declare(nm,this);
	}
    //create(_argc,_argv,nm,flag);
    }

static int _max = 0;
int
session::create(int _argc, char* _argv[], const char* _name, int flag) {
    if (_max++) return 1; 
    if (!session::_default)
    	session::_default = this;
    if (!_create) _create = a_new_session;
    BODY = CREATE(_name);
    //cerr << "Creating session " << (void*) this << " " << (void*) BODY << endl;
    if ( BODY && ((BODY != this) && (BODY != session::_default))) {
			//cerr << "session for " << _name << endl;
			BODY->create(_argc,_argv,(char*)_name,flag);
			//cerr << "session for " << _name << endl;
			}
    ::_thesession = this;
    //if (!tk) tk = new class kit((char*)_name);
    if (!_event) {
	_event = new event(tk,_argc,_argv);
	_register(_event);
	}
    if (!kit::_default) {
		kit::_default = new class kit ((char*)_name);
		_register (kit::_default);
		}
       tk = kit::_default;
    if( BODY && (BODY != this) && (BODY != session::_default))
					BODY->install(this);
    //_register(_event);// _register(BODY); 
    _trace = 0;
    tk->trace(0);
    //promise ( tk && BODY);
    return 1;
}

session::~session() { 
	HUSHDELETE STATE;
	}


void session::depends(const char* s) {
if (!s) _depends = 0;
else {
	strcpy(depbuf,s);
        session::_depends = (char*)depbuf;
	}
}

int session::load(const char* lib) {
	if (_depends) sprintf(depbuf,"%s:%s",_depends,lib);
	else if (lib) sprintf(depbuf,"%s",lib);
	_depends = depbuf;
	return 1;
	}

void session::init(const char* f, void* p) {
    if (BODY) BODY->init(f,p);
}

void session::trace(int i) { _trace = i; if (tk) tk->trace(i); }
int session::tracelevel() const { return _trace; }


void session::install( void* p ) {
    require( p );
    require( BODY != this );
    if (BODY) BODY->install(p);
}

void session::prelude() {
		cerr << "session::prelude is empty" << endl;
		}

int session::main() { 
	      cerr << "session::main is empty" << endl;
	      return 0;
	      }

void session::map(int fl) { if(BODY) BODY->map(fl); }

int session::loop(int fl) { return BODY?BODY->loop(fl):0; }

int session::body(int fl) { return BODY?BODY->body(fl):0; }


int session::run( int proc(class kit*,int,char**) ) {
    if (session::_depends) {
	string s = _depends;
	s.split(":");
        string* p = 0;
	iter<string>& it = s;
	loader l;
	while ( (p = it()) ) {
		//cerr << "Loading with " << (char*) *p << endl;
		l.load((char*)*p);
		}
	}
    if (STATE)
      create(STATE->_argc,STATE->_argv,STATE->_name,STATE->_flags);
    if (BODY) {
	cerr << "session::run -> body" << endl;
    	return BODY->run(proc);
    } else {
	cerr << "session::run -> ..." << endl;
    this->prelude();
    return this->main();
    }
}

double session::clock() const { return STATE->_clockval; }
double session::clock(double val) { 
    double tmp = clock();
    STATE->_clockval = val;
    return tmp;
}

double session::time() const {   // global inittime !!
    double curacttime = 0;
#ifndef NOTIME
    struct timeval tv;
    struct timezone tz;
    
    if (!_inittime++) gettimeofday(&inittime,NULL);
    // set actual starting time
    gettimeofday(&tv,&tz);
    const long delta_sec = tv.tv_sec - inittime.tv_sec;
    const long delta_usec = tv.tv_usec - inittime.tv_usec;
    curacttime =  (double) delta_sec + 
	(double) delta_usec / (double) 1000000L;
#endif
    return curacttime;
}

   

kit* session::thekit() const { return tk; }

int session::mode() const { return BODY?BODY->mode():0; }
int session::mode(int i) { return BODY?BODY->mode(i):0; }
int session::mode(const char* i) { return BODY?BODY->mode(i):0; }



// And the rest

char* session::version() {
 char* v = _version;
 v[0] = 0;
#if defined(VERSION) && defined(__DATE__)
 sprintf(v,"%s %s",VERSION,__DATE__);
#endif
#ifdef __GNUG__
 strcat(v, " - GNU g++ ");
#ifdef __VERSION__
 strcat(v, __VERSION__);
#endif
#endif
#ifdef __SUNPRO_CC
 char buf[124];
 strcat(v, " - Sun CC %0x");
 strcpy(buf, v);
 sprintf(v, buf, __SUNPRO_CC);
#endif
 return v;
}




// compiler flags
//	Demo of "interesting" compiler defined flags.
//	Compile this with cc/CC/gcc/g++/etc
//	and check the output..
// Additonal information obtained from:
// richard@ihgp.ih.att.com --  Richard A. Rausch AT&T Bell Laboratories


int session::compilerflags() {

#ifdef __GNUC__
#if __GNUC__ < 2 || (__GNUC__ == 2 && __GNUC_MINOR__ <= 7)
	printf("This version (=< 2.7.X) does not like handler::handler()\n");
	printf("%d.%d\n", __GNUC__,  __GNUC_MINOR__);
#else
	printf("I hope this version (> 2.7.X) does like handler::handler()\n");
	printf("%d.%d\n", __GNUC__,  __GNUC_MINOR__);
#endif
#endif

#ifdef __GNUG__
	printf("-D__GNUG__ (This is GNU C++)\n");
#endif

#ifdef __VERSION__
	printf("-D__VERSION__=%s\n", __VERSION__);
#endif

#ifdef __STDC__
	printf("-D__STDC__ (This is an ANSI Standard C compiler)\n");
#endif

#ifdef __cplusplus
	printf("-D__cplusplus (obsolete)\n");
#endif

#ifdef __BUILTIN_VA_ARG_INCR
	printf("-D__BUILTIN_VA_ARG_INCR\n");
#endif

#ifdef __GCC_NEW_VARARGS__
	printf("-D__GCC_NEW_VARARGS__\n");
#endif

#ifdef __i386
	printf("-D__i386\n");
#endif

#ifdef __sparc
	printf("-D__sparc\n");
#endif

#ifdef __SUNPRO_CC
	printf("-D__SUNPRO_CC=0x%0x\n",__SUNPRO_CC);
#endif

#ifdef __SVR4
	printf("-D__SVR4\n");
#endif

#ifdef __unix
	printf("-D__unix\n");
#endif

#ifdef sparc
	printf("-Dsparc\n");
#endif

#ifdef sun
	printf("-Dsun\n");
#endif

#ifdef unix
	printf("-Dunix\n");
#endif
return 0;
}

