#include <hush/source.h>
#include <hush/midi/handler.h>
#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <string.h>

//#include <hush/io.h>
#include <hush/iostream.h>

#include <hush/midi/file.h>

midi_handler::midi_handler() {
	_handler = this;
	_master = 0;
	_in_seconds = 0;
	}

int midi_handler::read(const char* f, const char* out) {
	cerr << "midi_handler::read " << (f?f:"") << " " << (out?out:"") << endl;
	if (!_master) {
		_master = new midi_file();
		_master->_handler = this;
		}
	((midi_file*)_master)->midifile(f,out);
	return 1;
	}

int midi_handler::write(const char* f, int format, int trcks, int divis) {
	if (!_master) {
		_master = new midi_file();
		_master->_handler = this;
		}
	((midi_file*)_master)->write(f,format,trcks,divis);
	return 1;
	}

int midi_handler::open(const char* f, const char* mode) {
	if (!_master) {
		_master = new midi_file();
		_master->_handler = this;
		}
	((midi_file*)_master)->open((char*)f,mode);
	return 1;
	}

int midi_handler::close() {
	return _master?_master->close():0;
	}

FILE* midi_handler::in_file() { return _master?_master->in_file():stdin; }
FILE* midi_handler::out_file() { return _master?_master->out_file():stdout; }

int midi_handler::division() { return _master?_master->division():0; }
int midi_handler::track() { return _master?_master->track():0; }
int midi_handler::tracks() { return _master?_master->tracks():0; }
int midi_handler::tempo() { return _master?_master->tempo():0; }
char* midi_handler::file() { return _master?_master->file():0; }
long midi_handler::time() { return _master?_master->time():0; }
double midi_handler::seconds() { return _master?_master->seconds():0; }

int midi_handler::in_seconds() { return _in_seconds; }

int midi_handler::midi_getc() {
	return in_file()?fgetc(in_file()):getc(stdin);
	}

int midi_handler::midi_error( char *s )
{
	fprintf(stderr,"Error: %s\n",s);
	return 0;
}

int midi_handler::midi_header(int format, int ntrks, int ldivision) 
{
	fprintf(out_file(),"header(%s,[format=%d,ntrks=%d,division=%d]).\n","current",format,ntrks,ldivision);
	return 0;
}

int midi_handler::midi_trackstart()
{
	return 0;
}

int midi_handler::midi_trackend()
{
	//printf("Track end\n");
	return 0;
}

int midi_handler::midi_noteon(int chan, int pitch, int vol)
{
	midi_time();
	fprintf(out_file(),"note_on:[chan=%d,pitch=%d,vol=%d]).\n",chan+1,pitch,vol);
	return 0;
}

int midi_handler::midi_noteoff(int chan, int pitch, int vol)
{
	midi_time();
	fprintf(out_file(),"note_off:[chan=%d,pitch=%d,vol=%d]).\n",chan+1,pitch,vol);
	return 0;
}

int midi_handler::midi_pressure(int chan, int pitch, int press)
{
	midi_time();
	fprintf(out_file(),"pressure:[chan=%d,pitch=%d,press=%d]).\n",chan+1,pitch,press);
	return 0;
}

int midi_handler::midi_parameter(int chan, int control, int value)
{
	midi_time();
	fprintf(out_file(),"parameter:[chan=%d,c1=%d,c2=%d]).\n",chan+1,control,value);
	return 0;
}

int midi_handler::midi_pitchbend(int chan, int msb, int lsb)
{
	midi_time();
	fprintf(out_file(),"pitch_bend:[chan=%d,msb=%d,lsb=%d]).\n",chan+1,msb,lsb);
	return 0;
}

int midi_handler::midi_program(int chan, int program)
{
	midi_time();
	fprintf(out_file(),"program:[chan=%d,program=%d]).\n",chan+1,program);
	return 0;
}

int midi_handler::midi_chanpressure(int chan, int press)
{
	midi_time();
	fprintf(out_file(),"channel_pressure:[chan=%d, pressure=%d]).\n",chan+1,press);
	return 0;
}

int midi_handler::midi_sysex(int leng, char * mess)
{
	midi_time();
	fprintf(out_file(),"sysex:[leng=%d]).\n",leng);
	return 0;
}

int midi_handler::midi_metamisc(int type, int leng, char* mess)
{
	midi_time();
	fprintf(out_file(),"meta_event(unrecognized):[type=0x%02x,leng=%d]).\n",type,leng);
	return 0;
}

int midi_handler::midi_seqspecific(int type, int leng, char* mess)
{
	midi_time();
	fprintf(out_file(),"meta_event(sequencer-specific):[type=0x%02x,leng=%d]).\n",type,leng);
	return 0;
}

int midi_handler::midi_text(int type, int leng, char* mess)
{
	static char *ttype[] = {
		NULL,
		"Text Event",		/* type=0x01 */
		"Copyright Notice",	/* type=0x02 */
		"Sequence/Track Name",
		"Instrument Name",	/* ...       */
		"Lyric",
		"Marker",
		"Cue Point",		/* type=0x07 */
		"Unrecognized"
	};
	int unrecognized = (sizeof(ttype)/sizeof(char *)) - 1;
	register int n, c;
	register char *p = mess;

	if ( type < 1 || type > unrecognized )
		type = unrecognized;
	midi_time();
	fprintf(out_file(),"meta_text:[type=0x%02x, tt='%s',leng=%d,\n",type,ttype[type],leng);
	fprintf(out_file()," text='");
	for ( n=0; n<leng; n++ ) {
		c = *p++;
		if ( c != '\n') 
		   fprintf(out_file(), (isprint(c)||isspace(c)) ? "%c" : "\\0x%02x" , c);
	}
	fprintf(out_file(),"']).\n");
	return 0;
}

int midi_handler::midi_seqnum(int num)
{
	midi_time();
	fprintf(out_file(),"meta_event : [ sequence_number = %d]).\n",num);
	return 0;
}

int midi_handler::midi_eot()
{
	midi_time();
	fprintf(out_file(),"meta_event(end_of_track):[]).\n");
	return 0;
}

int midi_handler::midi_keysig(int sf,int mi)
{
	midi_time();
	fprintf(out_file(),"key_signature:[sharp/flats=%d, minor=%d]).\n",sf,mi);
	return 0;
}

int midi_handler::midi_tempo(long ltempo)
{
	midi_time();
	fprintf(out_file(),"tempo:[microseconds_per_quarter_note=%d]).\n",(int)ltempo);
	return 0;
}

int midi_handler::midi_timesig(int nn, int dd, int cc,int bb)
{
	int denom = 1;
	while ( dd-- > 0 )
		denom *= 2;
	midi_time();
	fprintf(out_file(),"time:[signature=%d/%d, clocks/click=%d, 32-notes/24-midi-clocks=%d]).\n",
		nn,denom,cc,bb);
		return 0;
}

int midi_handler::midi_smpte(int hr, int mn, int se, int fr,int ff)
{
	midi_time();
	fprintf(out_file(),"smpte:[hour=%d,minute=%d,second=%d,frame=%d,fract-frame=%d]).\n",
		hr,mn,se,fr,ff);
		return 0;
}

int midi_handler::midi_arbitrary(int leng, char* mess)
{
	midi_time();
	fprintf(out_file(),"arbitrary:[leng=%d]).\n",leng);
	return 0;
}

int midi_handler::midi_time()
{
    if(in_seconds())
	fprintf(out_file(),"midi_event('%s',%d,time=%f,   ",file(),track(),seconds());
    else
	fprintf(out_file(),"midi_event('%s',%d,time=%ld, ",file(),track(),time());
	return 0;
}

//@ writing

int midi_handler::midi_putc(char c) {
	return out_file()?fputc(c,out_file()):putc(c,stdout);
	}

int midi_handler::midi_writetrack(int track)
{
    return 0;
    int i;
    char data[2];

    mf_write_tempo((long)500000); /* 120 beats/per/second */

    for(i = 1 ; i < 128; i++){
       data[0] = i; /* note number */ 
       data[1] = 64; /* velocity */ 
       write_event(480,1, note_on, data,  2);
       write_event(480,1, note_off, data,  2);
       //if(!mf_write_midi_event((unsigned long)480,(unsigned int)1,(unsigned char) note_on, (unsigned char*) data, (unsigned long) 2)) return(-1);
       //if(!mf_write_midi_event((unsigned long)480,(unsigned int)1,(unsigned char) note_off, (unsigned char*) data, (unsigned long) 2)) return(-1);
    }
    return(1);
} /* end of write_track() */

int midi_handler::midi_writetempotrack() {
	return 1;
	}


void midi_handler::mfwrite(int format, int ntracks, int division, FILE* fp) {
	if (_master) _master->mfwrite(format,ntracks,division,fp);
	}

int midi_handler::mf_write_midi_event( unsigned long delta_time, unsigned int chan, unsigned int type, unsigned char *data, unsigned long size) 
{
	return _master?_master->mf_write_midi_event(delta_time,chan,type,data,size):0;

	}
int midi_handler::mf_write_meta_event( unsigned long delta_time,
		unsigned char type, unsigned char* data,  unsigned long size) {
	return _master?_master->mf_write_meta_event(delta_time,type,data,size):0;
	}	
void midi_handler::mf_write_tempo( unsigned long tempo ) {
	if (_master) _master->mf_write_tempo(tempo);
	}


void midi_handler::write_header(int format, int ntracks, int division) {
	if (_master) _master->write_header(format,ntracks,division); 
	}

int midi_handler::begin_track(int trck) {
	return _master?_master->begin_track(trck):0;
	}

int midi_handler::end_track() {
	return _master?_master->end_track():0;
	}

void midi_handler::write_event(int delta, int chan, int type, const  char* data, int size) {
	if (_master)
		_master->write_event(delta,chan,type,data,size);
	}

void midi_handler::write_meta(int delta, int type, const  char* data, int size) {
	if (_master)
		_master->write_meta(delta,type,data,size);
	}

void midi_handler::write_tempo(int x) {
	if (_master)
		_master->write_tempo(x);
	}


char* midi_handler::note(int v)
{
	static char out[24];
	//char* primes[] = { "", "'","''","'''","''''","'''''" };
	char* primes[] = { "", "-1","-2","-3","-4","-5" };
	char* notes[] = { "c", "cis", "d", "dis", "e",
			"f", "fis", "g", "gis", "a",
			"ais", "b" };
	int x = v;
	int p = 0;
	if (v > 59) {
		while (x > 59) {
			x -= 12;
			p++;
			}
		}
	if (v < 48) {
		while (x < 48) {
			x += 12;
			p++;
			}
		}
	if (x >= 48  && x < 60) {
		sprintf(out,"%s%s",notes[x-48],primes[p]);
		}
	return (char*) out;
}
