#include <hush/source.h>
#include <hush/ml/ml.h>
#include <hush/ml/format.h>

#include <hush/session.h>
#include <hush/string.h>

#include <hush/iostream.h>
#include <string.h>
#include <stdio.h>

/*
ostream& operator<<(ostream& os, term& t) {
	os << t.str() << endl;
	return os;
	}

ostream& operator<<(ostream& os, term* t) {
	os << t->str() << endl;
	return os;
	}
*/

formatter* ml_term::_formatter = 0;

int ml_term::formatted() {
	if (!ml_term::_formatter) {
		formatter* x = ml_term::_formatter = new term_formatter(); 
		if (session::_default) session::_default->_register(x);
		}
	return 1;
	}

static int varstr(const char *f) {                  // Prolog style
	if (f[0] == '_' || ( f[0] >= 'A' && f[0] <= 'Z') )
		return 1;
	else
		return 0;
	}

ml_term::ml_term() : term(this) { 
	_tag = 0; _varno = 0; _f[0] = '\0'; _a1 = 0; _a2 = 0;
	}

ml_term::ml_term(const char* f) : term(this) {
	_tag = 0; _varno = 0; _a1 = 0; _a2 = 0;
	//cerr << "Making convar (ml) " << f << endl;
	s = _news(f);  // will be  registered
	if (!varstr(f)) {
		_tag = _con;
	} else {
		 _tag = _var;
		}
	//cerr << "Type: " << type() << endl;
	//cerr << "Isfun: " << _isfun() << endl;
} 


ml_term::ml_term(const char* f, const term* t1, const term* t2) : term(this)  {
	_tag = _fun; _varno = 0; // strcpy(_f,f);
	s = _news(f);
	if (iscomma(t1)) {
		term* x1 = a1of(t1);
		term* x2 = a2of(t1);
		_a1 = x1;
		_a2 = mk(",",x2,t2);
		_register(_a2);
	} else {
		_a1 = (term*) t1;
		_a2 = (term*) t2;
	}	
} 


ml_term::~ml_term() { 
	//*handler::_err  << "deleting " << this << endl; 
	}

//// As an iter


// Functions


int ml_term::type() const { return _tag; }
char* ml_term::fun() const { return (char*) s; }

int ml_term::nargs() const { 
	if (_a1 && _a2) return 2;
	else if (_a1) return 1;
	else return 0;
	}

int ml_term::narity( ) const {
	if (iscomma(this) && _a2) return 1 + _a2->narity();
	else return 1;
	}

int ml_term::arity() const { 
	if ( _iscon() || _isvar() ) return 0;
	else if (_a1 && _a2) return 1 + _a2->narity();
	else return 1;
	}

term* ml_term::a1() const { return (term*) _a1; }
term* ml_term::a2() const { return (term*) _a2; }

term* ml_term::nth( int i ) const { 
	if (i == 0) return (term*) this;
	else if (iscomma(this)) {
		if (i==1) return (term*) _a1;
		else if (i==2) return (term*) _a2;
		else if (_a2 && iscomma(_a2)) return (term*) _a2->nth(i-1);
		}
	else return 0;
	}

term* ml_term::arg( int i ) const { 
	if (i == 0) return (term*) this;
	else if (isfun(this)) {
		if (i==1) return (term*) _a1;
		else if (i==2) return (term*) _a2;
		else if (_a2 && iscomma(_a2)) return (term*) _a2->nth(i-1);
		}
	return 0;
	}

int ml_term::_isnil(const char*) const { return _iscon("[]"); }
int ml_term::_isnum(const char*) const { return 0; }
int ml_term::_iscon(const char* f) const {
	return (type() == _con) && (f?!strcmp(f,s):1);
	}
int ml_term::_isvar(const char* f) const {
       return ( type() == _var) && f?!strcmp(f,s):1;
	}
int ml_term::_isfun(const char* f) const {
       return ( type() == _fun) && (_a1 != 0) && (f?!strcmp(f,s):1);
	}

int ml_term::_ismeta(const char* f, int nar) const {
       int b = 0;
       if (s && s[0]=='#' && s[strlen(s)]==':') b = 1;  
       if (f) b = (b && !strcmp(s,f));
       if (nar>=0) b = (b && (nargs()==nar)); 
       return b;
       }

//char* ml_term::id() const { return (char*) _f; }

/// unparse

char* ml_term::str(formatter* f) const {
   //cerr << "Formatting with " << f << endl;
   if (f) {
	return f->_format(this); // Wall
	}
   else if (formatted()) return ml_term::_formatter->_format(this);
   else return "";
   }
