#include <hush/source.h>
#include <hush/ml/parser.h>
#include <hush/ml/scanner.h>
#include <hush/string.h>
#include <hush/iostream.h>

#define TYPE parser
#include "hush/sys/table.i"

int parser::initialized = 0;

int parser::initialize() {
if (!parser::initialized) {
	parser::initialized = 1;
	}
return parser::initialized;
}

char* parser::category() const { return "parser"; }

void parser::soinc(char* s, char* ext) {
if (_body()) _body()->soinc(s,ext);
}

void parser::files(char* s, char* ext) {
if (_body()) _body()->files(s,ext);
}

parser::parser(const char* k) {
	if (k) {
		;
		}
	}

parser::parser() {
_garbage = 0; _error = 0;
yylineno = 0; yycontext = 0; yytchar = 0;
yyout = &cout; yyin = &cin; yyerr = &cerr;
yyiter = 0; _scanner = 0; yyiter = 0;
}

parser::parser(iter<term>* l) : iter<term>(this) {
//cerr << "creating parser" << endl;
_garbage = 0; _error = 0;
yylineno = 0; yycontext = 0; yytchar = 0;
yyout = &cout; yyin = &cin; yyerr = &cerr;
yyiter = l; _scanner = 0; yyiter = 0;
}

parser::parser(scanner* l) : iter<term>(this) {
_garbage = 0; _error = 0;
yylineno = 0; yycontext = 0; yytchar = 0;
yyout = &cout; yyin = &cin; yyerr = &cerr;
yyiter = l; _scanner = l; _parser = 0;
}

parser::parser(void *x) : iter<term>((iter<term>*) x) {
_garbage = 0; _error = 0;
yylineno = 0; yycontext = 0; yytchar = 0;
yyout = &cout; yyin = &cin; yyerr = &cerr; 
_scanner = 0; _parser = 0; yyiter = 0;
}

parser::~parser() {
	if (_garbage) delete _garbage;
	}

void parser::init(istream* in, ostream* out) {
	if (in && out) ;//Wall
	}

parser* parser::create(istream* in, ostream* out) {
	if (in && out) ;//Wall
	return 0;
	}

term* parser::operator()() {
	cerr << "parser::()" << endl;
	if (_body()) return _body()->operator()();
	cerr << "ERROR: parser::()" << endl;
	return 0;
	}

int parser::lineno() { return _scanner?_scanner->yylineno:yylineno; }
char* parser::error() { return " --???-- "; }

parser* parser::_body() {
	if (_parser && (_parser != this)) return _parser;
	else return 0;
	}

int parser::yylex() {
	int tok = 0;
	if (_body()) return _body()->yylex();
	term* x = (*yyiter)();
	yylval = (term*) x;
	if (yylval) tok = (int) yylval->n;
	//cerr << "parser::yylex (" << tok << ")" << strof(x) << endl;
	if (tok<=0 && yylval) yylval->s = "";
	return (int) tok;;
	}

void parser::op(const char* k, const char* r, int i) { _scanner->op(k,r,i); }
void parser::left(const char* k, const char* r, int i) { _scanner->left(k,r,i); }
void parser::right(const char* k, const char* r, int i) { _scanner->right(k,r,i); }
void parser::pre(const char* k, const char* r, int i) { _scanner->pre(k,r,i); }
void parser::post(const char* k, const char* r, int i) { _scanner->pre(k,r,i); }
void parser::specifier(const char* k, const char* r, int i) { _scanner->specifier(k,r,i); }
void parser::declarative(const char* k, const char* r, int i) { _scanner->declarative(k,r,i); }
void parser::qualifier(const char* k, const char* r, int i) { _scanner->qualifier(k,r,i); }

void parser::modality(const char* k, const char* r, int i) { _scanner->modality(k,r,i); }
void parser::meta(const char* k, const char* r, int i) { _scanner->meta(k,r,i); }

void parser::stop(const char* k, const char* r, int i) { _scanner->stop(k,r,i); }
void parser::command(const char* k, const char* r, int i) { _scanner->command(k,r,i); }
void parser::atom(const char* k, const char* r, int i) { _scanner->command(k,r,i); }
void parser::type(const char* k, const char* r, int i) { _scanner->type(k,r,i); }

void parser::end(const char* k, const char* r, int i) { _scanner->end(k,r,i); }

void parser::alias(const char* k, const char* r, int i) { _scanner->alias(k,r,i); }

