#include <hush/source.h>
#include <hush/ml/scanner.h>
#include <hush/dictionary.h>
#include <hush/string.h>

#define TYPE scanner
#include "hush/sys/table.i"
#include <hush/sys/state.h>

#define noflevels 8

#define STATE ((scanner_state*)_state)

int scanner::initialized = 0;

int scanner::initialize() {
if (!scanner::initialized) {
	scanner::initialized = 1;
	}
return scanner::initialized;
}

struct scanner_state : public dvstate {

dictionary<string>* _op[noflevels];
dictionary<string>* _left[noflevels];
dictionary<string>* _right[noflevels];
dictionary<string>* _pre[noflevels];
dictionary<string>* _post[noflevels];

dictionary<string>* _spec[noflevels];
dictionary<string>* _decl[noflevels];
dictionary<string>* _qual[noflevels];

dictionary<string>* _mode[noflevels];
dictionary<string>* _meta[noflevels];

dictionary<string>* _stop[noflevels];
dictionary<string>* _command[noflevels];
dictionary<string>* _atom[noflevels];
dictionary<string>* _type[noflevels];

dictionary<string>* _end[noflevels];

dictionary<string>* _alias[noflevels];


#define init(_op)  \
	if (_op) { \
		for (int i = 0; i < noflevels; i++ ) { \
			_op[i] = 0; \
			} \
		} \

scanner_state() {

	init(_op);
	init(_left);
	init(_right);
	init(_pre);
	init(_post);

	init(_spec);
	init(_decl);
	init(_qual);

	init(_mode);
	init(_meta);

	init(_stop);
	init(_command);
	init(_atom);
	init(_type);

	init(_end);

	init(_alias);
	}

#define ndel(_op)  \
	if (_op) { \
		for (int i = 0; i < noflevels; i++ ) { \
			if (_op[i]) delete _op[i]; \
			} \
		} \

~scanner_state() {


	ndel(_op);
	ndel(_left);
	ndel(_right);
	ndel(_pre);
	ndel(_post);

	ndel(_spec);
	ndel(_decl);
	ndel(_qual);

	ndel(_mode);
	ndel(_meta);

	ndel(_stop);
	ndel(_command);
	ndel(_atom);
	ndel(_type);

	ndel(_end);

	ndel(_alias);
	}

};// state

scanner::scanner(scanner* x) : parser((void*)x) { // iter<term>(x) { 
	_scanner = x; _state = 0;
	}

scanner::scanner(istream* in, ostream* out) : //
				parser((void*)this) {
				// iter<term>(this) {
yyin = in; yyout = out; _state = 0;
}


scanner:: ~scanner() {

	if (STATE) delete STATE;


//cerr << "Deleting scanner" << endl;
}


scanner* scanner::_body() {
	//if (_scanner && (_scanner != this)) return _scanner;
	return 0;
	}

term* scanner::operator()() {
	if (_body()) return _body()->operator()();
	cerr << "scanner::operator() **ERROR**" << endl; 
	return 0;
	}

/*
int scanner::yywrap() {
	if (_body()) return _body()->yywrap();
	//cerr << "calling wrap" << endl;
	return(1);
	}
*/




char* scanner::get(const char* k, int* n) {
char* s = 0;
int tok = 0;
int level = 0;
if (!n) {
	cerr << "ERROR: no token address in scanner::get" << endl;
	return 0;
	}
if ( (s = op(k,&level)) ) tok = _op() + level;
else if ( (s = left(k,&level)) ) tok = _left() + level;
else if ( (s = right(k,&level)) ) tok = _right() + level;
else if ( (s = pre(k,&level)) ) tok = _pre() + level;
else if ( (s = post(k,&level)) ) tok = _post() + level;

else if ( (s = specifier(k,&level)) ) tok = _specifier() + level;
else if ( (s = declarative(k,&level)) ) tok = _declarative() + level;
else if ( (s = qualifier(k,&level)) ) tok = _qualifier() + level;

else if ( (s = modality(k,&level)) ) tok = _modality() + level;
else if ( (s = meta(k,&level)) ) tok = _meta() + level;

else if ( (s = stop(k,&level)) ) tok = _stop() + level;
else if ( (s = command(k,&level)) ) tok = _command() + level;
else if ( (s = atom(k,&level)) ) tok = _atom() + level;
else if ( (s = type(k,&level)) ) tok = _type() + level;

else if ( (s = end(k,&level)) ) tok = _end() + level;
if (n && *n) *n = tok;
return s;
}

int scanner::_op() { return 257; }
int scanner::_left() { return _op()+noflevels; }
int scanner::_right() { return _left()+noflevels; }
int scanner::_pre() { return _right()+noflevels; }
int scanner::_post() { return _pre()+noflevels; }
int scanner::_specifier() { return _post()+noflevels; }
int scanner::_declarative() { return _specifier()+noflevels; }
int scanner::_qualifier() { return _declarative()+noflevels; }
int scanner::_modality() { return _qualifier()+noflevels; }
int scanner::_meta() { return _modality()+noflevels; }
int scanner::_stop() { return _meta()+noflevels; }
int scanner::_command() { return _stop()+noflevels; }
int scanner::_atom() { return _command()+noflevels; }
int scanner::_type() { return _atom()+noflevels; }
int scanner::_end() { return _type()+noflevels; }

#define put_table(op,_op) void \
 scanner::op(const char* ki, const char* ri, int ii) { \
	int i = ii; char* r = (char*) ri; char* k = (char*)ki; \
	scanner_state* st = STATE; \
	if (!r) r = (char*) k; \
	if (!st) _state = (void*) (st = new scanner_state()); \
        if (i >= noflevels ) i = noflevels - 1; \
	if (st->_op[i] == 0) st->_op[i] = new dictionary<string>(1); \
	(*((st->_op)[i]))[k] = new string(r); \
	} 


put_table(op,_op)
put_table(left,_left)
put_table(right,_right)
put_table(pre,_pre)
put_table(post,_post)

put_table(specifier,_spec)
put_table(declarative,_decl)
put_table(qualifier,_qual)

put_table(modality,_mode)
put_table(meta,_meta)

put_table(stop,_stop)
put_table(command,_command)
put_table(atom,_atom)
put_table(type,_type)

put_table(end,_end)

put_table(alias,_alias)


#define get_table(op,_op) \
char* scanner::op(const char* k, int* n ) {  \
	char* r = _body()?_body()->op(k,n):0; if (r) return r;  \
	scanner_state* st = STATE; \
	if (!st) return 0; \
	string* p = 0; int i =  n?*n:0;  \
	if (i>=noflevels) i = noflevels - 1;   \
	if (st->_op) {  \
		for ( ; !p && i<noflevels ; i++) {  \
			if (st->_op[i]) p = (*(st->_op[i]))[k];  \
			}  \
	if (p && n) *n = i-1;  \
	if (p) r = (char*) *p; return r;   \
	}  \
return r; \
} \

get_table(op,_op)
get_table(left,_left)
get_table(right,_right)
get_table(pre,_pre)
get_table(post,_post)

get_table(specifier,_spec)
get_table(declarative,_decl)
get_table(qualifier,_qual)

get_table(modality,_mode)
get_table(meta,_meta)

get_table(stop,_stop)
get_table(command,_command)
get_table(atom,_atom)
get_table(type,_type)

get_table(end,_end)

get_table(alias,_alias)


char* scanner::value(const char* k) {
	//return k;
	if (!k) return 0;
	char* p = alias(k);
	if (p) return p;
	else return (char*)k;
	return (char*) (p?p:k);
	}

term* scanner::token(int tok, const char* s, term* v ) {
	term* x = new term(tok,value(s),v);
	if (!_garbage) _garbage = new handler();
	if (_garbage) _garbage->_register(x);
	else cerr << "Warning: no garbage collect" << endl;
	return x;
	}

