#include <hush/source.h>
#include <hush/session.h>

#include <hush/term.h>
#include <hush/ml/ml.h>

#include <hush/ml/format.h>
#include <hush/ml/parser.h>

#include <hush/iostream.h>
#include <string.h>
#include <stdio.h>

#include <hush/sys/light.h>

handler* term::_garbage = 0;

parser* term::_scanner = 0;
parser* term::_parser = 0;

term* term::_default = 0;
int term::_noformat = 0;
formatter* term::_formatter = 0;

void term::_hold(const term* t) {
	//static int nn = 0;
	handler *h = term::_garbage;
	if (!h) {
		h = term::_garbage = new handler();
		if (session::_default) session::_default->_register(h);
		}
	//cerr << "registering " << (void*) t << " " << nn++ << endl;
	if (t) h->_register(t);
	}

void term::_cleanup() {
	handler *h = term::_garbage;
	if (h) h->_cleanup();
	//delete h; 
	term::_garbage = 0;
	}


#define TYPE term
#include "hush/sys/table.i"

/*
ostream& operator<<(ostream& os, term& t) {
	os << t.str() << endl;
	return os;
	}

ostream& operator<<(ostream& os, term* t) {
	os << t->str() << endl;
	return os;
	}
*/

static term* thedefault() {
	if (!term::_default) { 
		term* x = term::_default = new ml_term();
		if (session::_default) session::_default->_register(x);
		}
	return term::_default;
	}

static int formatted() {
	if (!term::_noformat && !term::_formatter) {
		formatter* x = term::_formatter = new term_formatter(); 
		if (session::_default) session::_default->_register(x);
		}
	return !term::_noformat;
	}


static int varstr(const char *f) {                  // Prolog style
	if (f[0] == '_' || ( f[0] >= 'A' && f[0] <= 'Z') )
		return 1;
	else
		return 0;
	}

char* term::_news(const char* f) {
	if (!f) {
		cerr << "Warning: (null) functor";
		if (_scanner) {
			cerr << "  line: " << _scanner->yylineno;
			}
		cerr << endl;
		return "undefined";
		}
	char* p = new char[strlen(f)+1]; 
	strcpy(p,f);
	_register(p);
	return p;
	}

term* term::_body() const { return (v && (v!=this))?(term*)v:0 /*_body*/; }

term::term() { term::created++; _light = 0; n=0; s=0; v /*_body*/ =0;  }

term::term(int tok, const char* f, term* x) {
	term::created++; _light = 0;
	n = tok;
	s = _news(f);
	v = x;
	}

term::term(const char* f) {
	term::created++; _light = 0;
	n=0; s=0; v /*_body*/ =0; 
	//cerr << "Making convar (t) " << endl;
	if (!varstr(f))
		v /*_body*/ = thedefault()?_default->mk(f):0; 
	else
		v /*_body*/ = thedefault()?_default->mk(f):0;
	if (v /*_body*/) _register(v /*_body*/);
	} 

term::term(int i) {
	term::created++; _light = 0;
        n=0; s=0; v /*_body*/ = 0; 
	v /*_body*/ = thedefault()?_default->mk(i):0;
	if (v /*_body*/) _register(v /*_body*/);
	}

term::term(float i) {
	term::created++; _light = 0;
        n=0; s=0; v /*_body*/ = 0; 
	v /*_body*/ = thedefault()?_default->mk(i):0;
	if (v /*_body*/) _register(v /*_body*/);
	}

term::term(double i) {
	term::created++; _light = 0;
        n=0; s=0; v /*_body*/ = 0; 
	v /*_body*/ = thedefault()?_default->mk(i):0;
	}

term::term(void* x) {
	term::created++; _light = 0;
        n=0; s=0; v /*_body*/ =0;  _light = 0;
	v /*_body*/ = thedefault()?_default->mk(x):0;
	if (v /*_body*/) _register(v /*_body*/);
	}

term::term(const char* f, const term* a1, const term* a2) {
		term::created++; _light = 0;
		n=0; s=0; v=0;
		v /*_body*/ = thedefault()?_default->mk(f,a1,a2):0;
		if (v /*_body*/) _register(v /*_body*/);
	} 

term::term(const char* f, const term& a1, const term& a2) {
		term::created++; _light = 0;
		n=0; s=0; v=0;
		v /*_body*/ = thedefault()?_default->mk(f,&a1,&a2):0;
		if (v /*_body*/) _register(v /*_body*/);
	} 

term::term(const char* f, const term& a1) {
		term::created++; _light = 0;
		n=0; s=0; v=0;
		v /*_body*/ = thedefault()?_default->mk(f,&a1):0;
		if (v /*_body*/) _register(v /*_body*/);
	} 

term::term(const term* f, const term* _a1, const term* _a2 ) {
		term::created++; _light = 0;
		n=0; s=0; v=0;
		v /*_body*/ = thedefault()?_default->mk(f,_a1,_a2):0;
		if (v /*_body*/) _register(v /*_body*/);
	} 

term::term(const term* t) { //AE?->
	term::created++; _light = 0;
	v /*_body*/ = (term*) t; // AE?
	}

term::term(const term& t) {
	term::created++; _light = 0;
	n=t.n; s= t.s; v=t.v; //AE?
	}

term& term::operator=(const term* t) { //AE - very subtle, or wrong
	v /*_body*/ = (term*) t;
	return *this;
	}

term& term::operator=(const term& t) {
	n=t.n; s= t.s; v=t.v; //AE?
	return *this;
	}

term::~term() { 
	//if (s && n>=0) { delete s; s = 0; n = -1; }
	term::deleted++;
	if (_light) delete _light; _light = 0;
	//*handler::_err  << "deleting " << this << endl; 
	}

// mk s

term* term::mk(const char* ) { return 0; }
term* term::mkvar(const char* ) { return 0; } // variable
term* term::mk(int ) { return 0; } 
term* term::mk(float ) { return 0; } 
term* term::mk(double ) { return 0; } 
term* term::mk(void* ) { return 0; } /// wildcard + assignment 
term* term::mk(const char* , const term* , const term* ) { return 0; }
term* term::mk(const term* , const term* , const term* ) { return 0; } // higher order


//// As an iter

char* term::operator()() { return _body()?_body()->next():0; }
char* term::next() { return _body()?_body()->next():0; }

//// General

int term::_isop(const char c) const { 
   char b[2]; b[0] = c; b[1] = '\0';
   return _isfun(b);
   }

int term::_opin(const char* s) const {
if (!_isfun()) return 0;
else {
 int n = strlen(s);
 for (int i = 0; i < n; i++) {
     if (_isop(s[i])) return 1;
     }
 }
 return 0;
}


// Functions

int term::nargs() const { return _body()?_body()->nargs():0; }
int term::arity() const { return _body()?_body()->arity():0; }
int term::narity() const { return _body()?_body()->narity():0; }


char* term::category() const { return _body()?_body()->category():0; }

int term::level() const { return _body()?_body()->level():0; }
int term::type() const { return _body()?_body()->type():0; }


int term::ival() const { return _body()?_body()->ival():0; }
float term::fval() const { return _body()?_body()->fval():0; }
double term::dval() const { return _body()?_body()->dval():0; }
char* term::aval() const { 
	char* s = (char*) str(); 
	char* p = s;
	while( p && *p ) {
		   if ((*p == '\n') || (*p == '\t')) *p = ' '; 
		   p++;
		   }
	//cerr << "aval: " <<  s << endl;
	return s;
	}

term* term::arg( int i ) const { return _body()?_body()->arg(i):0; }
term* term::nth( int i ) const { return _body()?_body()->nth(i):0; }

char* term::fun() const { return _body()?_body()->fun():0; }
term* term::a1() const { return  (_isfun()) ? arg(1) : 0; }
term* term::a2() const { return  (_isfun()) ? arg(2) : 0; }

int term::_iswild( const char* s) const { return _body()?_body()->_iswild(s):0; }


int term::_isnil(const char* s) const { return _body()?_body()->_isnil(s):0; }
int term::_isnum(const char* s) const { return _body()?_body()->_isnum(s):0; }
int term::_iscon(const char* s) const { return _body()?_body()->_iscon(s):0; }
int term::_isvar(const char* s) const { return _body()?_body()->_isvar(s):0; }
int term::_isfun(const char* s) const { return _body()?_body()->_isfun(s):0; }

int term::_ismeta(const char* s, int n) const { return _body()?_body()->_ismeta(s,n):0; }

//char* term::id() const { return _body()?_body()->id():0; }

int term::has(const char* s) const { return _body()?_body()->has(s):0; }
int term::has(const term* t) const { return _body()?_body()->has(t):0; }



int term::matches(const char* s) const { return _body()?_body()->has(s):0; }
int term::matches(const term* t) const { return _body()?_body()->has(t):0; }


term* term::varlist() const { return _body()?_body()->varlist():0; }

term* term::rename(int i) { return _body()?_body()->rename(i):this; }

/// unparse

char* term::str(formatter* f) const {
   if (!formatted()) cerr << "Warning: no formatter available" << endl;
   if (f) return f->_format(this);
   //cerr << "term: trying default format" << endl;
   if (_body()) return _body()->str(f);
   else if (formatted()) return term::_formatter->_format(this);
   else if (s) return s;
   else return "";
   }


#define LIGHT (((term*)this)->_light)

void term::_register( const handler* h) const {
	if (!_light) LIGHT = new light_garbage();
	LIGHT->_register(h);
	}
void term::_register( const term* t) const {
	if (!_light) LIGHT = new light_garbage();
	LIGHT->_register(t);
	}
void term::_register( const dvstring* s) const {
	if (!_light) LIGHT = new light_garbage();
	LIGHT->_register(s);
	}
void term::_register(const char* a) const {
	if (!_light) LIGHT = new light_garbage();
	LIGHT->_register(a);
	}


