// clienthandler.c

#include <hush/source.h>

#include <hush/net/client/MIME.h>
#include <hush/net/client/clienthandler.h>
#include <hush/event.h>
#include <hush/string.h>
#include <string.h>
#include <stdlib.h>

#ifdef __MSWIN__
#include <io.h> // for mktemp
#endif

static const char* TMPFILE	= "/tmp/tmp.XXXXXXX";
const char* GETCMD	= "urlget";
const char* GETCMD_TMP	= "urlget_tmp";
const char* HEADER_CMD	= "urlheader";
const char* TYPE_CMD	= "urltype";

void init_client(kit* tk)
{
	// cout << "Initing client handler" << endl;
	clienthandler* h = new clienthandler();
	tk->_register(h);
	tk->bind(GETCMD, h);
	tk->bind(GETCMD_TMP, h);
	tk->bind(HEADER_CMD, h);
	tk->bind(TYPE_CMD, h);
}

clienthandler::clienthandler(): handler()
{
	mode(handler::explic);
	c = new client();
}

clienthandler::~clienthandler()
{
	delete c;
}


int clienthandler::operator()()
{
	char* result;

	event* e = _event; // _event from handler

	// switch on event using string class ...
	string args(e->argc(), e->argv());
	string s = args(0);

	if(s == GETCMD) {
		if (e->argc()==2) {
		    result = get(args(1));
		    tk->result(result);
		} 
		else if (e->argc()==3) get(args(1), args(2));
	}
	else if(s == GETCMD_TMP) {
		char tmpname[56];
		ofstream tmp;
		strcpy(tmpname, TMPFILE);
		mktemp(tmpname);
		
		get(args(1), tmpname);
		tk->result(tmpname);
	}
	else if(s == HEADER_CMD) {
		result = header();
		tk->result(result);
	}
	else if(s == TYPE_CMD) {
		result = urltype();
		tk->result(result);
	}
	return 1;
}

char* clienthandler::get(const char* url)
{
	char* buf;
	int length;

	c->get(url);
	if (!c->send_request()) return "";
	if (!c->get_reply_body(buf, length)) return "";
	return buf;
}


void clienthandler::get(const char *url, const char* name)
{
	ofstream tmp;
	int length;
	char *buf;
	
	c->get(url);
	if(!c->send_request()) return;
	if(!c->get_reply_body(buf, length)) return;

	tmp.open(name);
	tmp.write(buf, length);
	tmp.close();
	return;
}




char* clienthandler::header()
{	
	char* buf;
	int length;

	if (!c->get_reply_header(buf, length)) return "";
	return buf;
}



char* clienthandler::urltype()
{
	MIME *m = c->media();
	if(m) return (char*) *m;
	else return "";
}
