// file_client.c
//
// implementation file for the fileclient class
//
// October 1995
// Bastiaan Sch\"onhage

#include <hush/net/client/file_client.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <string.h>
#include <ctype.h>
#include <stdlib.h>

static const int max_length = 16;            // must be local!
static const int max_name_length = 256;

file_client::file_client(): client(0)
{
	url_ = 0;
	buffer_ = (char*) 0;
	length_ = 0;
	file_ = 0;	
	name_ = new char[max_name_length+1];
	defaultmime_ = new MIME("application", "octet-stream");

	// This list can be extended/overruled in your $HOME/.mime.types
	mime_dict["ht"] = new MIME("text", "html");
	mime_dict["htm"] = new MIME("text", "html");
	mime_dict["html"] = new MIME("text", "html");
	mime_dict["sgml"] = new MIME("text", "sgml");
	mime_dict["gif" ] = new MIME("image", "gif");
	mime_dict["tcl" ] = new MIME("application", "x-tcl");
	mime_dict["mus" ] = new MIME("application", "amuse");
	mime_dict["avi" ] = new MIME("video", "x-msvideo");
	mime_dict["mov" ] = new MIME("video", "quicktime");
	mime_dict["wrl" ] = new MIME("x-world", "x-vrml");

	char* home = getenv("HOME");
	char* mimefile = new char[1024];
	sprintf(mimefile, "%s/.mime.types", home);

	read_mime_types(mimefile);
	delete[] mimefile;

}

file_client::~file_client()
{
	if(buffer_) delete buffer_;
	if(defaultmime_) delete defaultmime_;
	if(url_) delete url_;
}

const char* file_client::type(void) const
{
	return "file";
}

// build request
void file_client::get(const char* URL)
{
	if(url_) delete url_;
	url_ = new url(URL);
}

bool file_client::send_request()
{

	strcpy(name_, "/"); // AE 
	strcat(name_, url_->host());
	strcat(name_, "/");
	strcat(name_, url_->locator());
	const unsigned len = strlen(name_);
	if (name_[len-1] == '/') name_[len-1] = '\0';

	
	if(file_) fclose(file_);
	file_ = fopen( name_ , "r");
	if(file_ == NULL) {
	  cerr << "Could not open file " << name_ << endl;
	  return false;
	}
	got_reply_ = false;
	return true;
}
	
// get reply
bool file_client::get_reply(char* &buffer, int &length)
{
	struct stat status;

	if ( fstat( fileno(file_), &status) != 0) return false;
	length_ =  status.st_size;

	if(buffer_) delete[] buffer_;
	buffer_ = new char[length_+1];
	//if( fread(buffer_, 1, length_, file_) != (unsigned) length_ ) 
	unsigned n = fread(buffer_, 1, length_, file_);
	if (n != (unsigned) length_) return false;
	buffer_[n] = '\0';
	fclose(file_);
	buffer = buffer_;
	length = length_ = n;
	got_reply_ = true;
	return true;
}

bool file_client::get_reply_header(char* &, int &)
{
	return false;
}

bool file_client::get_reply_body(char* &buffer, int &length)
{
	if(!got_reply_) return get_reply(buffer, length);
	else 
	{
		buffer = buffer_;
		length = length_;
		return true;
	}
}

bool file_client::get_reply_trailer(char* &, int &)
{
	return false;
}


// parse reply
MIME* file_client::media()
{
	const char *s = url_->locator();
	const char *suffix = strrchr(s, '.'); suffix++;
	MIME* mime_ = mime_dict[suffix];
	if (!mime_) {
	  cerr << "Warning: No mime type for suffix: " << suffix << endl;
	  cerr << "using default: " << (char*) *defaultmime_ << endl;
	  mime_ = defaultmime_;
	}  
	return mime_;
}

int file_client::length(void) 
{
	return length_;
}

int file_client::errorcode(void)
{	
	return 0;
}


bool file_client::get_reply()
{
	struct stat status;

	if ( fstat( fileno(file_), &status) != 0) return false;
	length_ =  status.st_size;
	return true;
}

bool file_client::get_reply_body(int &fd)
{
	fd = fileno(file_);
	return true;
}

bool file_client::read_mime_types(char* filename) {
	ifstream dotmimetypes(filename);
	if (!dotmimetypes) return false;
	// else cerr << "Debug: reading " << filename << endl;
	while (dotmimetypes) {
		char buf[1024];
		char suffix[128];
		char mime_type[128];
		char subtype[128];
		dotmimetypes.getline(buf, 1024);
		
		char *comment = strchr(buf, '#');
		if (comment) *comment = '\0';

		char *start = buf; 
		while (*start && isspace(*start)) start++;
		if (! *start) continue;
		char *end = start;
		while (*end && *end != '/') end++;
		if (! *end) continue; // ignore
		*end = '\0';
		strcpy(mime_type, start);
		
		start = ++end; if (! *start) continue;
		while (*start && isspace(*start)) start++;
		end = start;
		while (*end && !isspace(*end)) end++;
		if (! *end) continue; // ignore
		*end = '\0';
		strcpy(subtype, start);

		start = ++end; if (! *start) continue;
		while (*start && isspace(*start)) start++;
		if (! *start) continue; // ignore
		end = start;
		while (*end && !isspace(*end)) end++;
		*end = '\0';
		strcpy(suffix, start);
		// cerr << "Debug: Suffix: " << suffix << "\tmapped on " << mime_type << "/" << subtype << endl;
		mime_dict[suffix] = new MIME(mime_type, subtype);
	} return true;
}
