#include <hush/java.h>
#include <hush/io.h>

#include <widgets/canvas.h>
#include <hush/session.h>

#undef string

#include <native/hush_dv_widgets_canvas.h>

#define method(X) Java_hush_dv_widgets_canvas_##X


JNIEXPORT jint JNICALL method(init)(JNIEnv *env, jobject obj, 
	   jint root, jstring name, jstring options)
{
  java_vm<canvas>* vmp = new java_vm<canvas>(env,obj,"Canvas");
  char* path = vmp->get(name);
  char* opts = vmp->get(options);
  if (*opts=='*') opts=0; // AE empty widget
  int r = (int) root;
  canvas* result = 0;
  if (r)
  	result =  new canvas((widget*)r,path,opts);
  else {
	result = new canvas(path,opts);
        session::_default->_register(result);
  }
  result->_register(vmp);
  result->_vmp = vmp;
  vmp->release(name,path);
  vmp->release(options,opts);
  return (jint) result;
}

JNIEXPORT void JNICALL method(pack)(JNIEnv *env, jobject obj)
{
  java_vm<canvas> vm(env,obj);
  vm->pack();
}

JNIEXPORT jint JNICALL method(docurrent)(JNIEnv *env, jobject obj)
{
  java_vm<canvas> vm(env,obj);
  item* it = vm->current();
  return (jint) it;
}

JNIEXPORT jstring JNICALL method(doitems)(JNIEnv *env, jobject obj,
	      jint x, jint y)
{
  java_vm<canvas> vm(env,obj);
  char* its = vm->items(x,y);
  return vm.string(its);
}

JNIEXPORT jstring JNICALL method(dofinditems)(JNIEnv *env, jobject obj,
	      jint x, jint y, jint x1, jint y1)
{
  java_vm<canvas> vm(env,obj);
  char* its = vm->items(x,y,x1,y1);
  return vm.string(its);
}


JNIEXPORT jint JNICALL method(getoverlapping)(JNIEnv *env, jobject obj,
	      jint x, jint y)
{
  java_vm<canvas> vm(env,obj);
  item* it = vm->overlapping(x,y);
  return (jint) it;
}

JNIEXPORT void JNICALL method(region)(JNIEnv *env, jobject obj,
	      jint x, jint y, jint x1, jint y1)
{
  java_vm<canvas> vm(env,obj);
  vm->region(x,y,x1,y1);
}

JNIEXPORT void JNICALL method(docoordsystem)(JNIEnv *env, jobject obj,
	      jint i)
{
  java_vm<canvas> vm(env,obj);
  vm->coordsystem(i);
}

JNIEXPORT jint JNICALL method(canvasx)(JNIEnv *env, jobject obj,
	      jint i)
{
  java_vm<canvas> vm(env,obj);
  return vm->canvasx(i);
}

JNIEXPORT jint JNICALL method(canvasy)(JNIEnv *env, jobject obj,
	      jint i)
{
  java_vm<canvas> vm(env,obj);
  return vm->canvasy(i);
}

JNIEXPORT void JNICALL method(move)(JNIEnv *env, jobject obj,
	      jstring str, jint x, jint y)
{
  java_vm<canvas> vm(env,obj);
  char* _str = vm.get(str);
  vm->move(_str,x,y);
  vm.release(str);
}

JNIEXPORT jint JNICALL method(mkbitmap)(JNIEnv *env, jobject obj,
	      jint x, jint y, jstring img, jstring opts)
{
  java_vm<canvas> vm(env,obj);
  char* _opts = vm.get(opts);
  char* _img = vm.get(img);
  item* it = vm->bitmap((int)x,(int)y,_img,_opts);
  vm.release(img,_img);
  vm.release(opts,_opts);
  return (jint) it;
}

JNIEXPORT jint JNICALL method(mkimage)(JNIEnv *env, jobject obj,
	      jint x, jint y, jstring img, jstring opts)
{
  java_vm<canvas> vm(env,obj);
  char* _opts = vm.get(opts);
  char* _img = vm.get(img);
  item* it = vm->image(x,y,_img,_opts);
  vm.release(img,_img);
  vm.release(opts,_opts);
  return (jint) it;
}

JNIEXPORT jint JNICALL method(mkline)(JNIEnv *env, jobject obj,
	      jint x, jint y, jint x1, jint y1, jstring opts)
{
  java_vm<canvas> vm(env,obj);
  char* _opts = vm.get(opts);
  item* it = vm->line(x,y,x1,y1,_opts);
  vm.release(opts);
  return (jint) it;
}

JNIEXPORT jint JNICALL method(mksline)(JNIEnv *env, jobject obj,
	      jstring img, jstring opts)
{
  java_vm<canvas> vm(env,obj);
  char* _opts = vm.get(opts);
  char* _img = vm.get(img);
  item* it = vm->line(_img,_opts);
  vm.release(img,_img);
  vm.release(opts,_opts);
  return (jint) it;
}

JNIEXPORT jint JNICALL method(mkarc)(JNIEnv *env, jobject obj,
	      jint x, jint y, jint x1, jint y1, jstring opts)
{
  java_vm<canvas> vm(env,obj);
  char* _opts = vm.get(opts);
  item* it = vm->arc(x,y,x1,y1,_opts);
  vm.release(opts);
  return (jint) it;
}


JNIEXPORT jint JNICALL method(mkoval)(JNIEnv *env, jobject obj,
	      jint x, jint y, jint x1, jint y1, jstring opts)
{
  java_vm<canvas> vm(env,obj);
  char* _opts = vm.get(opts);
  item* it = vm->self()->oval(x,y,x1,y1,_opts); // AE
  vm.release(opts);
  return (jint) it;
}

JNIEXPORT jint JNICALL method(mkcircle)(JNIEnv *env, jobject obj,
	      jint x, jint y, jint rad, jstring opts)
{
  java_vm<canvas> vm(env,obj);
  char* _opts = vm.get(opts);
  item* it = vm->circle(x,y,rad,_opts);
  vm.release(opts);
  return (jint) it;
}

JNIEXPORT jint JNICALL method(mkpolygon)(JNIEnv *env, jobject obj,
	      jstring img, jstring opts)
{
  java_vm<canvas> vm(env,obj);
  char* _opts = vm.get(opts);
  char* _img = vm.get(img);
  item* it = vm->polygon(_img,_opts);
  vm.release(img,_img);
  vm.release(opts,_opts);
  return (jint) it;
}

JNIEXPORT jint JNICALL method(mkrectangle)(JNIEnv *env, jobject obj,
	      jint x, jint y, jint x1, jint y1, jstring opts)
{
  java_vm<canvas> vm(env,obj);
  char* _opts = vm.get(opts);
  item* it = vm->rectangle(x,y,x1,y1,_opts);
  vm.release(opts);
  return (jint) it;
}

JNIEXPORT jint JNICALL method(mktext)(JNIEnv *env, jobject obj,
	      jint x, jint y, jstring img, jstring opts)
{
  java_vm<canvas> vm(env,obj);
  char* _opts = vm.get(opts);
  char* _img = vm.get(img);
  item* it = vm->text(x,y,_img,_opts);
  vm.release(img,_img);
  vm.release(opts,_opts);
  return (jint) it;
}

JNIEXPORT jint JNICALL method(mkwindow)(JNIEnv *env, jobject obj,
	      jint x, jint y, jobject w, jstring opts)
{
  java_vm<canvas> vm(env,obj);
  java_vm<widget> vw(env,w);
  char* _opts = vm.get(opts);
  item* it = vm->window(x,y,vw.self(),_opts);
  vm.release(opts,_opts);
  return (jint) it;
}

JNIEXPORT void JNICALL method(postscript)(JNIEnv *env, jobject obj,
	      jstring opts)
{
  java_vm<canvas> vm(env,obj);
  char* _opts = vm.get(opts);
  vm->postscript(_opts);
  vm.release(opts,_opts);
}
