// <title>Implementation of the filechooser class</title>
// <h2>Implementation of the filechooser class</h2>
// <file>filechooser.c
// <author>bastiaan, martijn, jrvosse.
// <ignore>
// This file contains the the C++ implementation of our filechooser.
// It is annotated with pseudo HTML markup.
// </ignore>
// <p>We need to include some headers first:

#include <hush/source.h>
#include <string.h>             // some standard C and C++ include files
#include <stdlib.h>
#include <stdio.h>
//#include <iostream.h>

#include <hush/event.h>         // to handle hush events
#include <hush/string.h>	// hush string class

#include <hush/sys/io.h>
#define endl "\n"

#include <hush/widgets/tk/chooser.h>       // The filechooser class header
#include <hush/widgets/tk/widget.h>

// Options to configure the widgets:

#define TYPE filechooser
#define OPTS 0
#define CLASS tk_filechooser
#define CONSTRUCTOR _filename[0] = _pathname[0] = _mask[0] = 0;
#include "hush/widgets/tk/create.i"


static const char* frameopts= "-relief raised -bd 1";
static const char* entryopts= "-width 40 -relief sunken -bg white";

// Options to pack the widgets:

static const char* label_pack_opts = "-side left -padx 10 -pady 10";
static const char* entry_pack_opts = "-side right -padx 10 -pady 10 "
                                     "-fill x -expand true";

//<h3>Constructors & destructors</h3>
// The first constructor creates a dummy CLASS object
// which can be used for handling events from the script interface

/* AE -->
CLASS::CLASS() {
    _filename[0] = _pathname[0] = _mask[0] = 0;
}

// The 'real' constructor follows below.
// It sets up the GUI and bindings, and does a magic call to "alias()" and
// "redirect()" ...
// See the widget manual and tutorial to see what they do ...

CLASS::CLASS(const char *p, const char* options, int istop) : frame(p,0) {
    _filename[0] = _pathname[0] = _mask[0] = 0;

    widget* top;
    if (istop) top = new toplevel(p,options);
    else top = new frame(p,options);
    _register(top);

    redirect(top);

    alias(this);

    build_gui(top, options);  // set up GUI
    bindings();                // set up bindings
    mask("*");                 // set default mask 
    dirpath(".");	       // set default directory
}

CLASS::CLASS(const widget *parent, const char *p, const char* options, int istop) : frame(parent, p, 0) {
    _filename[0] = _pathname[0] = _mask[0] = 0;
    init(options)
    */


void CLASS::init( const char* options) {
    int istop = 1; // default toplevel (AE)
    widget* top;
    if (istop) top = new toplevel(path(),options);
    else top = new frame(path(),options);
    _register(top);
    redirect(top);
    alias(this);

    build_gui(top, options);  // set up GUI
    bindings();                // set up bindings
    mask("*");                 // set default mask 
    dirpath(".");	       // set default directory
}


// The destructor. Now we have garbage collection for widgets,
// it is no longer needed ... 

//AE CLASS::~CLASS() { }

// <h3>Common member functions</h3>
// Here come some "ordinary" member functions you
// expect every CLASS to have...
//
// <h4>Retrieving the selected filename</h4>
// This function is used to retrieve the result.
// It grabs the mouse pointer in order to prevent any
// activity on other windows.
// Then it waits untill the CLASS window is destroyed (!)
// which will be the case after pressing Ok or Cancel...

const char* CLASS::get() {
    grab(); // grab mouse pointer
    wait(); // wait until the user has made his choise
    return _filename[0] ? _filename : 0;   // contains filename or null string
}

// <h4>The filename mask</h4>
// The following two members can be used to get and set the mask.         
// A mask is quit common concept in CLASS.
// The default mask is "*".

const char* CLASS::mask() const { return _mask; } 

void CLASS::mask(const char *m) { // set mask
    strcpy(_mask,m);		// save mask
    _maskentry->del();      	// delete entry
    _maskentry->insert(_mask);	// insert new mask 
    rescan();
}

// <h4>The directory path</h4>
// A get and set pair for the directory path as well.
// <warning>
//   Side effect: changes applications current working directory
// </warning>

const char* CLASS::dirpath() const { return _pathname; }

int CLASS::dirpath(const char *p) {
    sprintf(buf, "cd %s", p);
    if (tk->eval(buf) == OK) { 			// chdir
	strcpy(_pathname, tk->evaluate("pwd"));	// save path
    	_pathentry->del();			// delete entry
    	_pathentry->insert(_pathname); 	// insert new path
	rescan(); 			// rescan directory
	return OK; 
    } else {
    	sprintf(buf, "Could not chdir to %s", p);
	tk->error(buf);
	return ERROR;
    }
}

// <h4>Rescanning the directory</h4>
// This function does a rescan, which is needed to update
// the widgets after the directory path or mask has changed.
// 
// <em>
//	Side effect: Using global Tcl variables $CLASS_fill and $i
// </em>

void CLASS::rescan() {
    _listbox  ->del();      // delete listbox
    
    // put all directories in listbox 
    sprintf(buf,"foreach i [ lsort [ glob -nocomplain .* * ] ] "
	    "{ if [file isdirectory $i] "
	    "{%s insert end $i }}",
	    _listbox->path() );
    tk->eval(buf); 
    
    // put files in $CLASS_fill variable after mask with glob
    //AE sprintf(buf,"set CLASS_fill [glob -nocomplain %s]", _mask); 
    sprintf(buf,"namespace eval filechooser {set fill [glob -nocomplain %s]}", _mask);
    tk->eval(buf);
    
    // put files from variable in listbox
    sprintf(buf,"foreach i [lsort ${CLASS_fill}] "
	    "{ if [file isfile $i] {%s insert end $i }}", 
	    _listbox->path() );
    tk->eval(buf); 
}

// <h3>The graphical user interface of the CLASS</h3>
// This function is pretty long. It build the complete GUI.
// Maybe I should have split it up into several smaller functions...
// Note: GUIs like this need to be prototyped interactively!
//
// We will put everything in a widget called "top".
// Top will be typically be equal to "this", but derived classes may
// want to build the GUI in a frame or whatever.

void CLASS::build_gui(widget* top, const char* opts) {
    // Need some frames to pack things right. All need the same options.
    // Three frames for the three entries and their labels 
    // (file, path and mask), one frame for the listbox and the 
    // associated scrollbar, and finally one to frame the buttons:
    frame* _fileframe   = new frame(top, "fileframe",  frameopts);
    frame* _maskframe   = new frame(top, "maskframe",  frameopts);
    frame* _pathframe   = new frame(top, "pathframe",  frameopts);
    frame* _boxframe    = new frame(top, "boxframe",   frameopts);
    frame* _buttonframe = new frame(top, "buttonframe",frameopts);
    
    // Now we can create the file, path and mask entry boxes.
    _fileentry = new entry(_fileframe, "fileentry", entryopts);
    _pathentry = new entry(_pathframe, "pathentry", entryopts);
    _maskentry = new entry(_maskframe, "maskentry", entryopts);
    
    // Create labels to tell user what all these entries mean:
    label* _filelabel = new label(_fileframe, "filelabel", "-width 5");
    label* _masklabel = new label(_maskframe, "masklabel", "-width 5");
    label* _pathlabel = new label(_pathframe, "pathlabel", "-width 5");

    _filelabel->text("File:");
    _masklabel->text("Mask:");
    _pathlabel->text("Path:");
    
    // Create a listbox to show the files of the current dir.
    // We need a scrollbar for large directories.
    _listbox   = new listbox(_boxframe, "box", "-width 40 -height 10");
    scrollbar* _scrollbar = new scrollbar(_boxframe, "scroll");
    _scrollbar->yview(_listbox); 
    _listbox->yscroll(_scrollbar);
    
    // An Ok and Cancel button are pretty basic as well:
    _okbutton = new button(_buttonframe, "ok");
    _okbutton->text("Ok"); 
    
    _cancelbutton = new button(_buttonframe, "cancel");
    _cancelbutton->text("Cancel"); 
    
    // Pack all widgets. 
    // The listbox, scrollbar and the associated frame are a
    // bit special: the should fill and expand if the
    // file selector window is resized by the user.
    // The entries only expand in the x dimension
    _fileframe->pack(); 
    _maskframe->pack();
    _pathframe->pack(); 
    _boxframe ->pack("-fill both -expand true");
    _buttonframe->pack();
    
    _filelabel->pack(label_pack_opts);
    _masklabel->pack(label_pack_opts);
    _pathlabel->pack(label_pack_opts);
    
    _fileentry->pack(entry_pack_opts);
    _maskentry->pack(entry_pack_opts);
    _pathentry->pack(entry_pack_opts);
    
    _listbox  ->pack("-side left -fill both -expand true"); 
    _scrollbar->pack("-side right -fill y -expand false");
    
    _cancelbutton->pack("-side right -padx 10 -pady 10");
    _okbutton    ->pack("-side left  -padx 10 -pady 10");
    
    options(opts);  // process options
}

// <h3>User interaction: Bindings</h3>
// The buttons and entries generate events if they pressed, hit, etc.
// All we have to do is to bind the event to an object which is 
// capable to process it. Such an object is called a handler.
// The CLASS itself is a pretty good handler, since it is 
// derived (via toplevel and widget) from the handler class.
// So all events will be handled by "this". 
// 
// We want to handle events corresponding to pressing the 
// buttons (ok or cancel), selecting a file from the listbox (select),
// and hitting Return in the entry widgets. 
// A Return in the file entry is interpreted to be the same as pressing
// the Ok button. Return in the mask entry should set a new mask
// and update (rescan) the directroy contents in the listbox.
// A Return in the path entry should probably result in a change dir,
// and an update of the filebox. We deal with this actions in the
// handlers application operator()().
//
// In general, it is convenient to supply an extra argument to the event,
// so we can determine what actually happened.

void CLASS::bindings() {
    _cancelbutton->bind(this, "Cancel");            // cancel pressed
    _okbutton->bind(this, "Ok");                    // ok button pressed

    _listbox->bind(this, "Select-Double" );  // double click is default binding
    _listbox->bind("<Button-1>", this, "Select" ); // single click

    _fileentry->bind("<Return>", this, "Ok") ;      // like pressing ok
    _maskentry->bind("<Return>", this, "MaskReturn") ; // new mask
    _pathentry->bind("<Return>", this, "PathReturn") ; // new path
}

// <h3>Event types</h3>
// There are two types of events:<ol>
//      <li>Events generated by the Tcl interpreter, since our
//          CLASS can be used as a Tcl command as well.
//      <li>Events generated by the CLASS's components, as defined
//          above. 
// </ol>
// As an example of the first event, assume a user feeding the following
// code to the interpreter:
// <pre>
// hush> CLASS .f                # Generates event
// hush> ...
// </pre>
//
// The statement should create a new CLASS called .f
// 
// <h4>Creating widgets</h4>
// This event is typical for all widgets and 
// a special member functions is designed to handle this:
// the create() function defined below.
//
// <h4>Dispatching of other event</h4>
// The default dispatch() member of the widget class 
// will call the create() function to handle 
// the event discribed above.
// Additionally, it will check the widget's jump table to see if
// one of the entries matches the current event.
// If both fail, it will call the widgets operator to handle
// the "unknown" event.

widget* CLASS::create(int argc, char* argv[]) { // create new widget
    CLASS *f = new CLASS(argv[0]);    
    f->options(flatten(--argc, ++argv));            // process options
    tk->result(f->path());                          // return path to tk
    return f;
}

// AE
int CLASS::operator()() {
//event* CLASS::dispatch(event* _event) {  
    int argc = _event->argc(); char **argv = _event->argv();
    string first = argv[0];
    if (first == type()) { 
	_register(create(--argc, ++argv));
	return OK;
    }
    //cerr << "AE: CLASS " << endl;
    string s = _event->arg(1);
    if (s == (char*)"Select" || s == (char*)"Select-Double") {
	// A select on the listbox:
	_fileentry->del(); // delete old filename
	
	int index = _listbox->selection();
	char* sel = _listbox->get(index);
	char* selection = new char[strlen(sel) +1];
	strcpy(selection, sel);
	
	// Perhaps the selection is a directory ...
	sprintf(buf,"file isdirectory \"%s\"", selection);
	int isdir = atoi(tk->evaluate(buf));
	
	if (isdir) {
	    if (s == (char*) "Select-Double") dirpath(selection);
	} else {
	    _fileentry->insert(selection); 
	    if (s == (char*)"Select-Double") {
		strcpy(_filename, _fileentry->get());
		destroy();
	    }
	}
	delete selection;
    } else if (s == (char*)"Cancel") {
	strcpy(_filename, "");  // Empty string on cancel
	destroy(); 
    } else if (s == (char*) "Ok") {
	strcpy(_filename, _fileentry->get());
	destroy(); 
    } else if (s == (char*) "MaskReturn") {
	// A [Return] in the mask entry. Update mask and listbox
	mask(_maskentry->get());
    } else if (s == (char*) "PathReturn") {
	// A [Return] in the path entry. Change directory.
	dirpath(_pathentry->get());
    } else if (s == (char*) "get") {  
	tk->result(get());          // Pass selected filename to toolkit
    } else if (s == (char*) "rescan") { 
	rescan();
    } else self()->eval(quote(--argc, ++argv));
    return OK;
}

void CLASS::options(const char*) { 
    // not implemented ...
}

// <h4>Error handling</h4>
// An error has occured, probably in the arguments of
// the script command. Print usage and return error.

int CLASS::usage(const char* msg) const {
    err() << type() << ": " << msg << endl;
    err() << "Usage: " << endl;
    err() << type() << " <pathname> [options]" << endl;
    err() << "Or: " << endl;
    err() << "<pathname> get" << endl;
    err() << "<pathname> rescan" << endl;
    return ERROR;
}

