// tk_widget.c -- tk widget class

#ifndef IMPLEMENT
#define IMPLEMENT
#define WIDGET_C

#include <hush/source.h>

#include <hush/widgets/tk/widget.h>
#include <hush/sys/io.h>
#define endl "\n"

#define TYPE widget
#define CLASS tk_widget
#define INIT 
#define INITIALIZE widget::declare(h->category(),h); \
		   init_widgets_tk( kit::lookup("tk") ); 

#include <hush/widgets/tk/create.i>

#include <hush/item.h>
#include <hush/sys/bind.h>

#include <stdio.h>

//extern void register_as(const char*,const widget*);

#include <hush/assert.h>

#include <string.h>
//#include <tk.h>
#endif

void CLASS::title(const char* newtitle) {
	sprintf(buf, "winfo toplevel %s", path());
	char* topl = tk->evaluate(buf);
	
	sprintf(buf, "wm title %s {%s}", topl, newtitle);
	tk = kit::lookup("tk");
	if (!tk) { cerr << "NO KIT" << endl; }
	else tk->eval(buf);
}

char* CLASS::title() const {
	sprintf(buf, "winfo toplevel %s", path());
	char* topl = tk->evaluate(buf);
	
	sprintf(buf, "wm title %s", topl);
	return tk->evaluate(buf);
}
	

void CLASS::alias(handler* caller) {    // caller default 0
        if (caller == 0) caller = this;			// means caller = this
        char* _nm = name();
 
        char* oldname = new char[strlen(_nm)+1];
        char* newname = new char[strlen(_nm)+5];     // + "-cmd"
 
        strcpy(oldname, _nm );                        // save old name
        strcpy(newname, _nm );                        // generate new name
        strcat(newname, "-cmd");                        // .. ..
 
        rename(newname);                           // change hush name
 
        sprintf(buf, "rename %s %s", oldname, newname); // in Tcl aswell
        tk->eval(buf);
 
        tk->bind(oldname, caller);                      // new binding
        tk->result(oldname);                            // set result
 
        if (tk->tracelevel() > 5) {
          err() << "CLASS::alias called by " << caller->info();
          err() << " to change name of " << this->type();
          err() << " from " << oldname << " to " << newname << endl;
        }
 
        delete[] oldname;
        delete[] newname;
}

char* CLASS::bindtags() const {               // jrvosse
        sprintf(buf, "bindtags %s", self()->path());
        return tk->evaluate(buf);
}
 
void CLASS::bindtags(const char* bindorder) { // jrvosse
        sprintf(buf, "bindtags %s {%s}", self()->path(), bindorder);
        tk->eval(buf);
}

void CLASS::image(const char* s) { 
        if (s[0] != '@')
                sprintf(buf,"%s configure -image %s",name(),s);
        else {
                sprintf(buf,"image create photo -file %s",s+1);
                char* r = tk->evaluate(buf);
                sprintf(buf,"%s configure -image %s",name(),r);
                }
        tk->eval(buf);
        }

void CLASS::cursor(const char* s) { 
        if (s[0] != '@')
                sprintf(buf,"%s configure -cursor %s",name(),s);
        else {
                sprintf(buf,"image create bitmap -file %s",s+1);
                char* r = tk->evaluate(buf);
                sprintf(buf,"%s configure -cursor %s",name(),r);
                }
        tk->eval(buf);
        }


int CLASS::width()  const { 
    sprintf(buf, "winfo width %s", thepath());
    char *r = tk->evaluate(buf);
    if (r) return atoi(r);
    else return 0;
}

int CLASS::height() const { 
    sprintf(buf, "winfo height %s", thepath());
    char *r = tk->evaluate(buf);
    if (r) return atoi(r);
    else return 0;
}


long CLASS::eval(const char* s) {
     sprintf(buf,"%s %s",name(),s);	
     return tk->eval(buf);
     }

char* CLASS::evaluate(const char* s) {
      sprintf(buf,"%s %s",name(),s);
      if (tk->eval(buf) == OK)
       return tk->result();
      else
       return 0;
      }


void CLASS::destroy() { 
	sprintf(buf,"destroy %s",path());
	tk->eval( buf );
	}

void CLASS::xscroll(const widget* s) {
	sprintf(buf,"%s configure -xscrollcommand {%s set}",
		name(),((widget*)s)->self()->path());
	// s->xview( self() );
	tk->eval( buf );
	}

void CLASS::yscroll(const widget* s) {
	sprintf(buf,"%s configure -yscrollcommand {%s set}",
		name(),((widget*)s)->self()->path());
	// s->yview( self() );
	tk->eval( buf );
	}


void CLASS::configure(const char* s) { 
	//char* p = name()?name():path();
	sprintf(buf,"%s configure %s",name(),s);
	tk->eval( buf );
	}

void CLASS::geometry(int x, int y) {
	sprintf(buf,"%s configure -width %d -height %d",name(),x,y);
	tk->eval( buf );
	}


widget* CLASS::pack(const char* options) {
        if ((!strcmp(options,".quit")) || (!strcmp(options,".quit")))
		return pack(".quit","-side bottom -fill x");
        else if (!strcmp(options,"quit"))
		return pack(".quit","-side bottom -fill x");
	else if (!strcmp(options,"."))
		return this;
	else if (!strcmp(options,"forget")) {
		sprintf(buf,"pack %s %s", options, path());
		tk->eval(buf);
	}
	else {
		sprintf(buf,"pack %s %s", path(), options);
		tk->eval(buf);
	}
	return this;
	}

widget* CLASS::pack(widget* w, const char* options) {
	if (options[0]=='{') 
  	  sprintf(buf,"pack append %s %s %s", path(), w->path(),
	  options);
	else 
	  sprintf(buf,"pack configure %s %s", w->path(), options);
	tk->eval(buf);
	return this;
	}

widget* CLASS::pack(const char* w, const char* options ) {
	if (!strcmp(w,"quit")) 
	    return pack(".quit",options);
	else if (!strcmp(w,".quit")) {
	    require(tk);
  	    tk->eval("button .quit -text quit -command quit"); 
	    }
	if (options[0]=='{') 
  	  sprintf(buf,"pack append %s %s %s", path(), w, options);
	else 
	 sprintf(buf,"pack %s %s", w, options);
	tk->eval(buf);
	return this;
	}

void CLASS::place(const char* options) {
	sprintf(buf,"place %s %s",path(), options);
	tk->eval(buf);
	}




void CLASS::source(const char* type, const char* args, const char* options) {
	sprintf(buf,"drag&drop source %s -button 3 -tokencmd {package_%s %s} -sendcmd send_%s %s",
	  path(),type,args,type,options);
	  tk->eval( buf );
	  }

void CLASS::target(const char* type, binding* ac, const char* args, const char* options) {
	sprintf(buf,"drag&drop target %s handler {%s {%s %s}} %s",
	  path(),type,ac->name(),args,options);
	  tk->eval( buf );
	  }

void CLASS::focus(const char* options) {
	sprintf(buf,"focus %s %s", options, path()); 
	tk->eval( buf );
	}

void CLASS::grab(const char* options) {
	sprintf(buf,"grab %s %s", options, path()); 
	tk->eval( buf );
	}

void CLASS::wait() {
	sprintf(buf,"tkwait window %s",path());
	tk->eval( buf );
	}

int CLASS::direct(const char *s, binding* a, const char* args) {
	sprintf(buf,"bind %s %s { %s %s }", self()->path(),s,a->name(),args);
	return tk->eval( buf );
	}

char* CLASS::cget(const char* opt) const {
	sprintf(buf, "%s cget -%s", name(), opt);
	return tk->evaluate(buf);
}


#ifndef NOINIT
void CLASS::init(const char* options) {
	char buf[1024];
	tk = kit::lookup("tk");
	if (options) { // AE
		sprintf(buf,"%s %s %s",type(),path(),options);
		cerr << buf << endl;
		tk->eval( buf );
		}
	}
#endif

#ifdef WIDGET_C

#ifndef HCLONLY
void CLASS::map() { } //AE Tk_MapWindow( (Tk_Window) tkwin() ); }
void CLASS::unmap() {  } //AETk_UnmapWindow( (Tk_Window) tkwin() ); }

extern void* thewindow(kit* tk, const char* path);

void* CLASS::tkwin() const { //return tk->tkwin();
       return tk->tkwin(path());
       //AE return thewindow(tk,path()); 
       }
#endif

void CLASS::font(char*) {
  err() << "ERROR tk_widget:;font" << endl;
  /* AE Tk_GetFontStruct((Tcl_Interp*) tk->interp(), 
  (Tk_Window) tkwin(), Tk_GetUid(s));
  */
  }


//AE void CLASS::install(binding* , const char*) { }


int CLASS::operator()() {
	return widget::operator()();
	}

    // default interpreter
    /*
       int operator()() {
       cout << "CLASS: " << argv[0] << " (" << type() << ")" << endl;
       if ( !strcmp( type() ,*argv) )  create(--argc,++argv);
       else if (!strcmp("self",argv[1]) ) tk->result(self()->path());
       else self()->eval( flatten(--argc,++argv) );
       return OK;
       }
       virtual void create(int ,char*[]) {}
       */

#endif // WIDGET_C
