%{
//@auto=0
//@class vr-ml
%}
%class lex
%p 10000
%e 1500
%{
#include <hush/grammar/input.h>
#include <hush/grammar/string.h>
#include <hush/grammar/converter.h>
#include <hush/grammar/attribute.h>

#include <hush/stack.h>

static char* replace(char* x);

#define out if (pr) cout
static int pr = 1;
static char endline[1024];
static int is_loaded = 0;

static int mode = 0;
static int slide_mode = 0;
static int level = 0;

static int has_children = 0;

static stack<string>* _paramstack = 0; 
static string* _args = 0;
static string* _params = 0;

static float g_cnt = 0;
static float ul_cnt = 0;

static int in_li = 0;
static int in_text = 0;
static int in_box = 0;
static int in_raw = 0;
static int in_world = 0;
%}

escape ("\\\"")
stringelem ({escape}|[^\"\n])
string (\"{stringelem}+\")
any ([^\>]|{string})
atts {any}*

%%@ rules

{string} {
	yytext[yyleng-1] = '\0';
	out << "\\\"" << yytext+1 << "\\\"";
	}

"<slide "{atts}">" |
"<scene "{atts}">" {
	yytext[yyleng-1] = '\0';
	char buf[128];
	int has_src = 0;
	int has_scale = 0;
	string src;
	string scale;
	_params = new string();
	_paramstack->push(_params);
	_args = new string();
   	string_converter x(yytext+7);
   	for (int i = 1; i <= x.argc(); i++ ) {
        	attribute p(x[i]);
           	if ( p.matches("src") ) { 
			has_src=1; src=p.value();
			//cerr << "src=" << (char*) src << endl;
			} 
           	else if ( p.matches("scale") ) { has_scale=1; scale=p.value(); } 
           	if ( p.matches("color") || p.matches("style") ) { 
			sprintf(buf,"style=%s",(char*)p.value());
			(*_args) << (char*) buf;
			} 
           	else if ( p.matches("raw") ) { 
			if (!strcmp((char*)p.value(),"true"))
	                	in_raw++;
	   	}
        }
	out << "## " << yytext << endl;
	level = 1; mode = slide_mode; // mode = 0;
	if (!is_loaded && in_raw == 0) {
		//out << "#@include style.vr" << endl; 
		out << "#@include @proto/slide/slide.wrl" << endl; 
		is_loaded = 1;
		}
	if (has_src) {
		out << "#@include " << (char*) src << endl;
		}
	if (in_raw == 0) {
		out << "slide {" << endl;
		out << "level " << level << endl;
		out << "mode " << mode << endl;
		out << "children [ " << endl;
		has_children++;
		}
	level = 2; mode = 0;
	}
"</slide>" |
"</scene>" {
	if (in_raw == 0) {
		out << "]" << endl;
		out << "args " << "\"" << (char*) (*_args) << "\"" << endl;
		out << "params " << "[ " << (char*) (*_params) << " ]" << endl;
		out << "}";
		delete _args; _args = 0;
		delete _params; _params = 0;
		has_children--;
		}
	else in_raw--;
	out << "## " << yytext << endl;
	level = 0;
	}

"<slides"{atts}">"|"<scenes"{atts}">" {
	if (has_children > 0)  {
		out << " ]" << endl;
		has_children--;
		}
	out << "## " << yytext << endl;
	out << "withHUD 1" << endl;
	out << "visibleSlide -1" << endl;
	out << "choice [ " << endl;
	}

"</slides>"|"</scenes>"  {
	out << " ] " << "## " << yytext << endl; 
	}

"<world"{atts}">" {
	if (!in_world) {
		out << "#VRML V2.0 utf8" << endl;
		out << "#@include SlideStyle.wrl" << endl;
	}
	in_world++;
}
"</world>" { 
	in_world--;
	if (!in_world) {
		out << "#END world" << endl;
	}
}

"<param "{atts}">" {
	yytext[yyleng-1] = '\0';
	if (!_params) {
		_params = new string(replace(yytext+6));
		cerr << "WARNING: params nil" << endl;
		}
	char buf[512];
	sprintf(buf," \"%s\" ",replace(yytext+6));
	(*_params) += buf; 
	}

"<?vrml"{atts}">" {
	yytext[yyleng-1] = '\0';
	int is_raw = 0;
	string type = "dummy";
	string src = "dummy";
	string value = "dummy";
	string height = "5";
	string width = "7";
   	string_converter x(yytext+6);
   	for (int i = 1; i <= x.argc(); i++ ) {
        	attribute p(x[i]);
           	if ( p.matches("type") ) type = p.value();
           	else if ( p.matches("src") ) src = p.value();
           	else if ( p.matches("value") ) src = p.value();
           	else if ( p.matches("height") ) height = p.value();
           	else if ( p.matches("width") ) width = p.value();
           	else if ( p.matches("effects") ) { 
			if (!strcmp(p.value(),"on"))
				slide_mode = 1;
			else slide_mode=0;
			}
           	else if ( p.matches("mode") ) { 
			if (!strcmp((char*)p.value(),"raw"))
	                	is_raw = 1;
	   	}
        }
	out << "## " << yytext << endl;
	if (type == "image") {
		g_cnt += 5; // AE
		out << "image {" << endl;
		//out << "scale " << "3 1.5 1.5" << endl; 
		//out << "translation 0 -" << g_cnt << " 0 " << endl;
		out << "src \"" << (char*) src << "\"" << endl;
		out << "}" << endl;
		}
	else if (type == "movie") {
		g_cnt += 5; // AE
		out << "image {" << endl;
		out << "scale " << "3 1.5 1.5" << endl; 
		out << "translation 0 -" << g_cnt << " 0 " << endl;
		out << "src \"" << (char*) src << "\"" << endl;
		out << "}" << endl;
		}
	else if (type == "text") {
		g_cnt += 2; // AE
		out << "text {" << endl;
		out << "translation 0 -" << g_cnt << " 0 " << endl;
		out << "string \"" << value << "\"" << endl;
		out << "}" << endl;
		}
	else {
		out << "## no type " << type << " " << g_cnt << endl;
		}
	}



"<p align".* { }

"<text"{atts}">" {
	yytext[yyleng-1] = '\0';
	out << "## " << yytext << endl;
	if (!in_text) {
	        out << "slide { slidetype 1\n args \"" 
	            << "type=text "
	            << *_args << " " << yytext+5 << "\"" << endl;
		out << "level " << level << endl;
		out << "mode " << mode << endl;
		out << "translation 0 -" << g_cnt << " 0 " << endl;
		out << "string [ \"";
		}
	in_text++;
}
"</text>" {
	in_text--;
	if (!in_text) {
		out << "\" ] " << endl;
		out << "}" << endl;
		}
	out << "## " << yytext << endl;
}

"<box"{atts}">" {
	out << "## " << yytext << endl;
	yytext[yyleng-1] = '\0';
	if (!in_box) {
	        out << "slide { slidetype 1\n args \"" 
	            << "type=box "
	            << *_args << " style=red " << yytext+5 << "\"" << endl;
		out << "level " << level << endl;
		out << "mode " << mode << endl;
		out << "translation 10 -" << g_cnt << " 0 " << endl;
		out << "string [ \"";
		}
	in_box++;
}
"</box>" {
	in_box--;
	if (!in_box) {
		out << "\" ] " << endl;
		out << "}" << endl;
		}
	out << "## " << yytext << endl;
	g_cnt += 1;
}

"<ol"{atts}">" |
"<ul"{atts}">" {
	if (0) {
	yytext[yyleng-1] = '\0';
        ul_cnt = g_cnt;
	out << "slide { slidetype 1\n args \"" 
	       << "type=" << yytext[1] << yytext[2] << " "
	       << *_args << " " << yytext+3 << "\"" << endl;
	out << "level " << level << endl;
	out << "mode " << mode << endl;
        out << "translation 0 -" << g_cnt << " 0 ";
        out << "string [ ";
	}
	g_cnt += 0.1; // AE
        }

"</ol>" | 
"</ul>" { 
	if (in_li) {
		out << " \" ]}" << endl;
		in_li = 0;
		}
	out << "## " << yytext << endl;
	//g_cnt = ul_cnt + 0.5;
	g_cnt +=  0.5;
        }

"<li"{atts}">" { 
	// in_li = 1; ul_cnt+=1; out << "\"- ";
	if (in_li) {
		out << " \" ]} " << endl; 
		}
	out << "## " << yytext << endl; 
	if (1) {
	yytext[yyleng-1] = '\0';
        //ul_cnt = g_cnt;
	out << "slide { slidetype 1\n args \"" 
	       << "type=" << yytext[1] << yytext[2] << " "
	       << *_args << " " << yytext+3 << "\"" << endl;
	out << "level " << level << endl;
	out << "mode " << mode << endl;
        out << "translation 0 -" << g_cnt << " 0 ";
        out << "string [ \"- ";
	//g_cnt += 1;
	in_li = 1;
	}
	}


"<xxxx/li>" { out << "\""; }
"<h"[0-9]{atts}">" {
	yytext[yyleng-1] = '\0';
        out << "slide { level 2 slidetype 1\n args \"" 
	    << "type=header depth=" << yytext[2] << " "
	    << *_args << " " << yytext+3 << "\"" << endl;
	out << "level " << level << endl;
	out << "mode " << mode << endl;
        out << "translation 0 -" << g_cnt+0.5 << " 0 " << endl;
        out << "string \"";
	g_cnt += 2;
        }
"</h"[0-9]">" {
        out << "\" }";
        }
\n {
	//if (in_li) { out << "\""; in_li = 0; } 
	if (in_text || in_li)  g_cnt++;
	out << endl;
	}
. { out << yytext[0]; }
%%
//@ main
int main() {
slide_mode = 1;
_paramstack = new stack<string>(); 
lex l; l();
return 0;
}

static char* replace(char* s) {
static char buf[4*1024];
char* p = (char*) buf;
char* b = s;
while( *b != '\0' ) {
	if (*b == '"') {
		*p++ = '\\';
		*p++ = *b++;
		}
	else *p++ = *b++;
	}
*p = '\0';
return (char*) buf;
}
//</>
