%{
//@auto=0
//@class html-latex
%}
%class lex
%a 7000
%p 13000
%e 4000
%o 7000
%n 2000
%{
#include <hush/shell.h>
#include <hush/grammar/input.h>
#include <hush/grammar/string.h>
#include <hush/grammar/converter.h>
#include <hush/grammar/attribute.h>

#define mp(X) { if (in_math) out << X; else out << yytext; }

static char buf[10*1024];
static string slidemode = "slide";

char endline[BUFSIZ];
int mode;
int owntype = 0;
int texmode = 1;
int in_control = 0;
string ext = "";

static int no_hline = 0;


//@ io (html-latex)

class io {
public:
ostream* o;
io(ostream* x) : o(x) { }
};

static int pr=1;

io& operator<<(io& x, const char* s) { if (pr==1) *(x.o) << s; return x; }
io& operator<<(io& x, int& n) { if (pr==1) *(x.o) << n; return x; }
io& operator<<(io& x, char& c) { if (pr==1) *(x.o) << c; return x; }

static io out(&cout);
static io* old = &out;

//@class=html-latex

//@ state variables

//extern "C" system(const char*);

static int doc_no = 0;
static int in_code = 0;
static int in_tag = 0;
static int in_math = 0;
static int in_table = 0;
static int in_textable = 0;

static int in_format = 0;
static int in_prog = 0;
static int c_line = 0;
static int in_com = 0;

static int c_nonu = 0;
static int nu=1;
static int pref = 0;
static int env = 0;
static int latex2e = 1;
static int bk = 0;
static int cmt = 0;
static int refs = 0;
static char* _lastag = "}";

//@ convenience functions

static char* lastag() { return _lastag; }

static void begin(char*,char*);
static void end(char*,char*);
static void cmd(char*,char*);

void nl() { out << endline; strcpy(endline,""); }

static int instring(char c, char* s) {
        while ( *s && *s++ != c );
        return (int) *s;
        }

static char _symbols[] = " <>#-,.;?@`={}$:\t\n/\\\"'{}^!%&[]()~<>*";

static int symbol(char c) { return instring(c,_symbols); }

static const char* slideref(const char* x) {
static char buf[4024];
strcpy(buf,x);
char* b = buf; //while (*b && symbol(*b)) b++;
char* p = b;
int i = 0;
while (*p && i <= 18 ) {
	int yes=0;
	if (symbol(*p) || *p == ' ') {
		yes = 1;
		*b = '-';
		}
	else *b = *p;
	b++; p++;
	if (yes) while (symbol(*p) || *p == ' ') p++;
	i++;
	}
*b='\0';
if (*(b-1) == '-') *(b-1) = '\0';
buf[18]='\0';
return (char*) buf;
}


//@ definitions
%}


escape ("\\\"")
ident [A-Za-z0-9]+
stringelem ({escape}|[^\"\n])
string (\"{stringelem}+\")

any ([^\>]|{string})
atts {any}*



filename XXX
URL [\/\:A-Za-z\.0-9\-\+]+

%% rules
"index{"[ \t]+ { out << "index{"; }
[ \t]*"&nbsp;" { out << "~"; }
"~" { if (in_math) out << yytext;
      else if (in_prog)
	out << "\\(\\sim\\)";
      else
	out << "\\" << yytext; 
    }


"<format type=html>" { in_format = 1; }
"<format type=wrl>" { in_format = 1; }
"</format>" { in_format = 0; }
"</image>" { }
"<world"{atts}">" { }
"</world>" { }
"<scene"{atts}">" { }
"</scene>" { }

"<html>"|"</html>" { }
"</text>" { }
"<channel"{atts}">"|"</channel>" { }
"<navigate"{atts}">"|"</navigate>" { }
"<navigate"{atts}"/>" { }
^"<!--<navigate".* {;}
"<set"{atts}">"|"</set>" { }
"<?vrml"{atts}">" { }

"<vrml"{atts}">"|"<ignore>"|"<comment>"|"<c>" { pr--; }
"</vrml>"|"</ignore>"|"</comment>"|"</c>" { if (!in_format) pr++; }
" <!----> ." { out << "."; }

%%@ new ones

"books_catalog" { out << "books\\_catalog"; }

"<quote"{atts}">" { out << "\\begin{quote}"; }
"</quote>" { out << "\\end{quote}"; }

"<xxxp align=right>" { out << "\\zline{"; if (pr) strcat(endline,"}"); } 
"<h5 align=right>" { out << "\\zline{"; if (pr) strcat(endline,"}"); } 
"</h5><br>" { ; }
"</p><br>" { out << ""; }
"<h4 align=right"{atts}">" { out << "\\zline{" << (bk?"\\paragraphfont{":""); /*if (0 && pr) strcat(endline,"}");*/ } 
"<l++>" { }
"<"[\+\-][0-9]+">" { }
"<novr>"|"</novr>" { }
"xxxx<v>" { out << "\\vspace{0.5cm}"; }
"<box>" { out << "\\fbox{"; }
"</box>" { out << "}"; }

%%@ operator() and names

"operator" { if ( in_prog ) out << "{\\it operator}\\(\\,\\)"; else out << yytext; }
"operator()" { if ( in_prog ) out << "{\\it operator}()"; else out << yytext; }
"operator()()" { if ( in_prog ) out << "{\\it operator}()()"; else out << yytext; }
[a-z]+ {
	if ( 0 && in_prog && (!strcmp("interface",yytext) ||
	    !strcmp("enum",yytext) ||
	    !strcmp("class",yytext) ||
	    !strcmp("public",yytext) ||
	    !strcmp("protected",yytext) ||
	    !strcmp("private",yytext) ||
	    !strcmp("virtual",yytext) ||
	    !strcmp("typedef",yytext) ||
	    !strcmp("const",yytext) ||
	    !strcmp("extern",yytext) ||
	    0)) out << "{\\it " << yytext << "}";
	else out << yytext;
	}


%%@ includes

"<exec"{atts}">"|"<EXEC"{atts}">" {
	yytext[yyleng-1]='\0';
	system(yytext+5);
	}
"<so "{atts}">" { 
	yytext[yyleng-1]='\0'; files(yytext+3,".html");
	}
"<cd "{atts}">" {
	yytext[yyleng-1]='\0';
        directories(yytext+4,".html");
	}

"<include "{atts}">" {
   int choice = 0;
   yytext[yyleng-1] = '\0';
   string dir = "";
   string ext = "";
   string file = "@temp";
   string cmd = "";
   string_converter x(yytext+8);
   for (int i = 1; i <= x.argc(); i++ ) {
       attribute p(x[i]);
           if ( p.matches("file") ) { 
			if (choice !=3 ) choice = 1;
			file = p.value();
			}
           if ( p.matches("files") ) { choice = 1; file = p.value(); }
           if ( p.matches("dir") ) { choice = 2; dir = p.value(); }
           if ( p.matches("dirs") ) { choice = 2; dir = p.value(); }
           if ( p.matches("ext") ) { ext = p.value(); }
           if ( p.matches("cmd") ) { choice = 3; cmd = p.value(); }
           }
   if (choice == 0) file = yytext+9;
   if (choice == 2 && !strcmp(ext,"")) ext = ::ext;
   //cerr << "FILE:" << file << " EXT:" << (char*) ext << endl;
   if (!texmode) {
	if (choice==2) directories(dir,ext); 
	else if(choice==3) {
		char buf[BUFSIZ];
		sprintf(buf,"%s > %s",(char*)cmd,(char*)file);
		system(buf); files(file,"");
		}
	else files(file,ext);
	}
   else if (texmode && !in_control) {
	if (choice==2) directories(dir,ext);
	else if(choice==3) {
		char buf[BUFSIZ];
		sprintf(buf,"%s > %s",(char*)cmd,(char*)file);
		system(buf); files(file,"");
		}
	else files(file,ext);
	}
   else out << yytext << ">";
}

^"\\include ".* { files(yytext+8,"");  }
^"\\so ".* { files(yytext+3,".html");  }
^"\\cd ".* { directories(yytext+4,".html"); }

%%@ why was this?

"<hush"{atts}">" { ;}
"</hush>" { ;}

"<app"{atts}">" { ;}
"</app>" { ;}

"<eps src="{atts}">"|"<EPS SRC="{atts}">" {
	    yytext[yyleng-1]='\0';
	    out << "\\epsfbox{" << yytext+9 << "}" << "\n";
	    }

%%@ document

"<document"{atts}">" { 
   in_math = 0;
   in_table=0;
   in_textable = 0;
   //pr=1;
   //cout << "%% pr=" << pr << "  control=" << in_control << endl;
   if (doc_no++ == 0) {
   if (yytext[10]=='{' || yytext[10]=='[' || yytext[10]=='>') {
	 if (bk) {
		out << "\\input style.sty\n";
		out << "\\input begin.sty\n";
		}
	else if (!latex2e) {
        	yytext[yyleng-1]='\0';
		out << "\\documentstyle" << yytext+9;
		if (yyleng <= 10) out << "{article}";
		out << "\\begin{document}";
		}
	else if (latex2e) {
        	yytext[yyleng-1]='\0';
		out << "\\documentclass" << yytext+9;
		if (yyleng <= 10) out << "{article}";
		out << "\\begin{document}";
		}
    } else {
     yytext[yyleng-1]='\0';
     int old = 1; int opt = 0; 
     string cl = "article";
     string style = "article";
     string packs = "";
     string type = "style";
     string options = "";
	string_converter x(yytext+9);
	for (int i = 1; i <= x.argc(); i++ ) {
		attribute p(x[i]);
		if ( p.matches("class") ) { old = 0; cl = p.value(); }
		else if ( p.matches("style") ) { old = 1; latex2e = 0; style = p.value(); }
		else if ( p.matches("options") ) options = p.value();
		else if ( p.matches("opt") ) { opt=1; packs = p.value(); }
		else if ( p.matches("packages") ) { opt=1; packs = p.value(); }
		else if ( p.matches("packs") ) { opt=1; packs = p.value(); }
		else if ( p.matches("type") ) { owntype=1; type = p.value(); }
		}
	if (owntype) {
		out << "\\input " << (char*) type << ".sty\n";
	} else if (!latex2e) {
		out << "\\documentstyle" << options << "{" 
		    << style << "}";
		out << "\\begin{document}";
	} else {
		out << "\\documentclass" << options << "{" 
		    << cl << "}";
		if (opt) out << "\\usepackage{" <<
		    packs << "}";
	        out << "\\begin{document}";
		}
	if (bk && !owntype) out << "\\input begin.sty\n";
	}
     }
 }

"</document>" {
	if (--doc_no == 0) {
		if (bk && !owntype) out << "\\input end.sty\n";
		else out << "\\end{document}";
		}
	}

"<p "{atts}">"[ \t\n]* {
   int has_align = 0;
   string_converter x(yytext+2);
   for (int i = 1; i <= x.argc(); i++ ) {
       attribute p(x[i]);
           if ( p.matches("align") || p.matches("lalign") ) { 
			has_align=1;
			}
           }

     if (has_align) {  out << "\\zline{";
     if (pr) strcat(endline,"}");
     }
  }

"<text"{atts}">"[ \t\n]* {
   int has_align = 0;
   string_converter x(yytext+5);
   for (int i = 1; i <= x.argc(); i++ ) {
       attribute p(x[i]);
           if ( p.matches("align") || p.matches("lalign") ) { 
			has_align=1;
			}
           }

     if (has_align) {  out << "\\zline{";
     if (pr) strcat(endline,"}");
     }
  }


%%@ chapters and sections

"<c-nonum>" { c_nonu=1; }
"</c-nonum>" { c_nonu = 0; }
"</">  {  out << lastag(); }
"<nonu"{atts}">" { nu = 0; }
"</nonu"{atts}">" { nu = 1; }
"<chapter*"{atts}">" {  out << "\\chapter*{"; }
"<chapter"{atts}">" {  out << "\\chapter{"; }
"</chapter>" {  out << "}"; }
"<H"[0-9]">"|"<h"[0-9]">"[ \t\n]*|"<h"[0-9]" "{atts}">"[ \t\n]* {
   int has_align = 0;
   int has_num = 0;
   string_converter x(yytext+2);
   for (int i = 1; i <= x.argc(); i++ ) {
       attribute p(x[i]);
           if ( p.matches("align") || p.matches("lalign") ) { 
			has_align=1;
			}
           else if ( p.matches("num") ) { has_num = 1; }
           }

     if (has_align) {  out << "\\zline{" << (bk?"\\paragraphfont{":""); }
     else if (nu && !c_nonu && !has_num) {
	switch( yytext[2] ) {
	case '0':  out << "\\chapter{" << (bk?"\\chaptertitlefont{":""); break; 
	case '1':  out << "\\section{" << (bk?"\\sectionfont{":""); break; 
	case '2':  out << "\\subsection{" << (bk?"\\subsectionfont{":""); break; 
	case '3':  out << "\\subsubsection{" << (bk?"\\subsubsectionfont{":""); break; 
	case '4':  out << "\\paragraph{" << (bk?"\\paragraphfont{":""); break; 
	default:
	  out << "\\paragraph{" << (bk?"\\paragraphfont{":""); break; 
	};
     } else {
	switch( yytext[2] ) {
	case '0':  out << "\\chapter*{" << (bk?"\\chaptertitlefont{":""); break; 
	case '1':  out << "\\section*{" << (bk?"\\sectionfont{":""); break; 
	case '2':  out << "\\subsection*{" << (bk?"\\subsectionfont{":""); break; 
	case '3':  out << "\\subsubsection*{" << (bk?"\\subsectionfont{":""); break; 
	case '4':  out << "\\paragraph{" << (bk?"\\paragraphfont{":""); break; 
	default:
	  out << "\\paragraph{" << (bk?"\\paragraphfont{":""); break; 
	};
     }
  }
"</H"[0-9]">"|"</h"[0-9]">" {  out << (bk?"}":"") << "}"; }

%%@ ignore some html

"<!DOCTYPE".*\n { ; }
"<!doctype".*\n { ; }
"<base"{atts}">" { ; }
"<HTML>"|"<html>" { begin("","html"); }
"</HTML>"|"</html>" { end("","html"); }
"<META"{atts}">" { ;}
"<FONT"{atts}">"|"<font"{atts}">" { ; }
"</FONT>"|"</font>" { ;}
"<BODY"{atts}">"|"<body"{atts}">" { begin("","body"); }
"</BODY>"|"</body>" { end("","body"); }
"<TITLE>"|"<HEAD>"|"<ADDRESS>"

"<xxscript"{atts}">"|"<SCRIPT"{atts}">" { in_control++; pr--; }
"</xxscript>"|"</SCRIPT>" { in_control--; pr++; }

"<xxstyle"{atts}">"|"<STYLE"{atts}">" { in_control++; pr--; }
"</xxstyle>"|"</STYLE>" { in_control--; pr++; }

"<notex>"|"<title>"|"<head>"|"<address>" { ; } 
"</TITLE>"|"</HEAD>"|"</ADDRESS>"
"</notex>"|"</title>"|"</head>"|"</address>" { ; }
"<vrml"{atts}">"|"<!--\%"|"<c>"|"<control>" { pr--; }
"</vrml>"|"\%-->"|"</c>"|"</control>" { if (!in_format) pr++; }
"<HR>"|"<hr>" { ;}
"<P ALIGN=\"JUSTIFY\"><LI>" { out << "\\item "; } 
"<P ALIGN=\"JUSTIFY\">" { out << "\\par\\noindent "; } 
\n*"<p>"\n* {
	out << endline; strcpy(endline,"");
	out << "\n";
	}
"<P>"|"<lp>"|"<lp br="[0-9]+">"|"<lp "{atts}">"  {  out << "\n\n" << "\n"; }
"</p>"|"</P>" { ; }
"<!--" { cmt = 1; }
"-->" { if (cmt==0 && in_math) { out << "\\longrightarrow"; } else cmt = 0; }
"<A NAME="[^>]*">"|"<a name="[^>]*">" {
	char* p = (char*) yytext;
	yytext[yyleng-1]='\0';
        if ( yytext[yyleng-2] == '"' ) yytext[yyleng-2]='\0';
        if ( yytext[8] == '"' ) p++;
	if (! (pref || refs) ) out << "\\label{" << p+8 << "}";
	}
"<A "[^>]*">"|"<a "[^>]*">" { ;}

"x<tr"{atts}"> "     { ; }
"</th>"    { ; }
"</td>"    { ; }
"</tr>"    { out << "\\\\"; }
"<tr"{atts}">"[\ \t\n]*"<th"{atts}">" { if (!no_hline) out << "\\hline "; }
"<tr"{atts}">"[\ \t\n]*"<td"{atts}">" { if (!no_hline) out << "\\hline "; }
"<td"{atts}">" { out << "&"; }
"<th"{atts}">" { out << "&"; }

%%@ tables -- elementary form

"\\begin{tabular}" { out << yytext; in_textable=1; cerr << "TABLE" << endl; }
"\\end{tabular}" { out << yytext; in_textable=0; }

"<table"{atts}">" { 
	in_table = 1;
        yytext[yyleng-1] = '\0';
	string pos = "";
	string cols = "";
	string mode = "";
	string_converter x(yytext+6);
	for (int i = 1; i <= x.argc(); i++ ) {
		attribute p(x[i]);
		if ( p.matches("pos") ) pos = p.value();
		if ( p.matches("format") ) cols = p.value();
		if ( p.matches("cols") ) cols = p.value();
		if ( p.matches("mode") ) mode = p.value();
		if ( p.matches("hline") ) {
			no_hline = 1;
			}
		}
		out << "\\begin{tabular}" << pos << cols;
	}
"</table>" { in_table = 0; 
	     if (!no_hline) out << "\\hline";
	     out << "\\end{tabular}"; 
	     no_hline = 0;
	     }

%%@ slides -- see book

"<slice"{atts}">"|"<slide"{atts}">" { 
        yytext[yyleng-1] = '\0';
	string s1 = "X";
	string s2 = "X";
	string type = "";
	string mode = "slide";
	int has_caption = 0;
	string_converter x(yytext+6);
	for (int i = 1; i <= x.argc(); i++ ) {
		attribute p(x[i]);
		if ( p.matches("id") ) s1 = p.value();
		if ( p.matches("caption") ) { has_caption=1; s2 = p.value(); }
		if ( p.matches("cap") ) { has_caption=1; s2 = p.value(); }
		if ( p.matches("name") ) s2 = p.value();
		if ( p.matches("mode") ) mode = p.value();
		//if ( p.matches("type") ) type = p.value();
		if ( p.matches("tex") ) type = p.value();
		}
	if (!has_caption) mode="hide";
	slidemode=mode;
	if (s1=="X") s1= slideref((char*)s2);
	if (slidemode=="slide")
		out << "\\" << type << "slide{" << s1 << "}{" << s2 << "}{";
	}
"</slide>"|"</slice>" { if (slidemode=="slide") out << "}"; }

%%@ images

"<IMG"{atts}">"|"<img"{atts}">" { 
	yytext[yyleng-1] = '\0';
        string s; int fl=0;
	string w; int has_w = 0;
	string_converter x(yytext+4);
	for (int i = 1; i <= x.argc(); i++ ) {
	  attribute p(x[i]);
	  if ( p.matches("pdf") || p.matches("eps") || p.matches("EPS") ) {
	  	      fl = 1;
		      s = p.value();
		      }
	  else if (p.matches("lw") || p.matches("lwidth")) {
                has_w = 1; w = p.value();
		}
	}
	if (fl) {
	  out << "\\epsfig{ file=" << s;
	  if (has_w) out << ",width=" << w << "cm ";
	  out << "}";
	  }
   }

%%@ other html stuff

"</A>"|"</a>" { ;}
"<PRE>"[ \t\n]*"</PRE>"|"<pre>"[ \t\n]*"</pre>" { ; }
"<display><pre>"[ \t\n]*"</pre></display>" { ; }
"<program><pre>"[ \t\n]*"</pre></program>" { ; }
"<PRE>"|"<pre>" { if (!in_prog) { pref=1;  out << "\\begin{verbatim}"; } }
"</PRE>"|"</pre>" { if (!in_prog) { pref=0;  out << "\\end{verbatim}"; } }
"<MENU>"|"<UL>"|"<menu>"|"<ul>" {  out << "\\begin{itemize}"; }
"<menu"{atts}">"|"<ul"{atts}">" {  out << "\\begin{itemize}"; }
"</MENU>"|"</UL>"|"</menu>"|"</ul>" {  out << "\\end{itemize}"; }
"<OL>"|"<ol>" {  out << "\\begin{enumerate}"; }
"<ol"{atts}">" {  out << "\\begin{enumerate}"; }
"</OL>"|"</ol>" {  out << "\\end{enumerate}"; }
"<DL>"|"<dl"{atts}">" { if (!refs) out << "\\begin{description}"; }
"</DL>"|"</dl>" { if (!refs) out << "\\end{description}"; }
"<DT>[]".*|"<dt>[]".* { if (refs) out << "\\bibitem[" << yytext+5 << "]{" << yytext+5 << "}"; }

%%@ references

"<DT>"[ \t]*|"<dt>"[ \t]* { 
  if (pr) in_tag = 1; 
  if (refs)
	out << "\\bibitem{";
   else {
	out << "\\item[";
	}
  }

%%@ more html stuff

"<DD>"|"<dd>"[ \t]* {;}
"<BR>"|"<br>"|"<br "{atts}">" {  out << "\n";  /* "\\linebreak[0]";*/ }
"<LI>"|"<li>"|"<li"{atts}">"  {  out << "\\item "; }
"</li>"|"</LI>"  { }
"<EM>"|"<em>"   {  out << "{\\em "; }
"</EM>"|"</em>"  {  out << "}"; }
"<rm>"   {  out << "{\\rm "; }
"</rm>"  {  out << "}"; }
"<TT>"|"<tt>"   {  out << "{\\tt "; }
"</TT>"|"</tt>"  {  out << "}"; }
"<I>"|"<i>"   {  out << "{\\it "; }
"</I>"|"</i>"  {  out << "}"; }
"<B>"|"</B>" { ; }
"<strong>"|"<b>"   {  out << "{\\bf "; }
"</strong>"|"</b>"  {  out << "}"; }
"<CODE>"|"<code>" { in_code=1; pref=1;  out << "\\verb?"; }
"</CODE>"|"</code>"  { in_code=0; pref=0;  out << "?"; }
"\\</latex>%" { out << "\\%"; }

%%@ latex specifics

"<bibliography"{atts}">"|"<references>" {
	refs=1; out << "\\begin{thebibliography}{W1}\\raggedright ";
	}
"</bibliography>"|"</references>" {
	 out << "\\end{thebibliography}"; refs=0;
	}


"<ref"{atts}">" {  out << "\\ref{"; }
"</ref>" {  out << "}"; }
"<cite"{atts}">"|"[<cite"{atts}">" {  out << "\\cite{"; }
"</cite>"|"</cite>]" {  out << "}"; }

"<ADDRESS>"|"<address>" { begin("{\\tt ","address"); }
"</ADDRESS>"|"</address>" { end("}","address"); }

"<footnote>" {  out << "\\footnote{"; }
"</footnote>" {  out << "}"; }
"<caption>" {  out << "\\caption{"; }
"</caption>" {  out << "}"; }
"<footnotetext>" {  out << "\\footnotetext{"; }
"</footnotetext>" {  out << "}"; }
"<abstract>" {  out << "\\begin{abstract}"; }
"</abstract>" {  out << "\\end{abstract}"; }
"<figure>" {  out << "\n\\begin{figure}"; }
"</figure>" {  out << "\\end{figure}\n"; }

"<CENTER>"|"<center>" {  out << "\\begin{center}"; }
"</CENTER>"|"</center>" {  out << "\\end{center}"; }


%%@ programs and displays

"<display><novr><c></c><c><pre></c></novr></display>" { }
"<display"{atts}"><novr><c></c><c><pre></c></novr></display>" { }
"<display"{atts}">"[ \t\n]*"</display>" { }
"<display"{atts}">"[ \t\n]*("<pre>"\n)*\n* {
   int has_font = 0; string font;
   yytext[yyleng-1] = '\0';
   string_converter x(yytext+8);
   for (int i = 1; i <= x.argc(); i++ ) {
       attribute p(x[i]);
           if ( p.matches("font") ) { 
			has_font=1;
			font = p.value();
			}
           }
	in_prog = 1; out << "\\begin{verse}";
	if (has_font) out << "\\" << (char*) font;
	out << "\\small " << "%%";
	};
^"  " { if (!in_prog) out << "\\ \\ "; else out << "\\mbox{}"; }
\n*"</display>" { nl(); in_prog = 0; out << "\\end{verse}";  }
\n*"</pre></display>" { nl(); in_prog = 0; out << "\\end{verse}";  };
"</pre> <!--</program>-->" { in_prog = 0; out << "\\end{verse}";  }

"<!--<program"{atts}">-->"[ \t\n]*("<pre>"\n)* { in_prog = 1; out << "\\begin{verse}\\tt\\small "; };
"<program"{atts}">"[ \t\n]*("<pre>"\n+)* { in_prog = 1; out << "\\begin{verse}\\tt\\small "; };
\n*"</program>" { nl(); in_prog = 0; out << "\\end{verse}";  };
\n*"</pre></program>" { nl(); in_prog = 0; out << "\\end{verse}";  };

\n*"&lt;pre&gt;"\n* { nl();} 
\n*"&lt;/pre&gt;"\n* { nl();}

%%@ symbols
"&copy;" { out << "\\copyright "; }
"&AElig;" { out << "\\AE "; }
"&aelig;" { out << "\\ae "; }

%%@ comments are always tricky

\/\/ { if (in_prog) { c_line = 1; strcat(endline,"}"); out << "//{\\it "; }
       else out << yytext; }

\/\/\/ { if (in_prog) {  
		c_line = 1;
		strcat(endline,"}}");
	  	out << "\\hspace*{\\fill}\\nolinebreak {"; 
		if (yytext[3]=='+') out << "//{\\it ";
		else out << "{\\it";
	} else out << yytext;
  }

%%@ symbols

"<c-inprog>" { in_prog=1; }
"</c-inprog>" { in_prog=0; }

"{" { if (!in_math && !c_line && in_prog && !cmt) out << "\\{"; else out << yytext; }
"}" { if (!in_math && !c_line && in_prog && !cmt) out << "\\}"; else out << yytext; }

"{{ " { if (in_math) out << "\\{"; else out << yytext; }
" }}" { if (in_math) out << "\\}"; else out << yytext; }

"#"[A-Za-z]* { if (in_prog && !cmt) out << " \\" << yytext; else out << yytext; }
"#" { if (in_prog && !cmt) out << " \\# "; else out << yytext; }
"*" { if (in_prog && !cmt) out << "\\mbox{\\small *}"; else out << yytext; }

"^" { if (in_prog && !cmt) out << "\\(\\hat{\\mbox{}}\\)"; else out << yytext; }

"&"  {
	if (in_textable) out << "&";
	else if (in_prog && !cmt) out << "\\&";
	else { mp(" \\:\\&\\: "); }
	}

"/=" mp(" /\\!\\sim ");
"|>" mp(" \\vartriangleright ");
"<|" mp(" \\vartriangleleft ");
"->" mp(" \\rightarrow ");
"|->" mp(" \\mapsto ");
"*" mp("\\ast ");
"\\*" mp("\\times ");
"|" mp(" \\: | \\: ");
"[]" mp(" \\, \\Box \\: ");
"<" mp("{\\mbox{\\(<\\)}}" );
">" mp("{\\mbox{\\(>\\)}}" );
"<<" mp(" \\mbox{\\(\\langle\\)} " );
">>" mp(" \\mbox{\\(\\rangle\\)} ");
"<=>" mp("\\desdak ");
"=>" mp("\\Rightarrow ");
"<=" mp("\\mbox{\\(\\leqslant\\)}");
"==>" mp("\\Longrightarrow ");
"<==" mp("\\Longleftarrow ");
"->" mp("\\rightarrow ");
"<-" mp("\\mbox{\\(\\leftarrow\\)}");
"<->" mp("\\leftrightarrow ");
"[[" mp("\\lsemb ");
"]]" mp("\\rsemb ");
"||" mp( "\\parll ");
"||_" mp("\\lmerge ");
"==" mp("\\mbox{\\(\\equiv\\)}");
"!=" mp("\\mbox{\\(\\neq\\)}");
"=~" mp("\\simeq ");
"<=" mp("\\leq ");
">=" mp("\\geqslant ");
"|-" mp("\\vdash ");
"|=" mp("\\models ");
"\\-" mp("\\neg ");
"/\\" mp("\\wedge ");
"\\/" mp("\\vee ");
"\\." mp("\\cdot ");
"..." mp("\\ldots ");
"-*->" mp("\\apijl{*}");
"-{"[A-Za-z0-9\%\\]+"}->" {
	char buf[64];
	yytext[yyleng-3]='\0';
	sprintf(buf,"\\apijl{%s}",yytext+2);
	mp(buf);
	}


"#"[A-Z] {
	 if (in_math) {
	 	sprintf(buf,"{\\cross%s\\skipx}",yytext+1);
		out << buf;
         } else out << yytext;
      }
"_#"[A-Z] {
	 if (in_math) {
	 	sprintf("_{\\subcross%s\\skipx}",yytext+2);
		out << buf;
	 } else out << yytext;
	}
"|"[A-Z] {
	 if (in_math) {
	        sprintf(buf," {\\cal %s\\skipx} ",yytext+1);
		out << buf;
	 } else out << yytext;
	}


"\\LaTeX" { out << yytext; }
"LaTeX" { out << "\\LaTeX\\ "; }
"\\TeX" { out << yytext; }
"TeX" { out << "\\TeX\\ "; }

"\\0" { if (in_math) out << "\\emptyset "; else out << yytext; }

"<newpage>" { out << "\\newpage "; }

%%@ doing maths again? and iso

"&#9;" { ; }
"&#160;" { ; }
"&#232;" { out << "\\`e"; }
"&#235;" { out << "\\\"e"; }
"&#246;" { out << "\\\"o"; }

"$"|"$$" { if (!in_math) in_math=1; else in_math=0; out << yytext; }
"<MATH>"|"<math>" { in_math = 1; out << "$"; }
"</MATH>"|"</math>" { in_math = 0; out << "$"; }

"-&gt;" { if (pref) out << "->"; else out << "\\(\\rightarrow\\)"; }
"&lt;-" { if (pref) out << "<-"; else out << "\\(\\leftarrow\\)"; }
\\"&" { if (pref) out << "&"; else out << "\\&"; }
"&" { if (pref) out << "&"; else out << "\\&"; }

"&LT"(";")?|"&lt"(";")? {  if (pref) out << "<"; else out << "\\(<\\)"; }
"&GT"(";")?|"&gt"(";")? {  if (pref) out << ">"; else out << "\\(>\\)";  }
"&AMP"(";")?|"&amp"(";")? { if (in_textable) out << "&"; else  out << "\\&"; }
"&EUML"(";")?|"&"[oe]"uml"(";")? {  out << "\\\"{" << yytext[1] << "}"; }
"&iuml;" { out << "\\\"{\\i}"; }
"&auml;" { out << "\\\"{a}"; }
"&eacute;" { out << "\\'e"; }
"&egrave;" { out << "\\`e"; }
"&quot;" { out << "\""; }
"<DIR>"|"</DIR>"|"<SUP>"|"<SUB>" { }
"\\_" {  ECHO; }
"_" {  {
	if (cmt || pref || in_math) ECHO;
	else out << "\\_";
	}
   }

%%@ funny stuff and newlines 

"$this" { if (!( pref || in_code) ) out << "\\$this"; else ECHO; }
\n\n+ {
	c_line = 0;
	out << endline; strcpy(endline,"");
	if (in_prog)  out << "\n\n";
		//out << "\\\\\n\\\\\n";
	else out << yytext;
	}
\n { 
    c_line = 0;
    out << endline; strcpy(endline,"");
      {
     if (in_tag && !refs) out << "]";
     else if (in_tag && refs) out << "}";
     if (in_prog || (in_table > 1)) out << "\\\\\n";
     else if (!in_code) out << "\n";
     if (0 && in_table > 0 ) in_table++;
     }
   if (in_tag) in_tag=0;
   }
[ ] { if (in_prog && !in_math) out << "\\ "; else out << " "; }
\t { if (in_prog && !in_math) out << "\\ \\ \\ \\ \\ \\ \\ \\ ";
     else out << "        "; }
. {  out << yytext[0]; }

%%@ ...

"<l>"|"</l>"|"<latex>"|"</latex>" { ; }


"<?"[a-z]+">" { ; }
"<?/"[a-z]+">" { ; }

%%@ the old troff letters and symbols for math and sgml

"\\e " { if (in_math) out << " \\:\\in\\: "; else out << yytext; /*AE*/ }

"<latex>\\</latex>" {
	if (!in_math) out << "\\";
	}
"(#pcdata)" { out << (char*) "(\\#pcdata)"; }
\![a-zA-Z] { out << "$\\!$" << yytext; }
\#[a-zA-Z] { out << "\\" << yytext; } 
\&[a-zA-Z] { out << "\\" << yytext; } 
\%[a-zA-Z][a-zA-Z]+ { out << "\\" << yytext; }
\%[a-zA-Z] {
	char c = yytext[1];
	switch (c) {
        case 'a': out << "\\alpha "; break;
        case 'b': out << "\\beta "; break;
        case 'c': out << "\\xi "; break;
        case 'd': out << "\\delta "; break;
        case 'e': out << "\\varepsilon "; break;
        case 'f': out << "\\phi "; break;
        case 'g': out << "\\gamma "; break;
        case 'h': out << "\\theta "; break;
        case 'i': out << "\\iota "; break;
        case 'j': out << "\\iota "; break;
        case 'k': out << "\\kappa "; break;
        case 'l': out << "\\lambda "; break;
        case 'm': out << "\\mu "; break;
        case 'n': out << "\\nu "; break;
        case 'o': out << "\\omega "; break;
        case 'p': out << "\\pi "; break;
        case 'q': out << "\\psi "; break;
        case 'r': out << "\\rho "; break;
        case 's': out << "\\sigma "; break;
        case 't': out << "\\tau "; break;
        case 'u': out << "\\upsilon "; break;
        case 'v': out << "\\upsilon "; break;
        case 'w': out << "\\omega "; break;
        case 'x': out << "\\xi "; break;
        case 'y': out << "\\eta "; break;
        case 'z': out << "\\zeta "; break;
        case 'A': out << "\\Alpha "; break;
        case 'B': out << "\\Beta "; break;
        case 'C': out << "\\Xi "; break;
        case 'D': out << "\\Delta "; break;
        case 'E': out << "\\Varepsilon "; break;
        case 'F': out << "\\Phi "; break;
        case 'G': out << "\\Gamma "; break;
        case 'H': out << "\\Theta "; break;
        case 'I': out << "\\Iota "; break;
        case 'J': out << "\\Iota "; break;
        case 'K': out << "\\Kappa "; break;
        case 'L': out << "\\Lambda "; break;
        case 'M': out << "\\Mu "; break;
        case 'N': out << "\\Nu "; break;
        case 'O': out << "\\Omega "; break;
        case 'P': out << "\\Pi "; break;
        case 'Q': out << "\\Psi "; break;
        case 'R': out << "\\Rho "; break;
        case 'S': out << "\\Sigma "; break;
        case 'T': out << "\\Tau "; break;
        case 'U': out << "\\Upsilon "; break;
        case 'V': out << "\\Upsilon "; break;
        case 'W': out << "\\Omega "; break;
        case 'X': out << "\\Xi "; break;
        case 'Y': out << "\\Eta "; break;
        case 'Z': out << "\\Zeta "; break;
	}
  }

%%
//@ main

int main(int argc, char* argv[]) {

strcpy(endline,"");

pref = 0; env = 0;

while (argc > 1) {
	if (!strcmp(argv[1],"-bk")) { bk = 1; latex2e = 1; }
	if (!strcmp(argv[1],"-pre")) pref = 1;
	if (!strcmp(argv[1],"-env")) env = 1;
	if (!strcmp(argv[1],"-2e"))  latex2e = 1;
	if (!strcmp(argv[1],"-old"))  latex2e = 0;
	argc--; argv++;
	}

pref = 0; 

lex l; l();
return 0;
}

%%@ auxiliary

static
void begin(char* s, char* x = "verbatim") {
	if (env) {
		 out << "\\begin{" << x << "}";
		}
	else
		{
		 out << s;
		}
	}

static
void end(char* s, char* x = "verbatim") {
	if (env) {
		 out << "\\end{" << x << "}";
		}
	else {
		 out << s;
		}
	}

static
void cmd(char* s, char* x = "verb") {
	if (env) 
		 out << "\\" << x << " ";
	else
		 out << s;
	}
//</>
