%{
//@auto=0
//@class ht-ml
%}
%class vr_ml_lex
%o 18000
%p 15000
%e 5000
%a 12000
%n 2000
%{
#include <hush/hush.h>
#include <hush/grammar/input.h>
#include <hush/grammar/string.h>
#include <hush/grammar/converter.h>
#include <hush/grammar/attribute.h>

#include <hush/dictionary.h>
#include <hush/agent/resolver.h>
#include <hush/agent/query.h>

#include <hush/set.h>
#include <hush/stack.h>

#include <iostream.h>
#include <string.h>

//@ session info

vr_ml_lex* _lexer;

void text(dvstring* x) {
	_lexer->text((char*)*x);
	}


static query* _query = 0;
static session* thesession = 0;
static kit* tk;

static int warnings = 0;

static string the_slide;
static string the_caption;

//@ aliases

static int eat_n = 0;
static dictionary<string>* _alias = 0;

static void alias(const char* k, const char* v) {
	if (!_alias) _alias = new dictionary<string>(1);
	(*_alias)[k] = new string(v);
	//cerr << "Alias: " << k << " = " << v << endl;
	}

static char* thisalias(const char* k) {
	if (!_alias || !k) return 0;
	string* p = (*_alias)[k];
	if (!p) return 0;
	char* v = (char*) *p;
	//cerr << "Found: " << k << " => " << v << endl;
	return  v;
	}

static char* resolve(const char* k) {
	char *p = ::thisalias(k);
	if (!p) return (char*) k;
	else return ::resolve(p);
	}

//@ utilities

static string slidemode = "slide";
static int slideno = 0;
static int urlno = 0;

int iswhite(char c) { return ( c == ' ' || c == '\t' || c == '\n' ); } 

char* firstof(char* p) {
while ( *p && iswhite(*p) ) p++;
return p;
}

static int instring(char c, char* s) {
	while ( *s && *s++ != c );
	return (int) *s;
	}

static char _symbols[] = " <>#-,.;?@`={}$:\t\n/\\\"'{}^!%&[]()~<>*";

static int symbol(char c) { return instring(c,_symbols); }

static const char* slideref(const char* x) {
static char buf[4024];
strcpy(buf,x);
char* b = buf; //while (*b && symbol(*b)) b++;
char* p = b;
int i = 0;
while (*p && i <= 18 ) {
	int yes=0;
	if (symbol(*p) || *p == ' ') {
		yes = 1;
		*b = '-';
		}
	else *b = *p;
	b++; p++;
	if (yes) while (symbol(*p) || *p == ' ') p++;
	i++;
	}
*b='\0';
if (*(b-1) == '-') *(b-1) = '\0';
buf[18]='\0';
return (char*) buf;
}

//@ state variables and globals

static int auto_header = 0;
static int auto_footer = 0;

//@ query utils

char* unquote(const char* s) {
static char buf[1024*10];
	char* p = (char*) s; int i=0;
	while ( *p && *p != 0 ) {
		if ( *p != '\'' ) buf[i++] = *p++;
		else p++;
		}
	buf[i]='\0';
	return buf;
	}


static dictionary< stack<string> >* stacks;
static stack<string> object_stack;

void push(string* s, char* st=0) {
char* name = st;
if (!name) name="object";
stack<string>* x = (*stacks)[name];
if (!x) {
	x = new stack<string>;
	(*stacks)[name] = x;
	}
x->push(s);
//cerr << "pushing " << (char*)*s << endl;
}

string* pop(char* st=0) {
char* name = st;
if (!name) name="object";
stack<string>* x = (*stacks)[name];
string* s = x?(x->pop()):0; 
//cerr << "popping " << (char*)*s << endl;
return s;
}




static int in_cc = 0;
static int in_object = 0;
static int in_query = 0;
static int in_pipe = 0;
static int in_raw = 0;
static int in_ignore = 0;
static int in_slide = 0;
static int slynames = 0;

static int do_all = 0;

static int pr=1;
static int bk=0;
static int in_comment = 0;
static int in_dollar = 0;
static int raw_dollar = 0;

static char begin_dollar[24];
static char end_dollar[24];

static void reset_dollar() {
	    raw_dollar = 0;
            strcpy(begin_dollar,"<math>");
            strcpy(end_dollar,"</math>");
}

static int holdpr=0;
static char docstyle[1024];
//static char slidestyle[] = "<document type=rawslide>";
static char slidestyle[] = "<!--<document type=rawslide>-->\n<head>\n<STYLE ID=\"ietssxyz\" TYPE=\"text/css\"></STYLE>\n<script language=javascript src=@base/hush.js></script>\n<script language=\"JavaScript1.2\" src=\"@base/style.js\"></script>\n<script language=\"JavaScript1.2\" src=\"@base/color.js\"></script></head>";
static char slidebody[] = "<script language=Javascript1.2>slidebody();</script>\n<link rel=stylesheet type=\"text/css\" href=\"@base/style.css\">\n<script language=JavaScript1.2>slidemode=1;</script>\n";
static char rawstyle[] = "<document type=raw>";
string slide = "";
string id = "";
string caption = "";
string citefile = "refs";

static char pipefile[1024];
static int pipecnt = 0;
static int querycnt = 0;
static int rawcnt = 0;
static char filename[BUFSIZ] = "temp";

static char online[BUFSIZ] = ONLINE;
static char documents[BUFSIZ] = DOC;
static char manuals[BUFSIZ] = MAN;
static char home[BUFSIZ] = HIT;

static int _in_script = 0;
static int in_script() {
	return _in_script;
	}

static int in_li = 0;
static int in_a = 0;
static int refmode=0;
static int nameit=0;
static int citit=0;
static int refit=0;

static int authorno = 0;
static int bookmode = 0;
static int titlemode = 0;

static char tagfile[BUFSIZ];
static char reffile[BUFSIZ];
static char slidir[BUFSIZ];

char endline[BUFSIZ];
char endbrace[BUFSIZ];

static int in_b = 0;
static int braces = 0;

static int level = 1;
static string droot = "index";
static string up = "index";
static string fwd = "index";
static string bwd = "index";

//@ io

typedef ostream& OFUN(ostream&);
typedef OFUN* IOFUN();

static int nofio = 0;

ostream& begincmt(ostream& os) {
	 if (in_comment == 1) {
	 	os << " <!--";
	 	}
	 return os;
	 }

ostream& endcmt(ostream& os) {
	 if (in_comment == 0) {
		os << "--> ";
		}
	 return os;
	 }

OFUN* bcmt() {
	 in_comment++;
	 return begincmt;
	 }

OFUN* ecmt() {
	 in_comment--;
	 return endcmt;
	 }

static char buffer[10*1024];

class io {
public:
ostream* old;
ostream* o;
int _redirect;
io(ostream* x) : o(x), next(0) { _redirect = 0; }
void cmt(char* s)  {
	print(bcmt);
	print(s);
	print(ecmt);
	}


void print(OFUN f) {
        if (o) *(o) << f;
        if (next && !_redirect) next->print(f);
        }

void print(IOFUN x) {
        OFUN* f = x();
        if (o) *(o) << *f;
        if (next && !_redirect) next->print(*f);
        }

void print(char* s) {
	if (o) *(o) << s;
	if (next && !_redirect) next->print(s);
	}
void print(int& n) {
	if (o) *(o) << n;
	if (next && !_redirect) next->print(n); 
	}
void print(char& c) { 
	if (o) *(o) << c;
	if (next && !_redirect) next->print(c); 
	}

void flush() { o->flush(); }

void set(io* x) { next = x; }
io* child() { return next; }

void redirect(char* s = "@tempfile");
void hold() {
	o->flush();
	old = o;
	o = 0;
        nofio--;
	}

void reset() { 
	_redirect = 0;
	o->flush();
	((ofstream*)o)->close();
	o = old;
	old = 0;
        nofio++;
	}

char* get() { return (char*) buffer; }

void push(char* s = "@tempfile");

void pop() {
	cerr << "POP " << endl;
	if (next) {
		io* tmp = next->child();
		((ofstream*) (next->o))->flush();
		((ofstream*) (next->o))->close();
		delete next;
		next = tmp;
		nofio--;
		}
	}

int oke() { return pr; /*global!!*/ }
	
private:
io* next;
};

//@ io continued
 
inline
void io::push(char* s) {
	cerr << "PUSH " << s << endl;
	nofio++;
	ofstream* os = new ofstream();
	os->open(s,ios::out);
	assert(!os->fail());
	io* x = new io(os);
	/*
	x->print("<!-- %%%% Start ");
	x->print(s);
	x->print(" -->");
	*/
	if (!next) next = x;
	else {
	  //cerr << "NEXT " << endl;
	  x->set(next);
	  next = x;
	  }
	}

inline
void io::redirect(char* s) {
        _redirect = 1;
	o->flush();
	old = o;
if (s) {
	cerr << "RAW " << s << endl;
	ofstream* os = new ofstream();
	os->open(s,ios::out);
	o = os;
} else {
	cerr << "BIB " << endl;
	buffer[0]='\0';
	ostrstream* os = new ostrstream(buffer,0,10*1024);
	o = os;
}
}
 
io& operator<<(io& x, IOFUN f) { if (x.oke()) x.print(f); return x; }
io& operator<<(io& x, OFUN f) { if (x.oke()) x.print(f); return x; }
io& operator<<(io& x, char* s) { if (pr) x.print(s); return x; }
io& operator<<(io& x, int& n) { if (pr) x.print(n); return x; }
io& operator<<(io& x, char& c) { if (pr) x.print(c); return x; }
io& operator<<(io& x, string* p) { if (p) x << (char*) *p; return x; }

 
static io out(&cout);
static io* old = &out;

static io lst(&cerr);
 
//extern "C" system(const char*);
//extern "C" int atoi(char*);

//@ system

inline void execsystem(char* s) {
	//cerr << "EXEC SHELL: " << s << endl;
	system( s );
	}


//@ state variables -- continued

enum { htmlmode, textmode, slimode };

#define ifnat if (yytext[0]!='@')
#define iftext if (mode == textmode)
#define ifslide if ( mode == slimode )

int tex_mode = 0;
int ml_mode = 0;
int _in_ml = 0;

static int echo_mode = 0;
int in_escape = 0;
int in_program = 0;
int in_display = 0;

static int in_echo() {
	//return 0;
	if (echo_mode==1) return 1;
	else return 0;
	}

int in_ml() { return _in_ml; }

char* at() {
	if (ml_mode) return in_ml()?"@":"@";
	else return "@";
	}

char* dollar() {
	if (ml_mode) return in_ml()?"$":"$";
	else return "$";
	}

char* cent() {
	if (ml_mode) return in_ml()?"%":"%";
	else return "%";
	}

char* amp() {
	if (ml_mode) return in_ml()?"&":"\\&";
        else if (in_escape) return "&";
        else if (tex_mode && (in_display||in_program)) return "&amp;";
        else if (tex_mode) return "&";
 	else if (in_script() && !in_program) return "&";
	else return "&amp;";
	}

char* procent() { return cent(); }

char* ml_on() {
	if (ml_mode) {
		_in_ml++;
		return "<!--ml-on-->"; 
		}
	else return "";
	}

char* ml_off() {
	if (ml_mode) {
		_in_ml--;
		return "<!--ml-off-->"; 
		}
	else return "";
	}

int prog=0;
int in_cmode = 0;
int in_progcmt = 0;
int doc_no=0;
int mode = htmlmode;
int latexmode = 0;
char ext[12] = ".vr";
int in_control = 0;

static int latex() { return latexmode; }
static int all() { return do_all; }

char buf[BUFSIZ];

//@ raw mode

void begin_raw(char* s="") {
	in_raw++;
	if (mode==htmlmode && in_raw==1 && !latex()) {
		char buf[BUFSIZ];
		sprintf(buf,"@raw-%d-%s", rawcnt++,filename);
		buf[18]='\0';
		char file[BUFSIZ];
		sprintf(file,"%s.raw",buf);
		out << "<img align=top src=" << buf << ".gif>";
		out.redirect(file);
		//out.hold();
		//out.push(file);
		//out << bcmt;
		out << rawstyle << endl;
		out << s;
		}
	else out << s;
	}

void end_raw(char* s = "") {
	  if (mode==htmlmode && in_raw==1 && !latex()) {
	     out << s;
	     out << "</document>" << endl;
	     //out << ecmt;
	     out.reset();
	     //out.pop();
	     }
	  else out << s;
	  in_raw--;
	}

//@ query mode

static char queryformat[1024*4]="\n%s ";
static char resultformat[1024*4]="\n<!--%s-->";

class out_handler : public handler {
char buf[1024*10];
int n;
public:
out_handler(query* q=0) { _query=q; n = 0; } // for later
void result(const char* msg) {
	char* s = ::_query->option("result");
	if (s) strcpy(resultformat,s);
	sprintf(buf,resultformat,msg);
	if (s && s[0]!='\0') ::out << (char*)unquote(buf);
	}
void print(const char* msg) {
	char* s = _query->option("format");
	if (!strcmp(s,"-") || (s[0]=='\0')) 
		;//cerr << "no out format: " << msg << endl;
	else {
	if (s) strcpy(queryformat,s);
	//cerr << "out_handler::print " << msg << endl;
	if ( !strcmp(msg,"nl") && queryformat[0]!='\0') {
		if (s && s[0]) ::out << endl;
	} else if (strcmp(msg,"nl") && queryformat[0]!='\0') {
		sprintf(buf,queryformat,msg);
		if (s && s[0]) ::out << (char*)unquote(buf);
		}
	}
	}
int operator()() {
        if (!strcmp(_event->arg(0),"client"))
		result(_event->arg(1));
	else 
		print(_event->arg(1));
	return 0;
	}	
private:
query* _query;
}; 

static out_handler* _out = 0;

void begin_query() {
	if (in_query==1) {
		char program[BUFSIZ];
		sprintf(program,"@query-%s-%d",filename, querycnt++);
		out.redirect(program);
	        _query->option("program",program);
		in_escape = 1;
	        //out << "/* (AE) " << program << " */" << endl;
		}
	}

void end_query(CLASS* _scanner) {
  if (in_query==1) {
     if (_query && _query->option("style")) 
	     _scanner->files(_query->option("style"),"");

     in_escape = 0;
     out.reset();
     if (_query) {
	_query->option("out","true");
	if (_query->option("display")) {
	   if (strcmp(_query->option("display"),"true")) { // display != true
			  out << _query->option("display");
			  }
			  out << "<program><pre>" << endl;
			  in_program++; in_cc = 1;
			  _scanner->files(_query->option("program"),"");
			  in_program--; in_cc = 0;
			  out << "</pre></program>" << endl;
			}
	if (_query->option("header")) out << _query->option("header");
	cerr << "ht-ml::preparing query " << _query->category() << endl;
	_query->prepare();
	if (strcmp(_query->option("mode"),"exec")) { // mode != exec
        	_out = new out_handler(_query);
		_query->bind(0);
        	_query->bind((const char*)"cout", _out);
        	//_query->bind((const char*)"tell", _out);
	        cerr << "ht-ml::eval " << _query->category() << endl;
		int res = 0;
		if ( _query->option("ask") )
			   res = _query->eval(_query->option("ask")); 
		else if (_query->option("cmd"))
			   res = _query->eval(_query->option("cmd")); 
       		char* p = 0;
       		while ( ( p = _query->result(res) ) ) {
                        	_out->result ( p );
				}
	} else {
		_scanner->files(_query->option("file"),"");
	}
	if (_query->option("footer")) out << _query->option("footer");
        } else exit(1);
        strcpy(queryformat,"\n%s ");
        strcpy(resultformat,"\n<!--%s-->");
        in_query--;
        }
}

//@ pipe

void begin_pipe(char* s=0) {
	if (in_pipe==1) {
		char file[BUFSIZ];
		if (!s) 
		   sprintf(file,"@pipe-%s-%d",filename, pipecnt++);
		else
		   strcpy(file,s);
		strcpy(pipefile,file);
		out.redirect(file);
		}
	}

void end_pipe(char* s=0) {
	  if (in_pipe==1) {
	     out.reset();

	     }
	     in_pipe--;
	}

static int ul_cnt = 0;
static int h4_cnt = 0;

//@ definitions
%}

lightcolors (white|lightred|lightgreen|lightblue|lightyerllow) 
darkcolors (red|green|blue|yellow|silver|gold|black)
colors ({lightcolors}|{darkcolors})


escape ("\\\"")
ident [A-Za-z0-9]+
stringelem ({escape}|[^\"\n])
string (\"{stringelem}+\")

path  [A-Za-z0-9\@\~\#\:\-\.\/\?]+

value  ({path}|{string})
attrib ({ident}|{ident}[ \t\n]*"="[ \t\n]*{value})

attribs ([ \t\n]|{attrib})*

simple   ("<"{ident}">")
begintag ("<"{ident}{attribs}">")
endtag   "</"{ident}">")
beginpr  "<?"{ident}{attribs}">"
endpr    "<?/"{ident}">"

stylesheets (stylesheet|STYLESHEET)
stylesheet "<!"{stylesheets}[^\>]">"

doctypes (doctype|DOCTYPE)
elements (element|ELEMENT)
attlists (attlist|ATTLIST)

doctype  "<!"{doctypes}[^\>]">"
element  "<!"{elements}[^\>]">"
attlist  "<!"{attlist}[^\>]">"

begincmt [ \t]*"<!--"[ \t]*
endcmt   [ \t]*"-->"[ \t]*

old-any ({beginpr}|{endpr}|{doctype}|{element}|{attlist})

any ([^\>]|{string})
atts {any}*

old-atts [^\>]*
new-atts {attribs}

prefix  (http|mailto|ftp|news)
uri {prefix}":"[^\ \t\n\,\;\)\]\<\>]*

srcdirs (examples|sources|interfaces|packages)
docdirs (papers|tutorials|resources|{srcdirs})
online  (online)

src (hush|widgets|extern|sim|native)
ext (sigma|bpsim|grammar|ml|ami)
includedirs ("ml"|"java"|{src}|{ext})

hushdirs (papers|projects|talks|ftp)
places   (hush|dejavu|eliens|jrvosse|bastiaan|martijn|oo|se)
orgs     ("cs.vu.nl")

lpath ("../../"|""|"../")
lcfg (cfg|interface|category|component|install|lib|resource|hush|orbix|"orbix-1.3"|"orbix-2.0")

cfg (interface|category|component|resource|package|examples|hush|orbix|"orbix-1.3"|"orbix-2.0")

cc ("class"|"method"|"slide"|"package"|"module")
opt ("style"|"auto")

ccname ([^ \t\n]*|{string})

API ("HUSH_API"|"WIDGETS_API")

incc (ul|li)
%% rules

^"<!--<control>-->".* { }
"<!--" {}
"-->" {}
"<!--%" {}
"%-->" {}


^\.\".* {}

%%@ names

"Eliens"[0-9]+ { out << yytext; }
"\\Eliens" { out << yytext; }
"Eliens" { out << "Eli&euml;ns"; }
"Schonhage" { out << "Sch&ouml;nhage"; }

"<c-nonum>" {}
"</c-nonum>" {}
"<li>" { ul_cnt++; out << "\""; }
"</li>" { out << "\""; }
"<h4>" {
	out << "Transform { ";
	out << "translation 0 -1 0 ";
	out << "children [ ";
	out << "Text { ";
	out << "fontStyle FontStyle { style \"bold\" size 1.5 }";
	out << "string \"";
	}
"</h4>" { 
	out << "\" } ] }";
	}
"<ul>" {
	ul_cnt = 0;
	out << "Transform { ";
	out << "children [ ";
	out << "Text { ";
	out << "string [ ";
	}
"</ul>" { 
	out << " ] }";
	out << "] translation 0 -" << ul_cnt << " 0 }";
	}

%%@ scripts -- will be extended


%%@ document

"<ignore>" { in_ignore++; pr--; }
"</ignore>" { in_ignore--; pr++; }

"<world>" { out << "#VRML V2.0 utf8"; }
"<world"{atts}">" {
int tmp = pr;
if (doc_no++ == 0) {
   string style = "";
   int n_style = 0;
   string_converter x(yytext+5);
   for (int i = 1; i <= x.argc(); i++ ) {
        attribute p(x[i]);
           if ( p.matches("style") ) { 
			style=p.value();
	   }
    }
	pr = 1;
	char buf[BUFSIZ];
	if (strlen(yytext) <= 5) {
		sprintf(docstyle,"#VRML V2.0 utf8");

	} else
        	sprintf(docstyle,"%s",yytext);
	out << docstyle;
	if (auto_header) soinc("vrml-header","");
   }
pr = tmp;
}

"</world>" {
int tmp = pr;
if (--doc_no == 0) {
	pr = 1;
	if (auto_footer) soinc("vrml-footer","");
	}
pr = tmp;
}

%%@ file and dir inclusion

"@so ".*\n {
  if (in_program) {
	yytext[yyleng-1]='\0'; out << "&lt;" << yytext+1 << "&gt;";
  } else if (!tex_mode) {
	yytext[yyleng-1]='\0'; files(yytext+4,ext);
	}
  else if (mode != htmlmode && !in_control) {
	yytext[yyleng-1]='\0'; files(yytext+4,ext);
	}
  else out << yytext;
}

"@ds ".*\n {
  if (in_program) {
	yytext[yyleng-1]='\0'; out << "&lt;" << yytext+1 << "&gt;";
  } else {
  out << "<program><pre>\n  ";
  in_program = 1;
  if (!tex_mode) {
	yytext[yyleng-1]='\0'; files(yytext+4,"");
	}
  else if (mode != htmlmode && !in_control) {
	yytext[yyleng-1]='\0'; files(yytext+4,"");
	}
  else out << yytext;
  out << "</pre></program>" << endl;
  in_program = 0;
  }
}

"<so "{atts}">" {
  if (in_program) {
	yytext[yyleng-1]='\0'; out << "&lt;" << yytext+1 << "&gt;";
  } else if (!tex_mode || do_all) {
	yytext[yyleng-1]='\0'; files(yytext+4,ext);
	}
  else if (mode != htmlmode && !in_control) {
	yytext[yyleng-1]='\0'; files(yytext+4,ext);
	}
  else out << yytext;
}

"<include "{atts}">" {
if (0 && in_program) {
//yytext[yyleng-1]='\0'; out << "&lt;" << yytext+1 << "&gt;";
} else {
   int choice = 0;
   yytext[yyleng-1] = '\0';
   string dir = "";
   string ext = "";
   string file = "";
   string url = "";
   int get_url = 0;
   string cmd = "";
   string_converter x(yytext+8);
   for (int i = 1; i <= x.argc(); i++ ) {
       cerr << "ATTR: <include>" << x[i] << endl;
       attribute p(x[i]);
           if ( p.matches("file") ) { 
			if ((choice !=3) && (choice!=2)) choice = 1;
			file = p.value();
			if (file == "pipe")
				file = pipefile;
			}
           if ( p.matches("files") ) { choice = 1; file = p.value(); }
	   if ( p.matches("url") ) {
			char buf[1024];
			sprintf(buf,"@url-%s-%d",filename,urlno++);
			file = buf;
			resolver* r = new resolver();
		        r->file((char*)p.value(),buf);
			delete r;
			choice = 1;
			}
           if ( p.matches("dir") ) { choice = 2; dir = p.value(); }
           if ( p.matches("dirs") ) { choice = 2; dir = p.value(); }
           if ( p.matches("ext") ) { ext = p.value(); }
           if ( p.matches("cmd") ) { choice = 3; cmd = p.value(); }
           }
   if (choice == 0) file = yytext+9;
   if (choice == 2 && !strcmp(ext,"")) ext = ::ext;
   if (!tex_mode || do_all) {
	if (choice==2) directories(dir,ext,(char*)file); 
	else if(choice==3) {
		char buf[BUFSIZ];
		sprintf(buf,"%s > %s",(char*)cmd,(char*)file);
		execsystem(buf); files(file,"");
		}
	else files(file,ext);
	}
   else if (mode != htmlmode && !in_control) {
	if (choice==2) directories(dir,ext,(char*)file);
	else if(choice==3) {
		char buf[BUFSIZ];
		sprintf(buf,"%s > %s",(char*)cmd,(char*)file);
		execsystem(buf); files(file,"");
		}
	else files(file,ext);
	}
   else out << yytext << ">";
 }
}

%%@ directories

"@cd ".*\n {
  if (in_program) {
	yytext[yyleng-1]='\0'; out << "&lt;" << yytext+1 << "&gt;";
  } else if (!tex_mode || do_all) {
	yytext[yyleng-1]='\0'; directories(yytext+4,ext);
	}
  else if (mode != htmlmode && !in_control) {
	yytext[yyleng-1]='\0'; directories(yytext+4,ext);
	}
  else out << yytext;
}

"<cd "{atts}">" {
  if (in_program) {
	yytext[yyleng-1]='\0'; out << "&lt;" << yytext+1 << "&gt;";
  } else if (!tex_mode || do_all) {
	yytext[yyleng-1]='\0'; directories(yytext+4,ext);
	}
  else if (mode != htmlmode && !in_control) {
	yytext[yyleng-1]='\0'; directories(yytext+4,ext);
	}
  else out << yytext;
}

^"@#!".*\n {
  if (in_program) {
	yytext[yyleng-1]='\0'; out << "&lt;" << yytext+1 << "&gt;";
  } else {
	yytext[yyleng-1]='\0';
		execsystem(yytext+3);
 }
}


"<exec"{atts}">" {
  if (in_program) {
	yytext[yyleng-1]='\0'; out << "&lt;" << yytext+1 << "&gt;";
  } else {
       if (mode != tex_mode && !in_control) { 
		int infile = 0;
		yytext[yyleng-1] = 0;
		string cmd = "date";
		string file = "@temp";
	        string_converter x(yytext+6);
   		for (int i = 1; i <= x.argc(); i++ ) {
       		  attribute p(x[i]);
           	  if ( p.matches("cmd") ) cmd = p.value();
           	  if ( p.matches("file") )  {
				file = p.value();
				infile = 1;
				}
		}
		char buf[BUFSIZ];
		if (infile)
			sprintf(buf,"%s > %s",(char*)cmd,(char*)file);
		else
			sprintf(buf,"%s",(char*) cmd );
       		execsystem(buf);
		if (infile) files((char*)file,"");
       		out.flush();
       }
   }
}

^"@exec ".* {
  if (in_program) {
	//yytext[yyleng-1]='\0'; out << "&lt;" << yytext+1 << "&gt;";
  } else
	execsystem(yytext+5); out.flush();
  }


%%@ format -- isn't that obsolete?

"<format"{atts}">" {
	char buf[BUFSIZ];
        yytext[yyleng-1] = '\0';
        string_converter x(yytext+7);
        for (int i = 1; i <= x.argc(); i++ ) {
                attribute p(x[i]);
                if ( p.matches("mode") ) 
			if (!strcmp(p.value(),"text"))
				mode = textmode;
			else if (!strcmp(p.value(),"slide"))
				mode = slimode;
                }
        }

%%@ image


"<image"{atts}">" {
        yytext[yyleng-1] = '\0';
	int fl=0;
	string s;
	int w_pre = 0;
	string src="@images/logo.gif";
        string width="450";
        string height="275";
        string pre="";
        string post="";
        string_converter x(yytext+6);
        for (int i = 1; i <= x.argc(); i++ ) {
          attribute p(x[i]);
          if ( p.matches("name") ) {
		      char buf[64];
                      fl = 1;
		      sprintf(buf,"%s.gif",p.value());
                      src = (char*) buf;
		      sprintf(buf,"%s.eps",p.value());
		      s = (char*) buf;
                      }
          else if ( p.matches("eps") ) {
                      fl = 1;
                      s = p.value();
                      }
	  else if ( p.matches("v") ) { 
		char buf[128];
		sprintf(buf,"\\vspace{%scm}",p.value());
		post=(char*)buf;
		}
	  else if ( p.matches("src") ) src = p.value();
	  else if ( p.matches("width") ) width = p.value();
	  else if ( p.matches("height") ) height = p.value();
	  else if ( p.matches("pre") ) { w_pre=1; pre = p.value(); } 
	  else if ( p.matches("post") )  post = p.value(); 
        }
	//out << "<a href=" << (char*) src << ">";
	out << "<control><center>";
	out << "<script language=javascript>";
	out << "document.write(\"<img src=" << (char*) src << " ";
	out << " width=\" + imgw(" << (char*) width << ") + ";
	out << "\" height=\" + imgh(" << (char*) height << ") + ";
	out << "\">\");";
	out << "</script>";
	out << "</center></control>";
	//out << "</a>";
        //if (fl) out << "<!--\\epsfbox{" << s << "}\\vspace{0.5cm}-->";
        if (fl) {
		if (!w_pre) out << "<center>";
		out << "<!--";
		out << (char*) pre; 
		out << "\\epsfig{file=" << s;
		out << ",width=9.5cm}\\vspace{-2.5cm}"; 
		out << (char*) post;
		out << "-->";
		if (!w_pre) out << "</center>"; 
		}
   }


%%@ slide

"<slide"{atts}">"[ \t\n]*"</slide>" { }
"<slide"{atts}">"\n"<dl>"\n"</dl>"\n"</slide>" { }
"<slide"{atts}">" {
    in_comment = 0; // AE clean start
    in_slide++;
    //if (!in_ignore && latex()) { out << yytext; } 
    if (!in_ignore) {
	ifslide pr = 1;
	char buf[BUFSIZ];
        yytext[yyleng-1] = '\0';
	int slcolor=0;
        string color=(char*) "gold";
	string mode = "slide";
        string id = "X";
        string caption = "X";
        string file = "tempfile";
        string_converter x(yytext+6);
        for (int i = 1; i <= x.argc(); i++ ) {
                attribute p(x[i]);
                if ( p.matches("id") ) {
				id = slideref(p.value());
				file = (char*) id; // slideref( p.value());
			        if (id == "contents" || id == "title")
						color = "black";
				}
                if ( p.matches("caption") ) caption = p.value();
                if ( p.matches("cap") ) caption = p.value();
                if ( p.matches("name") ) caption = p.value();
                if ( p.matches("color") ) color = p.value();
                if ( p.matches("mode") ) mode = p.value();
                if ( p.matches("style") ) color = p.value();
                if ( p.matches("file") ) file = slideref( p.value() );
		if ( p.matches("dir") ) {
		  sprintf(buf,"%s/%s", slideref(p.value()),(char*)file);
		  file = buf;
		  }
                }
	slidemode = mode;
	char* idp = (char*) id;
	if (*idp == 'X') {
		file = slideref(caption);
		id = (char*) file; //slideref(caption);
		}
	else if (*idp == '.') {
		char buf[1024];
		sprintf(buf,"%s-%d",(char*)caption,slideno++);
		file = slideref(buf);
		id = (char*) file; // slideref(buf);
		}
	if (slidemode=="control") out << "<control>";
	sprintf(buf,"@slide-%s.html",(char*) id);
	if (!all() && !latex()) out.push(buf);
	sprintf(buf,"@slide-%s.html",(char*) id);
	the_slide = (char*) buf;
	::caption = caption;
	the_caption = (char*) caption;
	::id = id;
	if (!all() && !latex()) {
	*(out.child()) << "\n" << slidestyle << "\n";
	*(out.child()) << "\n<script language=Javascript1.2>" <<
               "slidestyle=\"" << color << "\";\n</script>\n";
	*(out.child()) << "\n" << slidebody << "\n";
	}
	if (!latex()) {
	if (slidemode=="slide")
	   out << "<hr><a name=\"" << (char*) id << "\">\n";
	out << "<a name=\"" << (char*) id << "\">\n";
	}
        out << bcmt << yytext << ">" << ecmt << endl;
	lst << "slide(\"" << prefix() << "@slide-" << (char*) id << "\");" << endl;
        }
}

"</slide>" { 
     if (!in_ignore && in_slide==1 && latex()) {
	    out << yytext;
     } else if (!in_ignore && in_slide==1 && !latex()) {
	    if (mode==htmlmode && slidemode=="slide") {
	    	out << "<!--<control>--><hr><center>"
		  << "<a href=" << "contents" << ".html>"
		  << "<i>slide</i>: </a>" // << (char*)id << ": "
	          << "<a href=\"" << (char*) the_slide << "\">"
	          << (char*) the_caption << "</a></center><hr><!--</control>-->";
		  }
	    out << "<!--" << yytext << "-->"; *(out.child()) << "\n</body></document>"; 
	    if (!all()) out.pop();
	    if (slidemode=="control") out << "</control>" << endl;
	    ifslide pr = 0;
	    }
      if (in_slide == 0) cerr << "Warning: superfluous </slide>" << endl;
      else in_slide--;
      }

%%@ query

^"#"("sh"|"/bin/sh") {
	if (in_query) {
		_query->option("sh","sh");
		}
	out << yytext;
	}

"<query>" {
	in_query++;
	if (in_query==1) {
		 _query = new query("");
		}
	begin_query();
	}
"</query>" { 
	end_query(this);
	}

"<param "{atts}">" {
     eat_n = 1;
     if(in_query==1) {
        yytext[yyleng-1] = '\0';
	string name="dummy";
	string value="dummy";
        string_converter x(yytext+7);
        for (int i = 1; i <= x.argc(); i++ ) {
          attribute p(x[i]);
	  if (p.matches("name"))  name=p.value();
	  else if(p.matches("value")) {
		value=p.value();
		_query->option((char*)name,(char*)value);
		}
	  else _query->option(p.key(),p.value());
	  }
     } else out << yytext;
}


"<query "{atts}">" {
in_query++;
if(in_query==1) {
        yytext[yyleng-1] = '\0';
	_query= new query(yytext+6);
        string_converter x(yytext+6);
        for (int i = 1; i <= x.argc(); i++ ) {
          attribute p(x[i]);
          if ( p.matches("fm") ) {
		 strcpy(queryformat, (char*) p.value());
		 }
          else if ( p.matches("format") ) {
		 strcpy(queryformat, (char*) p.value());
		 }
          else if ( p.matches("result") ) {
		 strcpy(resultformat, (char*) p.value ());
		 }
          else if ( p.matches("res") ) { 
		 strcpy(resultformat, (char*) p.value()) ; 
		 }
          }
	begin_query();
	}
   }

%%@ raw

"<raw>" { begin_raw(); }
"</raw>" { end_raw(); }

"<raw "{atts}">" {
in_raw++;
if(in_raw==1 && !latex()) {
        yytext[yyleng-1] = '\0';
	int fl=0;
	string s;
	string src="@images/logo.gif";
        string width="626";
        string height="400";
        string pre="";
        string post="";
        string_converter x(yytext+6);
        for (int i = 1; i <= x.argc(); i++ ) {
          attribute p(x[i]);
          if ( p.matches("eps") ) {
                      fl = 1;
                      s = p.value();
                      }
	  else if ( p.matches("src") ) src = p.value();
	  else if ( p.matches("width") ) width = p.value();
	  else if ( p.matches("height") ) height = p.value();
	  else if ( p.matches("pre") ) pre = p.value();
	  else if ( p.matches("post") ) post = p.value();
        }
	char file[BUFSIZ];
	if (mode==htmlmode) {
		char buf[BUFSIZ];
		sprintf(buf,"@raw-%d-%s",rawcnt++,filename);
		buf[18]='\0';
		sprintf(file,"%s.raw",buf);
		//out << "<image src=" << buf << ".gif>";

	//out << "<a href=" << (char*) src << ">";
	out << "<script language=javascript>";
	out << "document.write(\"<img src=" << (char*) buf << ".gif ";
	out << "width=\" + imgw(" << (char*) width << ") + ";
	out << "\"height=\" + imgh(" << (char*) height << ") + ";
	out << "\">\");";
	out << "</script>";
		out.redirect(file);
		//out.push(file);
                //out << bcmt;
		out << rawstyle << endl;
		}
   }
}

%%@ pipe

"<pipe>" { in_pipe++; begin_pipe(); }
"</pipe>" { end_pipe(); }

"<pipe "{atts}">" {
in_pipe++;
if(in_pipe==1) {
        yytext[yyleng-1] = '\0';
	int fl=0;
        string file="@pipe";
        string_converter x(yytext+6);
        for (int i = 1; i <= x.argc(); i++ ) {
          attribute p(x[i]);
          if ( p.matches("file") ) {
                      fl = 1;
                      file = p.value();
                      }
	}
        if (fl) begin_pipe((char*)file); else begin_pipe();
   }
}


%%@ vrml

"<object "{atts}("/")?">" {
        in_object++;
	int endit = 0;
	if ( yytext[yyleng-2] == '/')  { endit=1; yytext[yyleng-2] = '\0'; }
        else yytext[yyleng-1] = '\0';
	int fl=0;
	int tr=0;
	int sh=0;
	int app=0;
	string* finish = new string("");
	string id="";
	string rest="";
        string file="dummy";
        string type="dummy";
        string translate="";
        string rotation="";
        string scale="";
        string_converter x(yytext+7);
        for (int i = 1; i <= x.argc(); i++ ) {
          attribute p(x[i]);
          if ( p.matches("file") ) { fl = 1; file = p.value(); }
          else if ( p.matches("translate") ) { tr = 1; translate = p.value(); }
          else if ( p.matches("rotation") ) { tr = 1; rotation = p.value(); }
          else if ( p.matches("scale") ) { tr = 1; scale = p.value(); }
          else if ( p.matches("type") ) { sh = 1; type = p.value(); }
          else if ( p.matches("id") || p.matches("name")) { id = p.value(); }
	  else {
		char buf[1024];
		sprintf(buf," %s %s ",p.key(), p.value());
		rest << (char*) buf;
		}
	}
	if (type="transform") out << "Transform {";
	else if (type="group") out << "Group {";
	if (tr) {
	   if (translate != "") { out << "translate " << (char*) translate << endl; }
	   if (rotation != "") { out << "rotation " << (char*) rotation << endl; }
	   if (scale != "") { out << "scale " << (char*) scale << endl; }
	   out << "children [";
	(*finish) << "]}";
	}
	push(finish);
        if (fl)  files(file,"");
	if (endit) {
		string* s = pop();
		out << (char*) *s << endl;
		delete s;
        }
}

"</object>" { 
	string* s = pop();
	out << (char*) *s << endl;
	delete s;
	}

%%
//@ main


static char g_argv[24][256];

static void declare(const char* pkg, const char* cat) {
char k[1024]; char v[1024];
sprintf(k,"%s",pkg); sprintf(v,"%s/%s",cat,pkg); ::alias(k,v);
sprintf(k,"sources/%s",pkg); sprintf(v,"sources/%s/%s",cat,pkg); ::alias(k,v);
sprintf(k,"packages/%s",pkg); sprintf(v,"sources/%s/%s",cat,pkg); ::alias(k,v);
sprintf(k,"interfaces/idl/%s",pkg); sprintf(v,"sources/%s/idl/%s",cat,pkg); ::alias(k,v);
sprintf(k,"interfaces/include/%s",pkg); sprintf(v,"sources/%s/%s",cat,pkg); ::alias(k,v);
sprintf(k,"examples/%s",pkg); sprintf(v,"sources/%s/examples/%s",cat,pkg); ::alias(k,v);
sprintf(k,"examples/%s",cat); sprintf(v,"sources/%s/examples",cat); ::alias(k,v);
}


int main(int argc, char* argv[]) {

stacks = new dictionary< stack<string> >;

sprintf(slidir,".");

thesession = new session();
session::declare("global",thesession);
session::initialize();
kit::initialize();
kit::_default = new kit();

nofio = 0;
int _argc = 0;



//strcpy(begin_dollar,"$");
//strcpy(end_dollar,"$");

reset_dollar();

declare("sigma","simulation");
declare("bpsim","simulation");
declare("grammar","grammar");
declare("ml","grammar");

declare("ami","web");
declare("pl","logic");

::alias("dv/pl","logic/dv/pl");
::alias("dv/bp","logic/dv/bp");
::alias("dv/sp","logic/dv/sp");
::alias("dv/ls","logic/dv/ls");

::alias("dlp","dlp/src");

argc--; argv++;

mode = 0;

strcpy(tagfile,"refs.html");
//strcpy(tagfile,"");
strcpy(endline,"");
strcpy(endbrace,"");

strcpy(online,ONLINE);
strcpy(documents,DOC);
strcpy(manuals,MAN);
strcpy(home,HIT);

//@ options

while ((argc > 0) && (argv[0][0] == '-')) {
	if ( argv[0][1] == 'N' ) { 
		strcpy(filename,&argv[0][2]);
		cerr << "NAME = " << filename << endl;
		//argv++; argc--;
		}
	if ( argv[0][1] == 'A' ) { 
		//strcpy(filename,&argv[0][2]);
		//cerr << "NAME = " << filename << endl;
		auto_header = 1;
		auto_footer = 1;
		//argv++; argc--;
		}
	if ( ( argv[0][1] == 'I' ) || ( argv[0][1] == 'R' ) ) { 
		strcpy(g_argv[_argc],argv[0]);
		//cerr << "Copied " << g_argv[_argc] << endl;
		_argc++;
		}
	if ( argv[0][1] == 'D' ) { 
		strcpy(documents,&argv[0][2]);
		cerr << "DOC = " << documents << endl;
		}
	if ( argv[0][1] == 'M' ) { 
		strcpy(manuals,&argv[0][2]);
		cerr << "MAN = " << manuals << endl;
		}
	if ( argv[0][1] == 'H' ) { 
		strcpy(home,&argv[0][2]);
		cerr << "HOME = " << home << endl;
		}
	if ( !strcmp( *argv, "-latex" ) ) { 
		latexmode = 1; 
		cerr << "LaTeX = on" << endl;
		}
	else if ( !strcmp( *argv, "-warnings" ) ) { 
		warnings = 1; 
		cerr << "warnings = on" << endl;
		}
	else if ( !strcmp( *argv, "-text" ) ) { 
		mode = textmode; 
		cerr << "MODE = text" << endl;
		}
	else if ( !strcmp( *argv, "-all" ) ) { 
		do_all = 1;
		cerr << "DO ALL" << endl;
		}
	else if ( !strcmp( *argv, "-slide" ) ) { 
		mode = slimode; 
		cerr << "MODE = slide" << endl;
		}
	else if ( !strcmp( *argv, "-bk" ) ) { 
		bk = 1;
		cerr << "BK = true" << endl;
		}
	else if ( !strcmp( *argv, "-ml" ) ) { 
		ml_mode = 1;
		cerr << "ML = true" << endl;
		//strcpy(ext,".ml");
		}
	else if ( !strcmp( *argv, "-echo" ) ) { 
		echo_mode = 1;
		cerr << "ECHO = true" << endl;
		//strcpy(ext,".ml");
		}
	argc--; argv++;
	}

//@ process

CLASS l;
_lexer = &l;
//cerr << "Which paths?" << endl;
l.paths(_argc,  g_argv);
cerr << "ARG: " << argc << endl;
char lname[1024];
sprintf(lname,"@lst-%s.js",filename);
lst.push(lname);
nofio=0;
if ( argc == 0 )  l(); 
else
	while (argc-- > 0) 
		l.soinc( *argv++ );
exit(0);
return 0;
}


//@ auxiliary

instr(char c, char *s)
{
while( (*s != '\0') && (c != *s )) s++;
if (*s=='\0') return(0); else return(1);
}
 
firn(char *s) {
int i=0;
//finn=0;
if (*s != '\n') return(0);
else {
while ((*s !='\0') && (*s == '\n') ) { i++; s++; } 
if (*s == '\n') return(i+1);
else return(i);
}
}

char *tol(char *s)
{
char *p = s;
while ( *p == ' ' && *p != '\0') p++;
while (*s != '\0') {
        if (isupper(*s)) *s = tolower(*s);
        s++;
        }
return(p);
}
//</>
