%{
//@auto=0
//@class vr-t

%}
%class lex
%n 1500
%a 14000
%p 14000
%e 3000
%o 20000
%{
#include <hush/shell.h>
#include <hush/grammar/lex.h>
#include <hush/string.h>
#include <hush/dictionary.h>

int once = 0;
static int whatnext = 0;
static char _filename[256];
static char* filename = _filename;

#ifndef SRC
#define SRC "."
#endif

//@ aliases

static dictionary<string>* _alias = 0;
void alias(const char* k, const char* v) {
	if (!_alias) _alias = new dictionary<string>(1);
	(*_alias)[k] = new string(v);
	}
const char* alias(const char* k) {
	if (!_alias || !k) return 0;
	string* p = (*_alias)[k];
	if (!p) return 0;
	else return (const char*) (char*) *p;
	}

#undef string

//@ ...

static char _slide[512];
static char _package[512];
static char _clazz[512];
static char _method[512];

static int in_string = 0;
static int in_slide = 0;
static int style = 0;
static int _auto = 0;

static char _language[64];
static char _lang = 0;

static int in_enum = 0;

static int in_scanner() {
	return _lang == 'u' || _lang=='l' || _lang=='y' || _lang == 'm' || _lang == 'p';
	}

static int in_script() {
	return _lang=='t' || _lang=='m' || _lang == 'p';
	}

const char* at(const char* x) {
char* p = (char*)x;
while (*p != '@') p++;
return (const char*) p;
}

const char* package() {
static char buf[1024];
	switch (_lang) {
	case 'c':
	case 'h':
		return ""; 
	case 'i':
		if (_package[0]=='\0') return "";
		sprintf(buf,"%s.",_package);
		return (const char*) buf; 
	case 'j':
		if (_package[0]=='\0') return "";
		sprintf(buf,"%s.",_package);
		return (const char*) buf; 
	default:
	   return "";
	}
}

const char* prefix() {
	switch (_lang) {
	case 'c':
		return "@c-"; 
	case 'l':
	case 'y':
	case 'h':
		return "@h-"; 
	case 'i':
		return "@idl-"; 
	case 'j':
		return "@java-"; 
	default:
	   return "";
	}
}

const char* suffix() {
	switch (_lang) {
	case 'c':
	case 'h':
		return "html"; 
	case 'i':
		return "html"; 
	case 'j':
		return "html"; 
	default:
	   return "html";
	}
}

static int instr(char c, char* s) {
	while ( *s && *s++ != c );
	return (int) *s;
	}

static char _symbols[] = " -\t\n.,;:{}%$!`^#/\\\"'!%&[]()~<>*";

static int symbol(char c) { return instr(c,_symbols); }


const char* clazz() {
char* p = _clazz;
while (symbol(*p)) p++;
return p;
}

const char* slide() {
static int _slideno = 0;
static char _slideref[4024];
_slideref[0] = '\0';
sprintf(_slideref,"%c%d_%s%s%s%s",_lang,_slideno++,filename,prefix(),clazz(),_slide);
char* b  = _slideref;
char* p = b;
while ( *p && symbol(*p) ) p++;
int i = 0;
while (*p && i <= 18 ) {
	int yes=0;
	while (symbol(*p) || *p == ' ' || *p == '-') {
		//yes = 1; *b = '-';
		p++;
		}
	*b = *p;
	b++; p++;
	if (yes) while (symbol(*p) || *p == ' ') p++;
	i++;
	}
*b='\0';
if (*(b-1) == '-') *(b-1) = '\0';
_slideref[18]='\0';
return (char*) _slideref;
}

/*
while (*p && i <= 18) {
	if (symbol(*p)) *p = '-';
	p++;
	}
return (const char*) b;
}
*/



const char* classref(const char* cl) {
static char _classref[1024];
char ll = _lang;
char* p = _classref;
if (_lang == 'c') {
	_lang = 'h'; 
	sprintf(p,"%s",cl);
	_lang = ll;
} else {
	sprintf(p,"<a href=%s%s%s.%s>%s</a>",prefix(),package(),clazz(),suffix(),cl);
	cerr << "Class Reference: " << _classref << endl;
	}
return p;
}


char firstchar(char*);

int pr=1;
int in_html = 0;
int in_comm = 0;
int in_x_comm = 0;
int in_cc = 1;
int in_ital = 0;
int in_meta = 0;
int in_display = 0;


void slice(char* id="") {
cout << "<v></display>" << endl;
cout << "<slice>" << endl;
cout << "<p align=right text=orange>" << id << "</p>" << endl;
cout << "<display size=0.8 comment=blue>"; // << endl;
cout << "</v>" << endl;
}

void begin_display(char* id = 0) {
char* x = id?id:"blue";
cout << "<display comment=blue>"; // comment=" << x << ">";
in_display = 1;
}

void end_display(int x=0) {
if (in_display || x) cout << "</display>" << endl;
in_display = 0;
}


void begin_slide(char* id = 0) {
}

void end_slide() {
  if (in_slide) {
     if (pr)
	   end_display();
	   cout << "</slide>";
	   //begin_display();
   in_slide = 0;
   }
}

static int in_program() {
	return !in_html && !in_comm && !in_x_comm;
	}

static char endline[BUFSIZ];

char progbegin[BUFSIZ];
char progend[BUFSIZ];

static char* emph() { return "<i>"; }
static char* end_emph() { return "</i>"; }
static char* keyw() { return ""; } // "<b>"; }
static char* end_keyw() { return ""; } // "</b>"; }
static char* phrase() { return "<b><fontcolor=orange>"; }
static char* end_phrase() { return "</font></b>"; }

static int nmeta() { return 3; }

//@ definitions
%}

vrtag  ("text"|"channel"|"object"|"vrml")
cfgtag  ("doc"|"sty"|"function"|"intro"|"constra"|"references"|"class"|"com")
htmltag ("h1"|"h2"|"h3"|"h4"|"pre"|"ul"|"dl"|"dd"|"dt"|"li"|"a")
moretags ("base"|"blockquote"|"b"|"i")
srctag  ("c"|"display"|"control"|"ignore"|"program"|"slide"|"include"|"select")
textag  ("document"|"text"|"raw"|"format"|"cite"|"ref"|"bibliography"|"references")
qtag ("query"|"param"|"applet"|"pipe"|"slide"|"include"|"script")
xxtagname  ({vrtag}|{cfgtag}|{htmltag}|{moretags}|{srctag}|{textag}|{qtag})
tagname  [a-z]+
displaytag (references)

escape ("\\\"")
ident [A-Za-z0-9]+
stringelem ({escape}|[^\"\n])
string (\"{stringelem}+\")


any ([^\>]|{string})
atts {any}*

bl ([ \t\n]*"\%\{"[ \t\n]*)
el ([ \t\n]*"\%\}"[ \t\n]*)
comment ("//"|"##"|"\%\%"|"!!")
comm ("///"|"###"|"\%\%\%")
commm ("////"|"####"|"\%\%\%\%")
str [\"\#\~\/A-Za-z0-9\.\-\+\=\}\]\{\[]+
xxatts [ \t]*{str}({str}|[ \t])*

slice {comment}"||"
meta {comment}"@"
oldname [A-Za-z0-9\_\-\:]+  
name ([^ \t\n]+|{string})

modul (xhush|xwidgets|xdot)

module ("module"|"package")

specifier ("object"|"class"|"interface")
qualifier ("active"|"public"|"template<class".*\n)

scanner (start|token|defaults|prec)



API ("WIDGETS_API"|"HUSH_API")

access ("public"|"private"|"protected")

%%@ rules

"xxx\"@vr-"[^\.]+".wrl" {
	yytext[yyleng-4] = '\0';
	yytext++;
	cout << "\"" << "<a href=" << yytext
	<< ".html>" << yytext << ".wrl</a>";
	}

"@vr-"[^\.]+".wrl" {
	yytext[yyleng-4] = '\0';
	cout << "<a href=" << yytext
	<< ".html>" << yytext << ".wrl</a>";
	}

"@pl-"[^\.]+".pl" {
	yytext[yyleng-3] = '\0';
	cout << "<a href=" << yytext
	<< ".html>" << yytext << ".pl</a>";
	}

[\%\#]"@include "[A-Za-z0-9\-\@\/]+(".vr"|".pl") {
	int n = 9;
	yytext[n] = '\0';
	char* p = yytext+(n+1);
	cout << yytext << " <a href=" << p
	<< ".html>" << p << "</a>";
}

[\%\#]"@include "[A-Za-z0-9\-\@]+(".txt") {
	int n = 9;
	yytext[n] = '\0';
	char* p = yytext+(n+1);
	cout << yytext << " <a href=" << p << ">"
	<< p << "</a>";
}


"xxxx<references>" { if (pr) cout << "<b>&lt;</b>references<b>&gt;</b>"; }
"xxxx</a>" { if (pr)
		if (in_scanner()) cout << "&lt;/a&gt;";
		cout << yytext;
		}
"xxx<a href=mailto: " { if (pr)
		if (in_scanner()) cout << "&lt;" << yytext+1; 
		else cout << yytext;
		}
"<!--" { if (pr) cout << "&lt;!--"; }
"-->" { if (pr) cout << "--&gt;"; }
[a-z]"/* "  { if (pr) cout << yytext; }
"/*."   { if (pr) cout << yytext; }
\""<""a href=" {
        if (pr) cout << "\"&lt;" << yytext+2;
	}
"<"("?"|"!")?("/")?{tagname}{atts}">" {
	if (1 || in_scanner()) {
        	yytext[yyleng-1] = '\0';
		if (pr) cout << "<in-cc><b>&lt;" << yytext+1 << "&gt;</b></in-cc>";
		}
	else if (pr) cout << yytext;
	}
"<"("?")?("/")?[a-z]+">" {
	if (1 || in_scanner()) {
        	yytext[yyleng-1] = '\0';
		if (pr) cout << "<b>&lt;" << yytext+1 << "&gt;</b></in-cc>";
		}
	else if (pr) cout << yytext;
	}


[^ \t\n]{specifier} { if (pr) cout << yytext; }
{specifier}[^ \t\n] { if (pr) cout << yytext; }

[^ \t\n]{module} { if (pr) cout << yytext; }
{module}[^ \t\n] { if (pr) cout << yytext; }

\|[ \t]*{specifier} {
	if (pr) cout << yytext; //
	}
"enum" {
	if (in_html && pr) cout << yytext;
	else {
	 in_enum = 1;
	 if (pr) cout << "<b>enum</b>";
	 }
	}

%%@ the meta section -- slides

{comment}"</>".*\n { end_display(1); end_slide(); }

{comment}": ".* { slice(yytext+4); }

^{comment}[\-\<\:].* {
	   end_display(whatnext);
	   if (yytext[2] != '-') cout << endl;
	   whatnext = 0;
	   if (pr) cout << (char*) ((yytext[2]=='-')?(yytext+3):(yytext+2));
	}

%%@ options

{meta}"style"[ \t]*"="[ \t]*[A-Za-z0-9\_\-\+] {
	style = yytext[yyleng-1]-'0';
	}

{meta}"noauto" { _auto=0; }

{meta}"auto"[ \t]*"="[ \t]*[A-Za-z0-9\_\-\+] {
	_auto = yytext[yyleng-1]-'0';
	}

{meta}"type="{name} {
	//_auto = 0;
	strcpy(_clazz,yytext+nmeta()+6);
	}

{meta}"class="{name} {
	//_auto = 0;
	strcpy(_clazz,yytext+nmeta()+6);
	}

{meta}"package="{name} {
	//_auto = 0;
	strcpy(_package,yytext+nmeta()+8);
	}

{meta}"module="{name} {
	//_auto = 0;
	strcpy(_package,yytext+nmeta()+7);
	}

%%@ classes

{meta}"class "{name} {
	_auto = 0;
	strcpy(_clazz,yytext+nmeta()+6);
	if (in_slide) {
		if (pr) cout << "</display>\n<hr>\n</slide>";
		in_slide = 0;
		}
	in_slide = 1;
	style = 1;
        if (pr) {
	   char name[1024];
	   sprintf(name,"%s",_clazz);
	   cout << "<a name=class-" << name << ">";
	   cout << "</display>";
	   // if (style) cout << "<h4 align=right text=red>" << classref(name) << "</h4>";
	   cout << "<slide xid=" << name << " xcap=\"" << name << "\">";
	   if (style) cout << "<h4 align=right text=red>" << classref(name) << "</h4><hr>";
	   cout << "<display>";
	}
   }

%%@ slide, which is usually auto or implicit

{meta}"slide ".* {
	strcpy(_slide,yytext+nmeta()+6);
	end_slide();
	in_slide = 1;
	style = 1;
        if (pr) {
	   //if (style) cout << "<h4 align=right text=red>" << _slide << "</h4>";
	   cout << "<slide id=" << slide() << " xcap=\"" << _slide << "\">";
	   if (style) cout << "<h4 align=right text=red>" << _slide << "</h4>";
	begin_display();
	}
   }

%%@ implicit slide 

{meta}" ".* {
	strcpy(_slide,yytext+nmeta()+1);
	cerr << "Meta " << nmeta() << " " << _slide << endl;
	end_display(whatnext);
	end_slide();
	//if (in_slide) {
		//if (pr) cout << "</display><hr></slide>";
		//in_slide = 0;
	//	}
	in_slide = 1;
	style = 1;
        if (pr) {
	   char id[1024]; strcpy(id,slide());
	   //cout << "</display>";
	   cout << "<slide color=white a=0 id=\"" << id << "\" xcap=\"" << _slide << "\">";
	   cout << "<slice>" << endl;
	   if (style) cout << "<p align=right>" <<
	   "<a href=@slide-" << id << ".html><font color=red><text text=red>" <<
	   _slide << "</text></font></a></p>" << endl;
	begin_display("green");
	}
   }

{slice}.* { slice(yytext+4); }

{meta}[\*\+\#\-]" ".* {
	strcpy(_slide,yytext+nmeta()-1);
	cerr << "Meta " << nmeta() << " " << _slide << endl;
	end_display(whatnext);
	whatnext = 0;
	cout << endl;
	end_slide();
	style = 1;
        if (pr) {
	   cout << _slide << endl;
	}
   }

%%@ access

{access}":"[ \t]*"//@".* {
	char spec[12]; char* sp = spec;
	char* p = yytext;
	while (*p != ':') *sp++ = *p++;
	*sp++ = ':'; *sp = '\0';
	while (*p && *p++ != '@'); //nop 
	p++; // at @
	strcpy(_slide,p);
	if (in_slide) {
		if (pr) cout << "</display><hr>\n</slide>";
		in_slide = 0;
		}
	in_slide = 1;
	style = 1;
        if (pr) {
	   cout << "</display>";
	   //if (style) cout << "<h4 align=right text=red>" << _slide << "</h4>";
	   cout << "<slide xid=" << slide() << " xcap=\"" << _slide << "\">";
	   if (style) cout << "<h4 align=right text=red>" << _slide << "</h4><hr>";
	   cout << "<display>  " << "<b>" << spec << "</b>";
	}
   }

%%@ methods

{meta}"method "{name} {
	strcpy(_method,yytext+nmeta()+7);
	if (in_slide) {
		if (pr) cout << "</display><hr></slide>";
		in_slide = 0;
		}
	in_slide = 1;
        if (pr) {
	   char name[1024];
	   sprintf(name,"%s::%s",_clazz,_method);
	   cout << "</display>";// << endl;
	   cout << "<a name=" << _method << ">";
	   //if (style) cout << "<h4 align=right text=red>" << classref(_clazz) << "::" <<  _method << "</h4>";
	   cout << "<slide xid=" << name << " xcap=\"" << name << "\">";
	   if (style) cout << "<h4 align=right text=red>" << classref(_clazz) << "::" <<  _method << "</h4><hr>";
	   cout << "<display>";
	}
 }

%%@ class ending
%%% this shows how many people put the erroneously
%%% an addition semi-colon after a bracket.
%%% Just to make sure?

"};" {
     if (pr && !in_html) cout << "<b>};</b>";
     else if(pr) cout << "};";
     if (in_enum) {
       in_enum = 0; 
     } else if (in_slide && _auto && !in_html && !in_scanner()) {
	if (pr) cout << "</display><hr></slide><display>";
	}
     }

%%@ links to imported packages

"import"[ \t]*"hush"[^;]+/";" {
	char buf[1024];
	int star = 0;
	char* p = (char*) &yytext[7];
	while( *p == ' ' || *p == '\t' ) p++;
	strcpy(buf,p);
	p = buf;
	while( *p != '\0' && *p != '*' ) {
			if (*p == '.' ) *p = '/';
			p++;
		}
	if ( *p == '*' ) { star = 1; *p = '\0'; }
	char path[1024];
	char* suff = (char*)(star?"index.html":".java.html");
	sprintf(path,"%s/java/%s%s",SRC,buf,suff);
	if (pr) cout << "<a href=" << path << ">" << yytext << "</a>"; 
  }

%%@ packages and modules

"package "[A-Za-z0-9\_\-\:\.\;]+ {
     strcpy(_package,yytext+8);
     _package[strlen(_package)-1]='\0';
     cerr << "Package " << _package << endl;
     if (pr && in_html) cout << yytext;
     else if (pr) cout << "<b>" << yytext << "</b>";
     }

"module "{name} {
     strcpy(_package,yytext+7);
     //_package[strlen(_package)-1]='\0';
     cerr << "Module " << _package << endl;
     if (pr && in_html) cout << yytext;
     else if (pr) cout << "<b>" << yytext << "</b>";
     }

%%@ declarative specifiers

{specifier}[ \t]+{name}(";"|"*") {
	if (pr) cout << yytext; 
	}

%%@ the real specifiers, may be qualified

{qualifier}*[ \%\t]*{specifier}[ \t]+{API}*[ \t]*{name} {
     int n = yyleng-2;
     char* p = yytext+(yyleng-2);
     while (*p != ' ') {
	p--; n--;
	}
     if (!_auto || in_comm || in_html || in_script() ) {
	if (pr) cout << yytext;
     } else {
        strcpy(_clazz,yytext+n);
        strcpy(_slide,_clazz);
	if (in_slide) {
		if (pr) cout << "</display><hr></slide>";
		in_slide = 0;
		}
	in_slide = 1;
	style = 1;
        if (pr) {
	   cout << "<a name=slide-" << _clazz << ">";
	   cout << "</display>";
	   //if (style) cout << "<h4 align=right text=red>" << classref(_clazz) << "</h4>";
	   cout << "<slide xid=" << slide() << " xcap=\"" << _clazz << "\">";
	   if (style) cout << "<h4 align=right text=red>" << classref(_clazz) << "</h4><hr>";
	   cout << "<display>  ";
	   cout << "<b>" << yytext << "</b>";
	}
   }
 }

%%@ supoorting links to included files 

"<"[a-z0-9A-Z\-]+"."[hdlicpx]+">" { 
        char buf[512];
        strcpy(buf,yytext);
        buf[yyleng-1] = '\0';
        if (pr) cout << "&lt;" << buf+1 << "&gt;";
        }

\"[a-z0-9A-Z\-]+"."[hdlicpx]+\" { 
        char buf[512];
        strcpy(buf,yytext);
        buf[yyleng-1] = '\0';
        if (pr) cout << "&lt;" << buf+1 << "&gt;";
	}

%%@ symbols

"{"|"}"|"};"|";" { if (pr) cout << keyw() << yytext << end_keyw(); }
[A-Za-z\-]+" =" { if (pr) cout << keyw() << yytext << end_keyw(); }

"\$"[0-9A-Za-z\-\$]+ { if (pr) cout << keyw() << yytext << end_keyw(); }
"\$("[A-Za-z0-9\-\$]+")" { yytext[yyleng-1]='\0'; if (pr) cout << keyw() << yytext+2 << end_keyw(); }

^"include" {  if (pr) cout << keyw() << yytext << end_keyw(); }
"# DO NOT DELETE" { 
	if (_lang != 'l')
		pr = 0;
	else if (pr) cout << yytext;
	}

^"#include" { 
  if (in_cc) {
	if (pr) cout << keyw() << yytext+1 << end_keyw(); 
  } else 
	if (pr) cout << yytext; // << endl;
}


^"#define" { if (pr) cout << keyw() << yytext+1 << end_keyw(); }
^"#ifndef" { if (pr) cout << keyw() << yytext+1 << end_keyw(); }
^"#endif" { if (pr) cout << keyw() << yytext+1 << end_keyw(); }
^"#else" { if (pr) cout << keyw() << yytext+1 << end_keyw(); }
^"#if" { if (pr) cout << keyw() << yytext+1 << end_keyw(); }

"xxDEF" { if (pr) cout << phrase() << yytext << end_phrase(); }
"xxUSE" { if (pr) cout << phrase() << yytext << end_phrase(); }
"xxROUTE" { if (pr) cout << phrase() << yytext << end_phrase(); }
"xxTO" { if (pr) cout << phrase() << yytext << end_phrase(); }

{modul}"::"[a-z]+ { if (pr) cout << keyw() << yytext << end_keyw(); }

"[.]"|"[index]" { if (in_html) {
	if (pr) cout << "<a href=\"index.html\">[.]</a>";
       } else if (pr) cout << yytext;
       }
"[Up]" { if (in_html) {
	if (pr) cout << "<a href=\"../index.html\">[Up]</a>";
	} else if (pr) cout << yytext;
       }

%%@ keywords -- sholdn't this be a definition

"extends"|"final"|"implements"|"package"|"import"|"interface"|"class"|"public"|"protected"|"private"|"static"|"virtual"|"typedef"|"const"|"extern"|"operator"|"operator()"|"operator()()" {
	if (0 && pr && in_cc && !in_html && !in_ital) cout << keyw() << yytext << end_keyw();
	else if (pr) cout << yytext;
	}

%%@ hrefs
"<a href="[^>]+">" { if (pr) cout << yytext; }
"<" { if(pr) cout << "&lt;"; }
">" { if(pr) cout << "&gt;"; }
"&" { if(pr) cout << "&amp;"; }

%%@ special comments

^[ \t]*{comm}{comm}.* {
   if (pr) cout << "</display>";
   if (in_slide) {
	if (pr) cout << "<hr></slide>";
	}
   if (pr) {
       cout << "<h4>" << yytext+4 << "</h4><display>";
   }
		}
"/*"[ \t]* { 
	if (_lang != 'l') {
		in_comm = 1;
		in_x_comm = 1; 
		if (pr) cout << "<i>" << yytext;
	} else if (pr) cout << yytext;
	}

%%@ javadoc style comments

"/**"[ \t]* { 
	if (_lang != 'l') {
	in_html = 1; 
	cout << progend; 
	cout << "<" << "blockquote" << ">" << endl;
	} else if (pr) cout << yytext;
	}
^[ \t]*"*"  { if (!in_html &&pr) cout << yytext; }
[ \t]*"*/" { 
		if (in_html) {
			in_html = 0;
			cout << "</" << "blockquote" << ">";
			cout << progbegin;
			cout << "\n";
		} else if (in_x_comm) {
			in_x_comm = 0;
			in_comm = 0;
			cout << yytext << "</i>";
		} else cout << yytext;
	}
[ \t]*"@see ".* { cout << "<br>\t<i> see</i>: " << at(yytext)+4 << endl; }
[ \t]*"@version ".* { cout << "<br>\t<i> version</i>: " << at(yytext)+8 << endl; }
[ \t]*"@author ".* { cout << "<br>\t<i> author</i>: " << at(yytext)+7 << endl; }
[ \t]*"@parm ".* { cout << "<br>\t<i> parameter</i>: " << at(yytext)+5 << endl; }
[ \t]*"@param ".* { cout << "<br>\t<i> parameter</i>: " << at(yytext)+6 << endl; }
[ \t]*"@returns ".* { cout << "<br>\t<i> returns</i>: " << at(yytext)+8 << endl; }
[ \t]*"@return ".* { cout << "<br>\t<i> returns</i>: " << at(yytext)+7 << endl; }
[ \t]*"@raises ".* { cout << "<br>\t<i> raises</i>: " << at(yytext)+7 << endl; }
[ \t]*"@raise ".* { cout << "<br>\t<i> raises</i>: " << at(yytext)+6 << endl; }

%%@ for lex and yacc

^"%{"  { if (pr && !in_scanner()) cout << yytext; } 
^"%}"  { if (pr && !in_scanner()) cout << yytext; } 
^"%"[A-Za-z]" "[0-9]* {
	if (pr && in_scanner()) cout << "<b>" << yytext << "</b>"; 
	}
^"%"{scanner} {
	if (pr && in_scanner()) cout << "<b>" << yytext << "</b>"; 
	}

%%@ for old times sake (still lex and yacc)

^{bl}{comm} { 
  if (!in_html) { 
  	in_comm = 1; in_html = 1; cout << progend; 
	if (in_slide) {
		if (pr) cout << "<hr></slide>" << endl;
		in_slide = 0;
		}
  	if (!once && firstchar(yytext)=='#') {
		in_cc = 0;
		strcpy(progbegin,"<display>");
		strcpy(progend,"</display>");
		once++;
		}
	}
  }

^{bl}[ \t\n]*{comm}[ \t\n]*"<html>"[ \t\n]* { 
	in_comm = 1; in_html = 1; cout << progend; 
	if (!once && firstchar(yytext)=='#') {
		in_cc = 0;
		strcpy(progbegin,"<display>");
		strcpy(progend,"</display>");
		once++;
		}
	}

%%@ regular comments - at the beginning of a line

^[ \t]*{comm} { 
  if (!in_html) { 
  	in_comm = 1; in_html = 1; cout << progend; 
	if (in_slide) {
		if (pr) cout << "<hr></slide>" << endl;
		in_slide = 0;
		}
  	if (!once && firstchar(yytext)=='#') {
		in_cc = 0;
		strcpy(progbegin,"<display>");
		strcpy(progend,"</display>");
		once++;
		}
	}
  }

%%@ again, for old times sake (comments + meta)

^[ \t]*{comm}[ \t\n]*"<html>"[ \t\n]* { 
	in_comm = 1; in_html = 1; cout << progend; 
	if (!once && firstchar(yytext)=='#') {
		in_cc = 0;
		strcpy(progbegin,"<display>");
		strcpy(progend,"</display>");
		once++;
		}
	}
^[ \t\n]*{comm}[ \t\n]*"</html>"[ \t\n]*{el} {
    in_html = 0; cout << progbegin; cout << "\n"; }
^{bl}[ \t\n]*{comm}[ \t\n]*"<ignore>"[ \t\n]* { pr = 0; }
^[ \t\n]*{comm}[ \t\n]*"</ignore>"[ \t\n]*{el} { pr = 1; }
^{bl}[ \t\n]*{comm}[ \t\n]*"</html>"[ \t\n]*"<ignore>"[ \t\n]* { pr = 0; }
^[ \t\n]*{comm}[ \t\n]*"<ignore>"[ \t\n]*"</html>"[ \t\n]*{el} { pr = 0; }
^{bl}[ \t\n]*{comm}[ \t\n]*"<html>"[ \t\n]*"</ignore>"[ \t\n]* { pr = 1; }
^[ \t\n]*{comm}[ \t\n]*"</ignore>"[ \t\n]*"<html>"[ \t\n]*{el} { pr = 1; }
"</html>"[ \t\n]*\n{el} { if (in_comm) { in_comm = 0; in_html = 0;
	cout << progbegin; cout << "\n"; } }
{comm} { 
	if (!in_html) { in_ital = 1; if (pr) cout << " <i>" << yytext; } 
	else if (pr) cout << yytext;
	}
{commm} { 
	if (!in_html) { in_ital = 1; if (pr) cout << "  <i>" << yytext; } 
	else if (pr) cout << yytext;
	}
{comment} { in_comm = 1; if (pr) cout << yytext; }

%%@ newlines are tricky

\n {    if (!in_x_comm) in_comm = 0;
	if (in_ital && pr) cout << "</i>";
	in_ital = 0;
	if (pr) cout << endl; 
	if (pr && !in_html) cout << ""; }
\n\n+ {
	if (in_ital && pr) cout << "</i>";
	in_ital = 0;
	if (!in_x_comm) in_comm = 0;
	if (pr) cout << "\n" << endl; 
	//if (pr && !in_html) cout << "  ";
	if (in_html) {
	   in_html = 0;
	   cout << progbegin; cout << "\n";
	   }
	}
\t { if (pr) cout << "        "; }
. { if (pr) cout << yytext[0]; }

%%
//@ main
int main(int argc, char* argv[]) {
char buf[1024];
whatnext = 1;
strcpy(filename,"temp");

sprintf(buf,"%s/%s/%s",SRC,"java","hush"); alias("hush",buf);

strcpy(_language,"unknown");
_auto = 0;
_lang = 'u';
argc--; argv++;
while ((argc > 0) && ((argv[0][0] == '-')||(argv[0][0] == '+'))) {
	if (argv[0][0] == '+') _auto = 1;
	else if ( argv[0][1] == 'N' ) { 
		strcpy(filename,&argv[0][2]);
		cerr << "NAME = " << filename << endl;
		//argv++; argc--;
		}
	else {
		_lang = argv[0][1];
		strcpy(_language,&argv[0][1]);
		}
	argv++; argc--;
	}
int len = strlen(filename);
if (len > 12) filename = filename + (len-12); 
cerr << "vr-t.flt: language = " << _language << endl;
if (_lang == 'c') _auto = 1;
strcpy(_package,"");
strcpy(_clazz,"");
strcpy(_method,"");
strcpy(progbegin,"<display>");
strcpy(progend,"</display>");
strcpy(endline,"");
//cout << "<document>";
//cout << "<body bgcolor=\"#FFFFFF\" text=\"#000000\">";
//cout << "\n" << endl;
cout << progbegin << endl;
lex l; l();
//if (!in_html) cout << progend; 
//cout << "</document>" << endl;
return 0;
}

char firstchar(char* s) {
while ( *s != 0 && ( *s == ' ' || *s == '\t' || *s == '\n' ) ) s++;
return *s;
}
//</>
