#!/bin/sh
##@ slush

cc=
hh=
prefix=
location=
depth=""
tell=silent
self=@BIN/slush
html="@BIN/ht-ml -"
dest=@SLUSH
type=nil;
ext=zero;
level=source;
base=@SRC; 
mode=check;
root=@SLUSH
inc=$root/include
src=$root/source
man=$root/man
idl=$root/idl
doc=$root/doc
lib=$root/library
util=$root/util
make=make
trash=/tmp
ext=core

##@ options

while [ $# -ge 1 ]; do case $1 in
	-|-r) flags=$1;;
	-l) level=$2; shift;;
	-b) base=$2; shift;;
	-p) prefix=$2; shift;;
	-prefix) cc=true; prefix=$2; shift;;
	-location) hh=true; location=$2; shift;;
	-m|-mode) mode=$2; shift;;
	-v) tell=verbose;;
	-replace) mode=replace; root=@DOC; doc=@DOC;;
	-doc)  mode=doc;;
	-h) type=h; ext=.h;;
	-idl) type=idl; ext=.idl;;
	-dest) dest=$2; shift;;
	-depth) depth=$2; shift;;
	-flush) mode=flush;;
	-slush) mode=slush;;
	-find) mode=find; ext="$2"; shift;;
	-ext) ext="$2"; shift;;
	-check) mode=check;;
	-path) path=$2; shift;;
	-root) root=$2; shift;;
	-src) src=$2; shift;;
	-inc) inc=$2; shift;;
	-man) man=$2; shift;;
	-lib) lib=$2; shift;;
	-util) util=$2; shift;;
	-trash) trash=$2; shift;;
	-make) make=$2; shift;;
	-a) only=true;; 
	-hh)  hh=true;;
	-cc) cc=true;;
	*)  break;;
	esac;
	shift;
done;

##@ check args

args="$*"
if [ $mode = "slush" ]; then
	if [ ! -d $root ]; then echo "Creating $root"; mkdir $root; fi;
	if [ ! -d $inc ]; then echo "Creating $inc"; mkdir $inc; fi;
	if [ ! -d $src ]; then echo "Creating $src"; mkdir $src; fi;
fi;
if [ $mode = "doc" ]; then
	if [ ! -d $dest ]; then echo "Creating $dest"; mkdir $dest; fi;
fi;

##@ process cc hh

if [ "$cc" = "true" ]; then
#echo "Slushing for cc with prefix $prefix"
L=`find-c`;  ## takes .c
X=""
for f in $L; do
	x="$src/$prefix`@BIN/slushit $f`";
	echo "$f -> ${x}"
	cp $f ${x}pp;
	done;
exit;
fi;

if [ "$hh" = "true" ]; then
echo "Slushing for hh with location $location"
L=`find-hh`;  ## takes .h and *.i
X=""
for f in $L; do
	x="$inc/$location$f";
	d=`@BIN/dirof $x`;
	if [ ! -d $d ]; then mkdir $d; fi;
	echo "$f -> $x";
	cp $f $x;
	done;
exit;
fi;
	
##@ process args
	
for i in $args; do
T=`@BIN/suffix $i`;
B=`basename $i $T`;
echo "slushing $mode with  $i    ---------   $B $T";
case $i in 
	core) echo "WARNING core";
	      rm core;;
esac;

##@ flush

if [ $mode = flush ]; then
  	if [ -d $i ]; then
      		case $i in
	 	old|tmp|BAK)
			echo "Removing $i";;
		@*);;
		*) (cd $i; $self -mode $mode "*");;
      		esac;
	else
	case $T in
          core|.bak|.CKP)
		echo "Removing $i to $trash";;
          .html|.t|.tex|.dvi)
		echo "Removing $i to $trash";;
	  .gz|.Z|.uu)
		echo "Warning: archive $i" ;;
	  *) ;;
	  esac;
        fi; ## -d

##@ doc

elif [ $mode = doc ]; then
  	if [ -d $i ]; then
		echo "Testing `pwd`/$i"
      		case $i in
	 	old|tmp|BAK)
			echo "Removing $i";;
		@*|store|keep|local)
		echo "Omitting $i";;
		*)
		  echo "Descending in $i";
		  if  [ ! -d $dest/$i ]; then mkdir $dest/$i; fi;
		  (cd $i;
		  $self -doc -dest $dest/$i *);
		  echo "Ascending from $i";
		  ;;
      		esac;
	else
	case $T in
          core|.bak|.CKP)
		echo "Removing $i to $trash";;
	  README|.txt)
		if [ $type = idl ]; then
			echo "In $i `pwd`";
			$html -txt -title $i > $i.html;
			cp $i.html $dest;
		        echo "Document $dest/$i.html";
		fi;;
	  index)
		if [ $type = idl ]; then
			echo "In $i for `pwd`";
			$html -index -title $i > $i.html;
			cp $i.html $dest;
		        echo "Document $dest/$i.html";
		fi;;
          .h|.idl)
		echo "Applying $html -cc `pwd`/$i"
                $html -cc $i > $i.html;
		cp $i.html $dest;
		echo "Document $dest/$i.html";;
	  *) ;;
	  esac;
        fi; ## -d

##@ replace

elif [ $mode = replace ]; then
  	if [ -d $i ]; then
      		case $i in
	 	old|tmp|BAK)
			echo "Removing $i";;
		@*) ;;
		*)
		  echo "Descending in $i";
		  (cd $i; $self -mode $mode -depth ../$depth *);
		  echo "Ascending from $i";
		  ;;
      		esac;
	else
	case $i in
          @dummy*|@stamp*|core|*.bak|*.CKP)
		echo "Removing $i to $trash";;
          *.html)
		mv $i $i.bak;
		echo "Changing $i with $depth";
		cat $i.bak | \
		  sed \
		     -e "s?@HUSH/include/??g"  \
		     -e "s?file:@DOC/?$depth?g"  \
		     -e "s:@DOC/:$depth:g"  \
			> $i;;
	  .gz|.Z|.uu)
		echo "Warning: archive $i" ;;
	  *) ;;
	  esac;
        fi; ## -d

##@ slush

elif [ $mode = slush ]; then
    if [ -d $i ]; then
      case $i in
       @*|store|old|tmp|BAK|keep|test|todo|lib)
      		echo "Omitting $i";
		;;
       cfg|library|man|include|idl|doc|examples)
                echo "WATCH: $i ";
		break;;
       *)
      
	 ##2
       case $flags in
          *)
	     if [ $tell = verbose ]; then
		echo Entering directory `pwd`/$i; fi;
	     ( 
	     echo Entering directory `pwd`/$i;
	     if [ ! -d $inc/$i ]; then
		echo making $inc/$i;
		mkdir $inc/$i;
		fi;
	     cd $i;
	     $self $flags -m $mode -p $prefix$i- -path $path/$i  \
		 -root $root -src $src -inc $inc/$i *)
	     if [ $tell = verbose ]; then
		echo Leaving directory `pwd`/$i; fi;
	     #break;;
	     ;;
       esac ##2
	#break;;
	;;
       esac ##1
     else ## file
	case $T in
    	.c|.cc|.cpp)
		 echo Source $src/$prefix$B.cpp;
		 cp $i $src/$prefix$B.cpp;
		 #break;;
		 ;;
    	.h|.i) 
                 echo Include $inc/$i;
		 cp $i $inc/$i;
		 #break;;
		 ;;
    	*) ;;
    	esac;
     fi;

##@ check

else ## if [ $mode = check ]; then
  if [ -d $i ]; then
	     (cd $i; $self $flags \
			-mode $mode \
		         -p $prefix \
			 -path $path \
			 -ext $ext \
		         -root $root \
			 -src $src \
			 -inc $inc/$i `ls`)
   else 
	if [ $mode = find ]; then
		case $T in
		     $ext) echo "Found $i - `pwd`";;
		     *);;
		esac;
		case $i in
		     $ext) echo "Found $i - `pwd`";;
		     *);;
		esac;
	else
		echo "File $i $B $T";
	fi

   fi;
fi; # fi; ## fi;
done; ## $i

##</>
